/**********************************************************************************************************************
*  Copyright (C), 2022-2026, SMARTLOGIC TECHNOLOGY LTD.
*  File Name: cpri_driver.h
*  Create Date: 23/12/25
*  Description: CPRI Driver Module Header File
*  Change History: 
*      <author>      <time>      <version>   <desc>
*    1. cheng.wan     23/12/25      1.0       refactor this module
**********************************************************************************************************************/

#ifndef __CPRI_DRIVE_H__
#define __CPRI_DRIVE_H__

#include "typedef.h"


#define  CPRI_RRU_MSG_LEN          (164)
#define  SWAP32(A)                 ((((uint32_t)(A) & 0xff000000) >> 24) | (((uint32_t)(A) & 0x00ff0000) >>  8) | \
                                    (((uint32_t)(A) & 0x0000ff00) <<  8) | (((uint32_t)(A) & 0x000000ff) << 24))

#define  CK_NORM_CTRL_WORD_TX_HFN  (0x78)
#define  SL_TEST_CTRL_WORD_TX_HFN  (0x78)

#define  OP_PT                     (0) 
#define  OP_SRL                    (1)
#define  OP_SLL                    (2)
#define  OP_CLR                    (3)



typedef struct CpriRruMsg
{
    uint32_t msg_addr;
	uint32_t msg_len;
}CpriRruMsg_t;

typedef enum VendorType
{
    SL_TEST = 0,        
    CK_NORM = 1,
}VendorType_e;



void     cpri_map_start(void);
void     cpri_map_stop(void);
uint32_t cpri_aux_get_tx_hfn();
uint32_t cpri_aux_get_rx_hfn();
void     cpri_aux_rx_init(uint32_t buffer0, uint32_t buffer1, uint32_t length0, uint32_t length1);
uint32_t cpri_headerram_rx(uint32_t bf_in_hf, uint32_t no_in_ctrlword);
void     cpri_headerram_tx_init();
int32_t  cpri_set_rru_msg(CpriRruMsg_t rru_msg);
void     cpri_ctrlword_insert(VendorType_e vendor);
void     cpri_axc_map_cfg(uint32_t option, uint32_t mode);


#endif





