/**********************************************************************************************************************
*  Copyright (C), 2022-2026, SMARTLOGIC TECHNOLOGY LTD.
*  File Name: cpri_driver.s.c
*  Create Date: 23/12/25
*  Description: CPRI Driver Module Source File
*  Change History: 
*      <author>      <time>      <version>   <desc>
*    1. cheng.wan     23/12/25      1.0       refactor this module
**********************************************************************************************************************/
/* -------------------------------------------------- <INC FILE> --------------------------------------------------- */
#include "ucp_utility.h"
#include "ucp_printf.h"
#include "ucp_js_ctrl.h"
#include "ucp_pma.h"
#include "phy_para.h"
#include "cpri_driver.h"

#ifdef TEST_ENABLE
#include "cpri_test.h"
#endif


/* --------------------------------------------------- <DEF&DEC> --------------------------------------------------- */
 


/* -------------------------------------------------- <FUNC DEF> --------------------------------------------------- */
/**********************************************************************************************************************
*  Function: cpri_axc_map_cfg
*  Description: 
*  Input: 
*         <name>                     <type>                      <desc>
*          option                    uint32_t                     cpri line speed option
*          mode                      uint32_t                     cpri axc mapping mode
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
     uint32_t sb_num_per_id[24] = {0};
DDR0 uint32_t sb_num_cfg_table[4][24] = 
{
    {1, 8, 8, 8, 8, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, // OTIC_MAP_FIGURE12
    {1, 1, 8, 8, 8, 8, 2, 2, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, // OTIC_MAP_FIGURE10
    {1, 1, 8, 8, 8, 8, 8, 8, 8, 8, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, // OTIC_MAP_FIGURE16
    {1, 8, 8, 8, 8, 8, 8, 8, 8, 2, 2, 2, 2, 2, 2, 2, 2, 1, 0, 0, 0, 0, 0, 0}, // OTIC_MAP_8NR_8LTE
};

void cpri_axc_map_cfg(uint32_t option, uint32_t mode)
{
    uint64_t reg_idx;
    uint32_t iq_buff_addr;


    if(option == 8)
    {
        if(mode == OTIC_MAP_FIGURE12)
        {
            memcpy_ucp(sb_num_per_id, sb_num_cfg_table[0], 96);

        }
        else if(mode == OTIC_MAP_FIGURE10)
        {
            memcpy_ucp(sb_num_per_id, sb_num_cfg_table[1], 96);
        }	
    }
    else if(option == 10)
    {
        if(mode == OTIC_MAP_FIGURE16)
        {
            memcpy_ucp(sb_num_per_id, sb_num_cfg_table[2], 96);
            
        }else if(mode == OTIC_MAP_8NR_8LTE)
        {
            memcpy_ucp(sb_num_per_id, sb_num_cfg_table[3], 96);         
        }
    }
    else
    {	
        memcpy_ucp(sb_num_per_id, sb_num_cfg_table[2], 96);
    }


    /* axc tx cfg */
    for(reg_idx = 0; reg_idx < 256; reg_idx++)
    {
        iq_buff_addr = reg_idx;

        if(reg_idx < sb_num_per_id[0])
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x1000 + reg_idx*4)) = (sb_num_per_id[0] << 16) + (iq_buff_addr << 8) + BIT7 + 0; 
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x1000 + reg_idx*4)) = (sb_num_per_id[1] << 16) + (iq_buff_addr << 8) + BIT7 + 1; 
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x1000 + reg_idx*4)) = (sb_num_per_id[2] << 16) + (iq_buff_addr << 8) + BIT7 + 2; 
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2] + sb_num_per_id[3]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x1000 + reg_idx*4)) = (sb_num_per_id[3] << 16) + (iq_buff_addr << 8) + BIT7 + 3; 
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2] + sb_num_per_id[3] + sb_num_per_id[4]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x1000 + reg_idx*4)) = (sb_num_per_id[4] << 16) + (iq_buff_addr << 8) + BIT7 + 4; 
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2] + sb_num_per_id[3] + sb_num_per_id[4] + sb_num_per_id[5]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x1000 + reg_idx*4)) = (sb_num_per_id[5] << 16) + (iq_buff_addr << 8) + BIT7 + 5; 
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2] + sb_num_per_id[3] + sb_num_per_id[4] + sb_num_per_id[5] + sb_num_per_id[6]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x1000 + reg_idx*4)) = (sb_num_per_id[6] << 16) + (iq_buff_addr << 8) + BIT7 + 6; 
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2] + sb_num_per_id[3] + sb_num_per_id[4] + sb_num_per_id[5] + sb_num_per_id[6] + sb_num_per_id[7]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x1000 + reg_idx*4)) = (sb_num_per_id[7] << 16) + (iq_buff_addr << 8) + BIT7 + 7; 
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2] + sb_num_per_id[3] + sb_num_per_id[4] + sb_num_per_id[5] + sb_num_per_id[6] + sb_num_per_id[7] + \
                          sb_num_per_id[8]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x1000 + reg_idx*4)) = (sb_num_per_id[8] << 16) + (iq_buff_addr << 8) + BIT7 + 8; 
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2] + sb_num_per_id[3] + sb_num_per_id[4] + sb_num_per_id[5] + sb_num_per_id[6] + sb_num_per_id[7] + \
                           sb_num_per_id[8] + sb_num_per_id[9]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x1000 + reg_idx*4)) = (sb_num_per_id[9] << 16) + (iq_buff_addr << 8) + BIT7 + 9; 
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2] + sb_num_per_id[3] + sb_num_per_id[4] + sb_num_per_id[5] + sb_num_per_id[6] + sb_num_per_id[7] + \
                           sb_num_per_id[8] + sb_num_per_id[9] + sb_num_per_id[10]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x1000 + reg_idx*4)) = (sb_num_per_id[10] << 16) + (iq_buff_addr << 8) + BIT7 + 10; 
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2]  + sb_num_per_id[3] + sb_num_per_id[4] + sb_num_per_id[5] + sb_num_per_id[6] + sb_num_per_id[7] + \
                           sb_num_per_id[8] + sb_num_per_id[9] + sb_num_per_id[10] + sb_num_per_id[11]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x1000 + reg_idx*4)) = (sb_num_per_id[11] << 16) + (iq_buff_addr << 8) + BIT7 + 11; 
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2]  + sb_num_per_id[3]  + sb_num_per_id[4] + sb_num_per_id[5] + sb_num_per_id[6] + sb_num_per_id[7] + \
                           sb_num_per_id[8] + sb_num_per_id[9] + sb_num_per_id[10] + sb_num_per_id[11] + sb_num_per_id[12]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x1000 + reg_idx*4)) = (sb_num_per_id[12] << 16) + (iq_buff_addr << 8) + BIT7 + 12; 
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2]  + sb_num_per_id[3]  + sb_num_per_id[4]  + sb_num_per_id[5] + sb_num_per_id[6] + sb_num_per_id[7] + \
                           sb_num_per_id[8] + sb_num_per_id[9] + sb_num_per_id[10] + sb_num_per_id[11] + sb_num_per_id[12] + sb_num_per_id[13]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x1000 + reg_idx*4)) = (sb_num_per_id[13] << 16) + (iq_buff_addr << 8) + BIT7 + 13; 
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2]  + sb_num_per_id[3]  + sb_num_per_id[4]  + sb_num_per_id[5]  + sb_num_per_id[6] + sb_num_per_id[7] + \
                           sb_num_per_id[8] + sb_num_per_id[9] + sb_num_per_id[10] + sb_num_per_id[11] + sb_num_per_id[12] + sb_num_per_id[13] + sb_num_per_id[14]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x1000 + reg_idx*4)) = (sb_num_per_id[14] << 16) + (iq_buff_addr << 8) + BIT7 + 14; 
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2]  + sb_num_per_id[3]  + sb_num_per_id[4]  + sb_num_per_id[5]  + sb_num_per_id[6]  + sb_num_per_id[7] + \
                           sb_num_per_id[8] + sb_num_per_id[9] + sb_num_per_id[10] + sb_num_per_id[11] + sb_num_per_id[12] + sb_num_per_id[13] + sb_num_per_id[14] + sb_num_per_id[15]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x1000 + reg_idx*4)) = (sb_num_per_id[15] << 16) + (iq_buff_addr << 8) + BIT7 + 15; 
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2]  + sb_num_per_id[3]  + sb_num_per_id[4]  + sb_num_per_id[5]  + sb_num_per_id[6]  + sb_num_per_id[7] + \
                           sb_num_per_id[8] + sb_num_per_id[9] + sb_num_per_id[10] + sb_num_per_id[11] + sb_num_per_id[12] + sb_num_per_id[13] + sb_num_per_id[14] + sb_num_per_id[15] + \
                           sb_num_per_id[16]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x1000 + reg_idx*4)) = (sb_num_per_id[16] << 16) + (iq_buff_addr << 8) + BIT7 + 16; 
        }
        else if(reg_idx < (sb_num_per_id[0]  + sb_num_per_id[1] + sb_num_per_id[2]  + sb_num_per_id[3]  + sb_num_per_id[4]  + sb_num_per_id[5]  + sb_num_per_id[6]  + sb_num_per_id[7] + \
                           sb_num_per_id[8]  + sb_num_per_id[9] + sb_num_per_id[10] + sb_num_per_id[11] + sb_num_per_id[12] + sb_num_per_id[13] + sb_num_per_id[14] + sb_num_per_id[15] + \
                           sb_num_per_id[16] + sb_num_per_id[17]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x1000 + reg_idx*4)) = (sb_num_per_id[17] << 16) + (iq_buff_addr << 8) + BIT7 + 17; 
        }
        else if(reg_idx < (sb_num_per_id[0]  + sb_num_per_id[1]  + sb_num_per_id[2]  + sb_num_per_id[3]  + sb_num_per_id[4]  + sb_num_per_id[5]  + sb_num_per_id[6]  + sb_num_per_id[7] + \
                           sb_num_per_id[8]  + sb_num_per_id[9]  + sb_num_per_id[10] + sb_num_per_id[11] + sb_num_per_id[12] + sb_num_per_id[13] + sb_num_per_id[14] + sb_num_per_id[15] + \
                           sb_num_per_id[16] + sb_num_per_id[17] + sb_num_per_id[18]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x1000 + reg_idx*4)) = (sb_num_per_id[18] << 16) + (iq_buff_addr << 8) + BIT7 + 18; 
        } 
        else if(reg_idx < (sb_num_per_id[0]  + sb_num_per_id[1]  + sb_num_per_id[2]  + sb_num_per_id[3]  + sb_num_per_id[4]  + sb_num_per_id[5]  + sb_num_per_id[6]  + sb_num_per_id[7] + \
                           sb_num_per_id[8]  + sb_num_per_id[9]  + sb_num_per_id[10] + sb_num_per_id[11] + sb_num_per_id[12] + sb_num_per_id[13] + sb_num_per_id[14] + sb_num_per_id[15] + \
                           sb_num_per_id[16] + sb_num_per_id[17] + sb_num_per_id[18] + sb_num_per_id[19]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x1000 + reg_idx*4)) = (sb_num_per_id[19] << 16) + (iq_buff_addr << 8) + BIT7 + 19; 
        }
        else if(reg_idx < (sb_num_per_id[0]  + sb_num_per_id[1]  + sb_num_per_id[2]  + sb_num_per_id[3]  + sb_num_per_id[4]  + sb_num_per_id[5]  + sb_num_per_id[6]  + sb_num_per_id[7] + \
                           sb_num_per_id[8]  + sb_num_per_id[9]  + sb_num_per_id[10] + sb_num_per_id[11] + sb_num_per_id[12] + sb_num_per_id[13] + sb_num_per_id[14] + sb_num_per_id[15] + \
                           sb_num_per_id[16] + sb_num_per_id[17] + sb_num_per_id[18] + sb_num_per_id[19] + sb_num_per_id[20]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x1000 + reg_idx*4)) = (sb_num_per_id[20] << 16) + (iq_buff_addr << 8) + BIT7 + 20; 
        }   
        else if(reg_idx < (sb_num_per_id[0]  + sb_num_per_id[1]  + sb_num_per_id[2]  + sb_num_per_id[3]  + sb_num_per_id[4]  + sb_num_per_id[5]  + sb_num_per_id[6]  + sb_num_per_id[7] + \
                           sb_num_per_id[8]  + sb_num_per_id[9]  + sb_num_per_id[10] + sb_num_per_id[11] + sb_num_per_id[12] + sb_num_per_id[13] + sb_num_per_id[14] + sb_num_per_id[15] + \
                           sb_num_per_id[16] + sb_num_per_id[17] + sb_num_per_id[18] + sb_num_per_id[19] + sb_num_per_id[20] + sb_num_per_id[21]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x1000 + reg_idx*4)) = (sb_num_per_id[21] << 16) + (iq_buff_addr << 8) + BIT7 + 21; 
        } 
        else if(reg_idx < (sb_num_per_id[0]  + sb_num_per_id[1]  + sb_num_per_id[2]  + sb_num_per_id[3]  + sb_num_per_id[4]  + sb_num_per_id[5]  + sb_num_per_id[6]  + sb_num_per_id[7] + \
                           sb_num_per_id[8]  + sb_num_per_id[9]  + sb_num_per_id[10] + sb_num_per_id[11] + sb_num_per_id[12] + sb_num_per_id[13] + sb_num_per_id[14] + sb_num_per_id[15] + \
                           sb_num_per_id[16] + sb_num_per_id[17] + sb_num_per_id[18] + sb_num_per_id[19] + sb_num_per_id[20] + sb_num_per_id[21] + sb_num_per_id[22]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x1000 + reg_idx*4)) = (sb_num_per_id[22] << 16) + (iq_buff_addr << 8) + BIT7 + 22; 
        }  
        else if(reg_idx < (sb_num_per_id[0]  + sb_num_per_id[1]  + sb_num_per_id[2]  + sb_num_per_id[3]  + sb_num_per_id[4]  + sb_num_per_id[5]  + sb_num_per_id[6]  + sb_num_per_id[7] + \
                           sb_num_per_id[8]  + sb_num_per_id[9]  + sb_num_per_id[10] + sb_num_per_id[11] + sb_num_per_id[12] + sb_num_per_id[13] + sb_num_per_id[14] + sb_num_per_id[15] + \
                           sb_num_per_id[16] + sb_num_per_id[17] + sb_num_per_id[18] + sb_num_per_id[19] + sb_num_per_id[20] + sb_num_per_id[21] + sb_num_per_id[22] + sb_num_per_id[23]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x1000 + reg_idx*4)) = (sb_num_per_id[23] << 16) + (iq_buff_addr << 8) + BIT7 + 23; 
        }
        else                                                                 
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x1000 + reg_idx*4)) = 0;
        }
    }


    /* axc rx cfg */
    for(reg_idx = 0; reg_idx < 256; reg_idx++)
    {
        iq_buff_addr = reg_idx;

        if(reg_idx < sb_num_per_id[0])
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0xC00 + reg_idx*4)) = (iq_buff_addr << 8) + BIT7 + 0; 
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0xC00 + reg_idx*4)) = (iq_buff_addr << 8) + BIT7 + 1; 
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0xC00 + reg_idx*4)) = (iq_buff_addr << 8) + BIT7 + 2;
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2] + sb_num_per_id[3]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0xC00 + reg_idx*4)) = (iq_buff_addr << 8) + BIT7 + 3;
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2] + sb_num_per_id[3] + sb_num_per_id[4]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0xC00 + reg_idx*4)) = (iq_buff_addr << 8) + BIT7 + 4;
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2] + sb_num_per_id[3] + sb_num_per_id[4] + sb_num_per_id[5]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0xC00 + reg_idx*4)) = (iq_buff_addr << 8) + BIT7 + 5;
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2] + sb_num_per_id[3] + sb_num_per_id[4] + sb_num_per_id[5] + sb_num_per_id[6]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0xC00 + reg_idx*4)) = (iq_buff_addr << 8) + BIT7 + 6;
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2] + sb_num_per_id[3] + sb_num_per_id[4] + sb_num_per_id[5] + sb_num_per_id[6] + sb_num_per_id[7]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0xC00 + reg_idx*4)) = (iq_buff_addr << 8) + BIT7 + 7;
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2] + sb_num_per_id[3] + sb_num_per_id[4] + sb_num_per_id[5] + sb_num_per_id[6] + sb_num_per_id[7] + \
                           sb_num_per_id[8]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0xC00 + reg_idx*4)) = (iq_buff_addr << 8) + BIT7 + 8;
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2] + sb_num_per_id[3] + sb_num_per_id[4] + sb_num_per_id[5] + sb_num_per_id[6] + sb_num_per_id[7] + \
                           sb_num_per_id[8] + sb_num_per_id[9]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0xC00 + reg_idx*4)) = (iq_buff_addr << 8) + BIT7 + 9;
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2] + sb_num_per_id[3] + sb_num_per_id[4] + sb_num_per_id[5] + sb_num_per_id[6] + sb_num_per_id[7] + \
                           sb_num_per_id[8] + sb_num_per_id[9] + sb_num_per_id[10]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0xC00 + reg_idx*4)) = (iq_buff_addr << 8) + BIT7 + 10;
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2] + sb_num_per_id[3] + sb_num_per_id[4] + sb_num_per_id[5] + sb_num_per_id[6] + sb_num_per_id[7] + \
                           sb_num_per_id[8] + sb_num_per_id[9] + sb_num_per_id[10] + sb_num_per_id[11]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0xC00 + reg_idx*4)) = (iq_buff_addr << 8) + BIT7 + 11;
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2]  + sb_num_per_id[3]  + sb_num_per_id[4] + sb_num_per_id[5] + sb_num_per_id[6] + sb_num_per_id[7] + \
                           sb_num_per_id[8] + sb_num_per_id[9] + sb_num_per_id[10] + sb_num_per_id[11] + sb_num_per_id[12]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0xC00 + reg_idx*4)) = (iq_buff_addr << 8) + BIT7 + 12;
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2]  + sb_num_per_id[3]  + sb_num_per_id[4]  + sb_num_per_id[5] + sb_num_per_id[6] + sb_num_per_id[7] + \
                           sb_num_per_id[8] + sb_num_per_id[9] + sb_num_per_id[10] + sb_num_per_id[11] + sb_num_per_id[12] + sb_num_per_id[13]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0xC00 + reg_idx*4)) = (iq_buff_addr << 8) + BIT7 + 13;
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2]  + sb_num_per_id[3]  + sb_num_per_id[4]  + sb_num_per_id[5]  + sb_num_per_id[6] + sb_num_per_id[7] + \
                           sb_num_per_id[8] + sb_num_per_id[9] + sb_num_per_id[10] + sb_num_per_id[11] + sb_num_per_id[12] + sb_num_per_id[13] + sb_num_per_id[14]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0xC00 + reg_idx*4)) = (iq_buff_addr << 8) + BIT7 + 14;
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2]  + sb_num_per_id[3]  + sb_num_per_id[4]  + sb_num_per_id[5]  + sb_num_per_id[6]  + sb_num_per_id[7] + \
                           sb_num_per_id[8] + sb_num_per_id[9] + sb_num_per_id[10] + sb_num_per_id[11] + sb_num_per_id[12] + sb_num_per_id[13] + sb_num_per_id[14] + sb_num_per_id[15]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0xC00 + reg_idx*4)) = (iq_buff_addr << 8) + BIT7 + 15;
        }
        else if(reg_idx < (sb_num_per_id[0] + sb_num_per_id[1] + sb_num_per_id[2]  + sb_num_per_id[3]  + sb_num_per_id[4]  + sb_num_per_id[5]  + sb_num_per_id[6]  + sb_num_per_id[7] + \
                           sb_num_per_id[8] + sb_num_per_id[9] + sb_num_per_id[10] + sb_num_per_id[11] + sb_num_per_id[12] + sb_num_per_id[13] + sb_num_per_id[14] + sb_num_per_id[15] + \
                           sb_num_per_id[16]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0xC00 + reg_idx*4)) = (iq_buff_addr << 8) + BIT7 + 16;
        }
        else if(reg_idx < (sb_num_per_id[0]  + sb_num_per_id[1] + sb_num_per_id[2]  + sb_num_per_id[3]  + sb_num_per_id[4]  + sb_num_per_id[5]  + sb_num_per_id[6]  + sb_num_per_id[7] + \
                           sb_num_per_id[8]  + sb_num_per_id[9] + sb_num_per_id[10] + sb_num_per_id[11] + sb_num_per_id[12] + sb_num_per_id[13] + sb_num_per_id[14] + sb_num_per_id[15] + \
                           sb_num_per_id[16] + sb_num_per_id[17]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0xC00 + reg_idx*4)) = (iq_buff_addr << 8) + BIT7 + 17;
        }
        else if(reg_idx < (sb_num_per_id[0]  + sb_num_per_id[1]  + sb_num_per_id[2]  + sb_num_per_id[3]  + sb_num_per_id[4]  + sb_num_per_id[5]  + sb_num_per_id[6]  + sb_num_per_id[7] + \
                           sb_num_per_id[8]  + sb_num_per_id[9]  + sb_num_per_id[10] + sb_num_per_id[11] + sb_num_per_id[12] + sb_num_per_id[13] + sb_num_per_id[14] + sb_num_per_id[15] + \
                           sb_num_per_id[16] + sb_num_per_id[17] + sb_num_per_id[18]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0xC00 + reg_idx*4)) = (iq_buff_addr << 8) + BIT7 + 18;
        }
        else if(reg_idx < (sb_num_per_id[0]  + sb_num_per_id[1]  + sb_num_per_id[2]  + sb_num_per_id[3]  + sb_num_per_id[4]  + sb_num_per_id[5]  + sb_num_per_id[6]  + sb_num_per_id[7] + \
                           sb_num_per_id[8]  + sb_num_per_id[9]  + sb_num_per_id[10] + sb_num_per_id[11] + sb_num_per_id[12] + sb_num_per_id[13] + sb_num_per_id[14] + sb_num_per_id[15] + \
                           sb_num_per_id[16] + sb_num_per_id[17] + sb_num_per_id[18] + sb_num_per_id[19]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0xC00 + reg_idx*4)) = (iq_buff_addr << 8) + BIT7 + 19;
        }
        else if(reg_idx < (sb_num_per_id[0]  + sb_num_per_id[1]  + sb_num_per_id[2]  + sb_num_per_id[3]  + sb_num_per_id[4]  + sb_num_per_id[5]  + sb_num_per_id[6]  + sb_num_per_id[7] + \
                           sb_num_per_id[8]  + sb_num_per_id[9]  + sb_num_per_id[10] + sb_num_per_id[11] + sb_num_per_id[12] + sb_num_per_id[13] + sb_num_per_id[14] + sb_num_per_id[15] + \
                           sb_num_per_id[16] + sb_num_per_id[17] + sb_num_per_id[18] + sb_num_per_id[19] + sb_num_per_id[20]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0xC00 + reg_idx*4)) = (iq_buff_addr << 8) + BIT7 + 20;
        }
        else if(reg_idx < (sb_num_per_id[0]  + sb_num_per_id[1]  + sb_num_per_id[2]  + sb_num_per_id[3]  + sb_num_per_id[4]  + sb_num_per_id[5]  + sb_num_per_id[6]  + sb_num_per_id[7] + \
                           sb_num_per_id[8]  + sb_num_per_id[9]  + sb_num_per_id[10] + sb_num_per_id[11] + sb_num_per_id[12] + sb_num_per_id[13] + sb_num_per_id[14] + sb_num_per_id[15] + \
                           sb_num_per_id[16] + sb_num_per_id[17] + sb_num_per_id[18] + sb_num_per_id[19] + sb_num_per_id[20] + sb_num_per_id[21]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0xC00 + reg_idx*4)) = (iq_buff_addr << 8) + BIT7 + 21;
        }
        else if(reg_idx < (sb_num_per_id[0]  + sb_num_per_id[1]  + sb_num_per_id[2]  + sb_num_per_id[3]  + sb_num_per_id[4]  + sb_num_per_id[5]  + sb_num_per_id[6]  + sb_num_per_id[7] + \
                           sb_num_per_id[8]  + sb_num_per_id[9]  + sb_num_per_id[10] + sb_num_per_id[11] + sb_num_per_id[12] + sb_num_per_id[13] + sb_num_per_id[14] + sb_num_per_id[15] + \
                           sb_num_per_id[16] + sb_num_per_id[17] + sb_num_per_id[18] + sb_num_per_id[19] + sb_num_per_id[20] + sb_num_per_id[21] + sb_num_per_id[22]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0xC00 + reg_idx*4)) = (iq_buff_addr << 8) + BIT7 + 22;
        }
        else if(reg_idx < (sb_num_per_id[0]  + sb_num_per_id[1]  + sb_num_per_id[2]  + sb_num_per_id[3]  + sb_num_per_id[4]  + sb_num_per_id[5]  + sb_num_per_id[6]  + sb_num_per_id[7] + \
                           sb_num_per_id[8]  + sb_num_per_id[9]  + sb_num_per_id[10] + sb_num_per_id[11] + sb_num_per_id[12] + sb_num_per_id[13] + sb_num_per_id[14] + sb_num_per_id[15] + \
                           sb_num_per_id[16] + sb_num_per_id[17] + sb_num_per_id[18] + sb_num_per_id[19] + sb_num_per_id[20] + sb_num_per_id[21] + sb_num_per_id[22] + sb_num_per_id[23]))
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0xC00 + reg_idx*4)) = (iq_buff_addr << 8) + BIT7 + 23;
        }                                                                        
        else
        {
            (*(volatile uint32_t *)(CPRI_CPU_BASE + 0xC00 + reg_idx*4)) = 0;
        }
    }


    if(option == 8)
    {
        if(mode == OTIC_MAP_FIGURE10)	
        {
            /* map tx cfg */
            for(reg_idx = 0; reg_idx < 256; reg_idx++)
            {
                if(reg_idx < 5)
                {
                    if(reg_idx < 4)
                    {
                                                                                      /* c_opcode       c_shift       b_opcode       b_shift       a_opcode        a_shift     valid    incr_addr     en */
                        (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_CLR << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (0 << 0);
                    }		 
                    else
                    {
                        (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_CLR << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (1 << 0);
                    }	
                }
                else if(reg_idx < (5 + 2))
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4))     = (OP_CLR << 30) + (0 << 24) + (OP_SRL << 22) + (32 << 16) + (OP_SRL << 14) + (48 << 8) + (1 << 5) + (1 << 4) + (1 << 0);
                    reg_idx++;
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4))     = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_CLR << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (0 << 0);
                }
                else if(reg_idx < (5 + 2 + 64)) 
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4))     = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_SRL << 14) + (32 << 8) + (1 << 5) + (1 << 4) + (1 << 0);
                    reg_idx++;
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4))     = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_PT  << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (0 << 0);                    
                }
                else if(reg_idx < (5 + 2 + 64 + 8))
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4))     = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_SRL << 14) + (32 << 8) + (1 << 5) + (1 << 4) + (1 << 0);                   
                    reg_idx++;	
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4))     = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_PT  << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (0 << 0);  
                }
                else if(reg_idx < (5 + 2 + 64 + 8 + 1))
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4))     = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_SRL << 14) + (40 << 8) + (1 << 5) + (1 << 4) + (1 << 0);                    
                }
                else
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4))     = 0;
                }
            }


            /* map rx cfg */
            for(reg_idx = 0; reg_idx < 256; reg_idx++)
            {
                if(reg_idx < 5)
                {
                                                                                  /* c_opcode       c_shift       b_opcode       b_shift       a_opcode        a_shift     valid    incr_addr     en */
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_CLR << 14) + (0  << 8) + (0 << 5) + (0 << 4) + (1 << 0);                  
                }
                else if(reg_idx < (5 + 2))
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_SLL << 14) + (32 << 8) + (1 << 5) + (1 << 4) + (1 << 0);   
                    reg_idx++;
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_SLL << 22) + (48 << 16) + (OP_CLR << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (1 << 0);   

                }
                else if(reg_idx < (5 + 2 + 64))
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_CLR << 14) + (0  << 8) + (0 << 5) + (0 << 4) + (1 << 0);                     
                    reg_idx++;
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_SLL << 22) + (32 << 16) + (OP_PT  << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (1 << 0);                    
                }
                else if(reg_idx < (5 + 2 + 64 + 8))
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_CLR << 14) + (0  << 8) + (0 << 5) + (0 << 4) + (1 << 0);                     
                    reg_idx++;
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_SLL << 22) + (32 << 16) + (OP_PT  << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (1 << 0); 
                }
                else if(reg_idx < (5 + 2 + 64 + 8 + 1))
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_SLL << 14) + (40 << 8) + (1 << 5) + (1 << 4) + (1 << 0);                      
                }
                else
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = 0;
                }
            }
        }


        if(mode == OTIC_MAP_FIGURE12)
        {
            /* map tx cfg */
            for(reg_idx = 0; reg_idx < 256; reg_idx++)
            {
                if(reg_idx < 5)
                {
                                                                                  /* c_opcode       c_shift       b_opcode       b_shift       a_opcode      a_shift      valid     incr_addr     en */
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0 << 16) + (OP_CLR << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (0 << 0);
                }
                else if(reg_idx < (5 + 2))
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0 << 16) + (OP_SRL << 14) + (32 << 8) + (1 << 5) + (1 << 4) + (1 << 0); 
                    reg_idx++;
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0 << 16) + (OP_CLR << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (0 << 0);
                }
                else if(reg_idx < (5 + 2 + 64))
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0 << 16) + (OP_SRL << 14) + (32 << 8) + (1 << 5) + (1 << 4) + (1 << 0);
                    reg_idx++;
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0 << 16) + (OP_PT  << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (0 << 0);
                }
                else if(reg_idx < (5 + 2 + 64 + 8))
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0 << 16) + (OP_CLR << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (0 << 0);
                }
                else if(reg_idx < (5 + 2 + 64 + 8 + 1))
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0 << 16) + (OP_PT  << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (1 << 0);
                }
                else
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4)) = 0;
                }
            }


            /* map rx cfg */
            for(reg_idx = 0; reg_idx < 256; reg_idx++)
            {
                if(reg_idx < 5)
                {
                                                                                  /* c_opcode       c_shift       b_opcode       b_shift       a_opcode      a_shift      valid     incr_addr    en */
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_CLR << 14) + (0 << 8) + (0 << 5) + (0 << 4) + (1 << 0);
                }
                else if(reg_idx < (5 + 2))
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_CLR << 14) + (0 << 8) + (0 << 5) + (0 << 4) + (1 << 0);
                    reg_idx++;
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_SLL << 22) + (32 << 16) + (OP_CLR << 14) + (0 << 8) + (1 << 5) + (1 << 4) + (1 << 0);
                }
                else if(reg_idx < (5 + 2 + 64))
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_CLR << 14) + (0 << 8) + (0 << 5) + (0 << 4) + (1 << 0);
                    reg_idx++;
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_SLL << 22) + (32 << 16) + (OP_PT  << 14) + (0 << 8) + (1 << 5) + (1 << 4) + (1 << 0);
                }
                else if(reg_idx < (5 + 2 + 64 + 8))
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_CLR << 14) + (0 << 8) + (0 << 5) + (0 << 4) + (1 << 0);
                }
                else if(reg_idx < (5 + 2 + 64 + 8 + 1))
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_PT  << 14) + (0 << 8) + (1 << 5) + (1 << 4) + (1 << 0);
                }
                else
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = 0;
                }
            }
        }
    }
	

    if(option == 10)
    {
        if(mode == OTIC_MAP_FIGURE16)
        {
            /* map tx cfg */
            for(reg_idx = 0; reg_idx < 256; reg_idx++)
            {
                if(reg_idx < 12)
                {
                    if(reg_idx < 11)
                    {
                                                                                      /* c_opcode       c_shift       b_opcode       b_shift       a_opcode        a_shift     valid    incr_addr     en */                        
                        (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_CLR << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (0 << 0); 
                    }						 
                    else
                    {
                        (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_CLR << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (1 << 0);                         
                    }					
                }
                else if(reg_idx < (12 + 2))
                {		 		 
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4))     = (OP_CLR << 30) + (0 << 24) + (OP_SRL << 22) + (32 << 16) + (OP_SRL << 14) + (48 << 8) + (1 << 5) + (1 << 4) + (1 << 0);                      
                    reg_idx++;
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4))     = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_CLR << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (0 << 0);                     
                }
                else if(reg_idx < (12 + 2 + 64))
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4))     = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_SRL << 14) + (32 << 8) + (1 << 5) + (1 << 4) + (1 << 0);                     
                    reg_idx++;
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4))     = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_PT  << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (0 << 0);                     
                }
                else if(reg_idx < (12 + 2 + 64 + 64))
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4))     = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_SRL << 14) + (32 << 8) + (1 << 5) + (1 << 4) + (1 << 0);                     
                    reg_idx++;	
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4))     = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_PT  << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (0 << 0);
                }
                else if(reg_idx < (12 + 2 + 64 + 64 + 49))
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4))     = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_CLR << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (0 << 0);                     
                }
                else if(reg_idx < (12 + 2 + 64 + 64 + 49 + 1))
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4))     = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_SRL << 14) + (32 << 8) + (1 << 5) + (1 << 4) + (1 << 0);                     
                }
                else
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4))     = 0;
                }
            }


            /* map rx cfg */
            for(reg_idx = 0; reg_idx < 256; reg_idx++)
            {
                if(reg_idx < 12)
                {
                                                                                  /* c_opcode       c_shift       b_opcode       b_shift       a_opcode        a_shift     valid    incr_addr     en */  
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_CLR << 14) + (0  << 8) + (0 << 5) + (0 << 4) + (1 << 0);                                            
                }
                else if(reg_idx < (12 + 2))
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_SLL << 14) + (32 << 8) + (1 << 5) + (1 << 4) + (1 << 0);                     
                    reg_idx++;
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_SLL << 22) + (48 << 16) + (OP_CLR << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (1 << 0); 
                }
                else if(reg_idx < (12 + 2 + 64))
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_CLR << 14) + (0  << 8) + (0 << 5) + (0 << 4) + (1 << 0); 
                    reg_idx++;
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_SLL << 22) + (32 << 16) + (OP_PT  << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (1 << 0);                     
                }
                else if(reg_idx < (12 + 2 + 64 + 64))
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_CLR << 14) + (0  << 8) + (0 << 5) + (0 << 4) + (1 << 0); 
                    reg_idx++;
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_SLL << 22) + (32 << 16) + (OP_PT  << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (1 << 0); 
                }
                else if(reg_idx < (12 + 2 + 64 + 64 + 49))
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_CLR << 14) + (0  << 8) + (0 << 5) + (0 << 4) + (1 << 0);                     
                }
                else if(reg_idx < (12 + 2 + 64 + 64 + 49 + 1))
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_SLL << 14) + (32 << 8) + (1 << 5) + (1 << 4) + (1 << 0);                     
                }
                else
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = 0;
                }
            }
        }

        if(mode == OTIC_MAP_8NR_8LTE)
        {
            /* map tx cfg */
            for(reg_idx = 0; reg_idx < 256; reg_idx++)
            {
                if(reg_idx < 12)
                {
                                                                                   /* c_opcode       c_shift       b_opcode       b_shift       a_opcode        a_shift     valid    incr_addr     en */                        
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_CLR << 14) + (0  << 8) + (0 << 5) + (1 << 4) + (0 << 0); 
                }
                else if(reg_idx < (12 + 2))                         // nr&lte's compress factor
                {		 		 
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_SRL << 14) + (32 << 8) + (1 << 5) + (1 << 4) + (1 << 0);                      
                    reg_idx++;
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_PT  << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (0 << 0);                     
                }
                else if(reg_idx < (12 + 2 + 64))                    // nr_cell0's iq data
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_SRL << 14) + (32 << 8) + (1 << 5) + (1 << 4) + (1 << 0);                     
                    reg_idx++;
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_PT  << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (0 << 0);                     
                }
                else if(reg_idx < (12 + 2 + 64 + 64))               // nr_cell1's iq data
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_SRL << 14) + (32 << 8) + (1 << 5) + (1 << 4) + (1 << 0);                     
                    reg_idx++;	
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_PT  << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (0 << 0);
                }
                else if(reg_idx < (12 + 2 + 64 + 64 + 32))          // lte_cell's iq data
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_SRL << 14) + (32 << 8) + (1 << 5) + (1 << 4) + (1 << 0);                     
                    reg_idx++;	
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_PT  << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (0 << 0);
                }
                else if(reg_idx < (12 + 2 + 64 + 64 + 32 + 16))     // reserved zone
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_CLR << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (0 << 0);                     
                }
                else if(reg_idx < (12 + 2 + 64 + 64 + 32 + 16 + 2)) // nr&lte's agc factor
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_SRL << 14) + (32 << 8) + (1 << 5) + (1 << 4) + (1 << 0);                     
                    reg_idx++;	
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_PT  << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (0 << 0);                  
                }
                else
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x800 + reg_idx*4)) = 0;
                }
            }    

            /* map rx cfg */
            for(reg_idx = 0; reg_idx < 256; reg_idx++)
            {
                if(reg_idx < 12)
                {
                                                                                  /* c_opcode       c_shift       b_opcode       b_shift       a_opcode        a_shift     valid    incr_addr     en */  
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_CLR << 14) + (0  << 8) + (0 << 5) + (0 << 4) + (1 << 0);                                            
                }
                else if(reg_idx < (12 + 2))                         // nr&lte's compress factor
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_CLR << 14) + (0  << 8) + (0 << 5) + (0 << 4) + (1 << 0); 
                    reg_idx++;
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_SLL << 22) + (32 << 16) + (OP_PT  << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (1 << 0); 
                }
                else if(reg_idx < (12 + 2 + 64))                    // nr_cell0's iq data
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_CLR << 14) + (0  << 8) + (0 << 5) + (0 << 4) + (1 << 0); 
                    reg_idx++;
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_SLL << 22) + (32 << 16) + (OP_PT  << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (1 << 0);                     
                }
                else if(reg_idx < (12 + 2 + 64 + 64))               // nr_cell1's iq data
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_CLR << 14) + (0  << 8) + (0 << 5) + (0 << 4) + (1 << 0); 
                    reg_idx++;
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_SLL << 22) + (32 << 16) + (OP_PT  << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (1 << 0); 
                }
                else if(reg_idx < (12 + 2 + 64 + 64 + 32))          // lte_cell's iq data
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_CLR << 14) + (0  << 8) + (0 << 5) + (0 << 4) + (1 << 0); 
                    reg_idx++;
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_SLL << 22) + (32 << 16) + (OP_PT  << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (1 << 0);                     
                }
                else if(reg_idx < (12 + 2 + 64 + 64 + 32 + 16))     // reserved zone
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_CLR << 14) + (0  << 8) + (0 << 5) + (0 << 4) + (1 << 0);                       
                }
                else if(reg_idx < (12 + 2 + 64 + 64 + 32 + 16 + 2)) // nr&lte's agc factor
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_CLR << 22) + (0  << 16) + (OP_CLR << 14) + (0  << 8) + (0 << 5) + (0 << 4) + (1 << 0); 
                    reg_idx++;
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = (OP_CLR << 30) + (0 << 24) + (OP_SLL << 22) + (32 << 16) + (OP_PT  << 14) + (0  << 8) + (1 << 5) + (1 << 4) + (1 << 0);                      
                }
                else
                {
                    (*(volatile uint32_t *)(CPRI_CPU_BASE + 0x400 + reg_idx*4)) = 0;
                }
            }            
        }
    }
  	
    CPRI_MAP_TOGGLE = BIT0;
}




/**********************************************************************************************************************
*  Function: cpri_map_start
*  Description: 
*  Input : none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void cpri_map_start(void)
{
    do_write((&CPRI_MAP_CFG), (BIT4|BIT0));
}


/**********************************************************************************************************************
*  Function: cpri_map_stop
*  Description: 
*  Input : none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void cpri_map_stop(void)
{
	do_write((&CPRI_MAP_CFG), BIT0);
}


/**********************************************************************************************************************
*  Function: cpri_aux_rx_init
*  Description: 
*  Input : 
*         <name>                     <type>                      <desc>
*         buffer0                    uint32_t                    buffer0's address
*         buffer1                    uint32_t                    buffer1's address
*         length0                    uint32_t                    buffer0's length
*         length1                    uint32_t                    buffer1's length
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void cpri_aux_rx_init(uint32_t buffer0, uint32_t buffer1, uint32_t length0, uint32_t length1)
{
    do_write(&AWADDR0,  buffer0);
    do_write(&AXI_LEN0, length0);
    do_write(&AWADDR1,  buffer1);
    do_write(&AXI_LEN1, length1);
}


/**********************************************************************************************************************
*  Function: cpri_aux_rx_enable
*  Description: 
*  Input : 
*         <name>                     <type>                      <desc>
*         axi_mode                   uint32_t                    aux rx mode select
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void cpri_aux_rx_enable(uint32_t axi_mode)
{
    AUX_CTRL_REG |= (axi_mode << 16);   // 1:写完第一块buffer停住, 2:写完两块buffer停住, 3：来回写两块buffer
}



/**********************************************************************************************************************
*  Function: cpri_aux_rx_disable
*  Description: 
*  Input : none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void cpri_aux_rx_disable(void)
{
    AUX_CTRL_REG &= 0xFFFCFFFF;         // BIT[17:16], 2'b00:axi disable(disable aux_rx & disable aux_tx)
}


/**********************************************************************************************************************
*  Function: cpri_aux_tx_enable
*  Description: 
*  Input : none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void cpri_aux_tx_enable(void)
{
    AUX_CTRL_REG |= BIT0;               // BIT[0]，2'b1:enable aux_tx
}


/**********************************************************************************************************************
*  Function: cpri_aux_tx_disable
*  Description: 
*  Input : none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void cpri_aux_tx_disable(void)
{
	AUX_CTRL_REG &= 0xFFFFFFFE;         // BIT[0]，2'b0:disable aux_tx
}


/**********************************************************************************************************************
*  Function: cpri_aux_tx_data_insert
*  Description: 
*  Input : 
*         <name>                     <type>                      <desc>
*         num                        uint32_t                    insert data num
*         bf_in_hf                   uint8_t *                   which basic frame in hyper frame
*         pos_in_bf                  uint8_t *                   whick pos to insert
*         data_tx                    uint8_t *                   data 
*         data_en                    uint8_t *                   data enable
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void cpri_aux_tx_data_insert(uint32_t num, uint8_t bf_in_hf[], uint8_t pos_in_bf[], uint32_t data_tx[], uint32_t data_en[])
{
    AUX_INS_NUM = num;          // 插入点的个数，有效值为1~10

    /* 配置data插入到一个HF中的哪个BF */
    DATA_CNT_X0 = (bf_in_hf[3] << 24) + (bf_in_hf[2] << 16) + (bf_in_hf[1] << 8) + bf_in_hf[0];
    DATA_CNT_X1 = (bf_in_hf[7] << 24) + (bf_in_hf[6] << 16) + (bf_in_hf[5] << 8) + bf_in_hf[4];
    DATA_CNT_X2 = (bf_in_hf[9] << 8)  + (bf_in_hf[8] << 0);

    /* 配置data插入到一个BF中的哪个位置 */
    DATA_CNT_POS0 = (pos_in_bf[3] << 24) + (pos_in_bf[2] << 16) + (pos_in_bf[1] << 8) + pos_in_bf[0];
    DATA_CNT_POS1 = (pos_in_bf[7] << 24) + (pos_in_bf[6] << 16) + (pos_in_bf[5] << 8) + pos_in_bf[4];
    DATA_CNT_POS2 = (pos_in_bf[9] << 8)  + (pos_in_bf[8] << 0);

    /* 配置待插入的32位data值 */
    AUX_DATA_TX0 = data_tx[0];
    AUX_DATA_TX1 = data_tx[1];
    AUX_DATA_TX2 = data_tx[2];
    AUX_DATA_TX3 = data_tx[3]; 
    AUX_DATA_TX4 = data_tx[4];
    AUX_DATA_TX5 = data_tx[5];
    AUX_DATA_TX6 = data_tx[6];
    AUX_DATA_TX7 = data_tx[7];
    AUX_DATA_TX8 = data_tx[8];
    AUX_DATA_TX9 = data_tx[9];

    /* 对插入的data按位使能插入 */
    AUX_DATA_EN0 = data_en[0];
    AUX_DATA_EN1 = data_en[1];
    AUX_DATA_EN2 = data_en[2];
    AUX_DATA_EN3 = data_en[3];
    AUX_DATA_EN4 = data_en[4];
    AUX_DATA_EN5 = data_en[5];
    AUX_DATA_EN6 = data_en[6];
    AUX_DATA_EN7 = data_en[7];
    AUX_DATA_EN8 = data_en[8];
    AUX_DATA_EN9 = data_en[9];

}


/**********************************************************************************************************************
*  Function: cpri_aux_tx_ctrl_insert
*  Description: 
*  Input : 
*         <name>                     <type>                      <desc>
*         num                        uint32_t                    insert ctrl num
*         bf_in_hf                   uint8_t *                   which basic frame in hyper frame
*         pos_in_bf                  uint8_t *                   whick pos to insert
*         ctrl_tx                    uint8_t *                   ctrl 
*         ctrl_en                    uint8_t *                   ctrl enable
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void cpri_aux_tx_ctrl_insert(uint32_t num, uint8_t bf_in_hf[], uint8_t pos_in_bf[], uint8_t ctrl_tx[], uint8_t ctrl_en[])
{

    /* 配置ctrl插入到一个HF中的哪个BF */
	CTRL_CNT_X0 = (bf_in_hf[3] << 24) + (bf_in_hf[2] << 16) + (bf_in_hf[1] << 8) + bf_in_hf[0];
	CTRL_CNT_X1 = (bf_in_hf[7] << 24) + (bf_in_hf[6] << 16) + (bf_in_hf[5] << 8) + bf_in_hf[4];
	CTRL_CNT_X2 = (bf_in_hf[9] << 8)  + (bf_in_hf[8] << 0);

   /* 配置ctrl插入到一个BF中的哪个位置 */
	CTRL_CNT_POS0 = (pos_in_bf[3] << 24) + (pos_in_bf[2] << 16) + (pos_in_bf[1] << 8) + pos_in_bf[0];
	CTRL_CNT_POS1 = (pos_in_bf[7] << 24) + (pos_in_bf[6] << 16) + (pos_in_bf[5] << 8) + pos_in_bf[4];
	CTRL_CNT_POS2 = (pos_in_bf[9] << 8)  + (pos_in_bf[8] << 0);

	/* AUX_DATA_TX0~9[31:0]:插入控制值 4bit*/
	AUX_CTRL_TX0 = ((ctrl_tx[7]&0xF) << 28) + ((ctrl_tx[6]&0xF) << 24) + ((ctrl_tx[5]&0xF) << 20) + ((ctrl_tx[4]&0xF) << 16) + \
                   ((ctrl_tx[3]&0xF) << 12) + ((ctrl_tx[2]&0xF) << 8)  + ((ctrl_tx[1]&0xF) << 4)  + ((ctrl_tx[0]&0xF) << 0);
	AUX_CTRL_TX1 = ((ctrl_tx[9]&0xF) << 4)  + ((ctrl_tx[8]&0xF) << 0);

	/* AUX_DATA_TX0~9[31:0]:插入data值 4bit*/
	AUX_CTRL_EN_TX0 = ((ctrl_en[7]&0xF) << 28) + ((ctrl_en[6]&0xF) << 24) + ((ctrl_en[5]&0xF) << 20) + ((ctrl_en[4]&0xF) << 16) + \
                      ((ctrl_en[3]&0xF) << 12) + ((ctrl_en[2]&0xF) << 8)  + ((ctrl_en[1]&0xF) << 4)  + ((ctrl_en[0]&0xF) << 0);
	AUX_CTRL_EN_TX1 = ((ctrl_en[9]&0xF) << 4)  + ((ctrl_en[8]&0xF) << 0);
		
}


/**********************************************************************************************************************
*  Function: cpri_aux_hfnsync_flag
*  Description: 
*  Input : none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
uint32_t cpri_aux_hfnsync_flag(void)
{
    return ((do_read_volatile(&AUX_INT_FLAG))&BIT2) >> 2;
}


/**********************************************************************************************************************
*  Function: cpri_aux_buffer1_full_flag
*  Description: 
*  Input : none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
uint32_t cpri_aux_buffer1_full_flag(void)
{
    return ((do_read_volatile(&AUX_INT_FLAG))&BIT4) >> 4;
}



/**********************************************************************************************************************
*  Function: cpri_aux_get_tx_hfn
*  Description: 
*  Input : none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
uint32_t cpri_aux_get_tx_hfn(void)
{
    return (((do_read_volatile(&AUX_CNT0)) >> 0)&0xFF);
}



/**********************************************************************************************************************
*  Function: cpri_aux_get_tx_bfn
*  Description: 
*  Input : none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
uint32_t cpri_aux_get_tx_bfn(void)
{
    return (((do_read_volatile(&AUX_CNT0)) >> 8)&0xFF);
}


/**********************************************************************************************************************
*  Function: cpri_aux_get_rx_hfn
*  Description: 
*  Input : none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
uint32_t cpri_aux_get_rx_hfn(void)
{
    return (((do_read_volatile(&AUX_CNT2)) >> 0)&0xFF);
}




/**********************************************************************************************************************
*  Function: cpri_headerram_tx_enable
*  Description: 
*  Input : none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void cpri_headerram_tx_enable(void)
{

    uint32_t val = do_read_volatile(&CPRI_FRAME_TX_CFG);
	val |= BIT5;
	do_write(&CPRI_FRAME_TX_CFG, val);

}


/**********************************************************************************************************************
*  Function: cpri_headerram_tx_disable
*  Description: 
*  Input : none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void cpri_headerram_tx_disable(void)
{

    uint32_t val = do_read_volatile(&CPRI_FRAME_TX_CFG);
	val &= (~BIT5);
	do_write(&CPRI_FRAME_TX_CFG, val);

}


/**********************************************************************************************************************
*  Function: cpri_headerram_tx
*  Description: 
*  Input : 
*         <name>                     <type>                      <desc>
*         bf_in_hf                   uint32_t                    which basic frame in hyper frame
*         no_in_ctrlword             uint32_t                    which word in ctrlword of basic frame
*         hdr_data                   uint32_t                    data
*         hdr_data_en                uint32_t                    data enable
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void cpri_headerram_tx(uint32_t bf_in_hf, uint32_t no_in_ctrlword, uint32_t hdr_data, uint32_t hdr_data_en)
{
	do_write(&CPRI_FRAME_TX_HDR_ADDR, (hdr_data_en << 24) + (bf_in_hf << 2) + no_in_ctrlword);
	do_write(&CPRI_FRAME_TX_HDR_DATA, hdr_data);
}



/**********************************************************************************************************************
*  Function: cpri_headerram_rx
*  Description: 
*  Input : 
*         <name>                     <type>                      <desc>
*         bf_in_hf                   uint32_t                    which basic frame in hyper frame
*         no_in_ctrlword             uint32_t                    which word in ctrlword of basic frame
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
uint32_t cpri_headerram_rx(uint32_t bf_in_hf, uint32_t no_in_ctrlword)
{
	uint32_t hdr_data;


	do_write(&CPRI_FRAME_RX_HDR_ADDR, (bf_in_hf << 2) + no_in_ctrlword);
	__ucps2_synch(f_SM);

	hdr_data = do_read_volatile(&CPRI_FRAME_RX_HDR_DATA);
	
	if(no_in_ctrlword == 3)
	{
		do_write(&CPRI_FRAME_RX_HDR_ADDR, (bf_in_hf << 2));
	}

	return hdr_data;
}


/**********************************************************************************************************************
*  Function: cpri_headerram_tx_init
*  Description: 
*  Input : none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void cpri_headerram_tx_init(void)
{
    uint32_t xs_idx, ns_idx;

    cpri_headerram_tx_disable();

    for(xs_idx = 0; xs_idx < 4; xs_idx++)
    {
        for(ns_idx = 0; ns_idx < 64; ns_idx++)
        {
            cpri_headerram_tx(64*xs_idx + ns_idx, 0, 0, 0);
            cpri_headerram_tx(64*xs_idx + ns_idx, 1, 0, 0);
            cpri_headerram_tx(64*xs_idx + ns_idx, 2, 0, 0);
            cpri_headerram_tx(64*xs_idx + ns_idx, 3, 0, 0);
        }
    }

    do_write(&CPRI_FRAME_RX_HDR_ADDR, 0);
    __ucps2_synch(f_SM);
}


/**********************************************************************************************************************
*  Function: cpri_set_rru_msg
*  Description: 
*  Input : 
*         <name>                     <type>                      <desc>
*         rru_msg                    CpriRruMsg_t                CK's ctrlword msg
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
uint8_t SECTION_ALIGNED g_rru_msg_data[CPRI_RRU_MSG_LEN];
int32_t set_cpri_rru_msg(CpriRruMsg_t rru_msg)
{
    if((rru_msg.msg_addr == 0) || (rru_msg.msg_len == 0))
    {
        return -1;
    }

    for(int i = 0; i < 41; i++)
    {
        *((uint32_t *)&g_rru_msg_data[0] + i) = SWAP32(do_read_volatile(rru_msg.msg_addr + i*4));		
    }

    return 0;
}



/**********************************************************************************************************************
*  Function: cpri_ctrlword_insert
*  Description: 
*  Input : 
*         <name>                     <type>                      <desc>
*         vendor                     VendorType_e                vender type
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
uint32_t ctrlword_tx_test_data[48*2];
uint32_t ctrlword_insert_cnt = 0;
void cpri_ctrlword_insert(VendorType_e vendor)
{
    uint32_t xs_idx  = 0;
    uint32_t ns_idx  = 0;
    uint32_t tx_hfn  = 0;
    uint32_t *p_data = NULL;
  

    ctrlword_insert_cnt++;

    tx_hfn = cpri_aux_get_tx_hfn();

    /* CK Vendor Specific Control Words*/
    if(vendor == CK_NORM)
    {	
        p_data = (uint32_t *)g_rru_msg_data;
        if((tx_hfn == CK_NORM_CTRL_WORD_TX_HFN) || (tx_hfn == (CK_NORM_CTRL_WORD_TX_HFN - 1)))
        {
            cpri_headerram_tx_disable();

            for(xs_idx = 0; xs_idx < 4; xs_idx++)
            {
                if(xs_idx == 3)
                {
                    for(ns_idx = 8; ns_idx < 13; ns_idx++)
                    {
                        cpri_headerram_tx(64*xs_idx + ns_idx, 0, (*(p_data + xs_idx*12 + ns_idx - 8)), 0xF);
                    }
                }
                else
                {
                    for(ns_idx = 8; ns_idx < 20; ns_idx++)
                    {
                        cpri_headerram_tx(64*xs_idx + ns_idx, 0, (*(p_data + xs_idx*12 + ns_idx - 8)), 0xF);
                    }
                }
            }

            cpri_headerram_tx_enable();
            debug_write((DBG_DDR_IDX_CPRI_BASE + 141), do_read_volatile(&AUX_CNT0));
        }
        debug_write((DBG_DDR_IDX_CPRI_BASE + 144), tx_hfn);
    }


    /* SL Test Vendor Specific Control Words*/
    if(vendor == SL_TEST)
    {
        p_data = ctrlword_tx_test_data + (ctrlword_insert_cnt%2)*48;
        if((tx_hfn == SL_TEST_CTRL_WORD_TX_HFN) || (tx_hfn == (SL_TEST_CTRL_WORD_TX_HFN - 1)))
        {
            cpri_headerram_tx_disable();

            for(xs_idx = 0; xs_idx < 4; xs_idx++)
            {
                for(ns_idx = 8; ns_idx < 20; ns_idx++)
                {
                    cpri_headerram_tx(64*xs_idx + ns_idx, 0, (*(p_data + xs_idx*12 + ns_idx - 8)), 0xF);
                }
            }

            cpri_headerram_tx_enable();
            debug_write((DBG_DDR_IDX_CPRI_BASE + 141), do_read_volatile(&AUX_CNT0));
        }
    }

}


