/**********************************************************************************************************************
*  Copyright (C), 2022-2026, SMARTLOGIC TECHNOLOGY LTD.
*  File Name: cpri_init.s.c
*  Create Date: 23/12/25
*  Description: CPRI Driver Module Source File
*  Change History: 
*      <author>      <time>      <version>   <desc>
*    1. cheng.wan     23/12/25      1.0       refactor this module
**********************************************************************************************************************/
/* -------------------------------------------------- <INC FILE> --------------------------------------------------- */
#include "ucp_utility.h"
#include "ucp_printf.h"
#include "inter_vector.h"
#include "ecs_rfm_dm_mgt.h"
#include "cpri_timer.h"
#include "cpri_driver.h"


/* --------------------------------------------------- <DEF&DEC> --------------------------------------------------- */
extern void cpri_link_init(uint32_t option);
extern void ecpri_cprimode_init(void);


/* --------------------------------------------------- <FUNC DEF> --------------------------------------------------- */
/**********************************************************************************************************************
*  function: cpri_ecprimode_init
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
void cpri_ecprimode_init(void)
{
    cpri_timer_ecprimode_clk_init();
    cpri_mtimer_init(NR_SCS_30K, 10);   // slot:500us, one tdd has 10 slots
}


/**********************************************************************************************************************
*  function: cpri_para_init
*  description: 
*  input : 
*         <name>                     <type>                      <desc>
*          option                    uint32_t                     cpri line speed option
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
volatile stCpriIntStat gCpriIntStatus;
void cpri_para_init(uint32_t option)
{
    EcsRfmDmLocalMgt_t  *pEcsDmLocalMgt = NULL;
    stCpriPara          *pCpriPara      = NULL; 
    stFhAlarmStat       *pAlarmStatus   = NULL;


    pEcsDmLocalMgt = get_ecs_rfm_dm_local_mgt();
    pCpriPara      = pEcsDmLocalMgt->pCpriPara;
    pAlarmStatus   = pEcsDmLocalMgt->pAlarmStatus;

    if(pCpriPara == NULL)
    {
        return;
    }

    pAlarmStatus->alarmFlag = FH_NO_ALARM;
    memset(pCpriPara, 0, sizeof(stCpriPara));
    memset((void*)(&gCpriIntStatus), 0, sizeof(stCpriIntStat));

    switch(option)
    {
        case (7):
        {
            pCpriPara->option_num           = option;
            pCpriPara->core_clk             = 245760000;
            pCpriPara->pcs_core_clk         = 245760000;
            pCpriPara->cpu_clk              = 200000000;
            pCpriPara->gmii_tx_rx_clk       = 125000000;
            pCpriPara->pcs_fine_delay_clk   = 250000000;
            pCpriPara->pcs_serdes_clk       = 491520000;
            pCpriPara->serdes_data_width    = 20;
            pCpriPara->cpri_rate_coff       = 16;
            pCpriPara->line_rate            = 9830400000;
            pCpriPara->valid_line_rate      = pCpriPara->cpri_rate_coff * 491500000;
            pCpriPara->encode_flag          = 0;        // 0:8B/10B, 1:64B/66B
            pCpriPara->rs_fec_flag          = 0;
            pCpriPara->len_word             = 128;
            pCpriPara->len_ctrl_word        = 128;
            pCpriPara->aux_cnt_pos_max      = 64;
            break;
        }

        case (8):
        {
            pCpriPara->option_num           = option;
            pCpriPara->core_clk             = 307200000;
            pCpriPara->pcs_core_clk         = 153600000;
            pCpriPara->cpu_clk              = 200000000;
            pCpriPara->gmii_tx_rx_clk       = 125000000;
            pCpriPara->pcs_fine_delay_clk   = 250000000;
            pCpriPara->pcs_serdes_clk       = 316800000;
            pCpriPara->serdes_data_width    = 32; 
            pCpriPara->cpri_rate_coff       = 20;
            pCpriPara->line_rate            = 10137600000;
            pCpriPara->valid_line_rate      = pCpriPara->cpri_rate_coff * 491500000;
            pCpriPara->encode_flag          = 1; 	
            pCpriPara->rs_fec_flag          = 0;
            pCpriPara->len_word             = 160;
            pCpriPara->len_ctrl_word        = 128;
            pCpriPara->aux_cnt_pos_max      = 80;
            break;
        }

        case (9):
        {
            pCpriPara->option_num           = option;
            pCpriPara->core_clk             = 368640000;
            pCpriPara->pcs_core_clk         = 184320000;
            pCpriPara->cpu_clk              = 200000000;
            pCpriPara->gmii_tx_rx_clk       = 125000000;
            pCpriPara->pcs_fine_delay_clk   = 250000000;
            pCpriPara->pcs_serdes_clk       = 380160000;
            pCpriPara->serdes_data_width    = 32;
            pCpriPara->cpri_rate_coff       = 24;
            pCpriPara->line_rate            = 12165120000;
            pCpriPara->valid_line_rate      = pCpriPara->cpri_rate_coff * 491500000;
            pCpriPara->encode_flag          = 1; 
            pCpriPara->rs_fec_flag          = 0; 
            pCpriPara->len_word             = 192;
            pCpriPara->len_ctrl_word        = 128;
            pCpriPara->aux_cnt_pos_max      = 96;
            break;
        }

        case (10):
        {
            pCpriPara->option_num           = option;
            pCpriPara->core_clk             = 737280000;
            pCpriPara->pcs_core_clk         = 368640000;
            pCpriPara->cpu_clk              = 200000000;
            pCpriPara->gmii_tx_rx_clk       = 125000000;
            pCpriPara->pcs_fine_delay_clk   = 250000000;
            pCpriPara->pcs_serdes_clk       = 760320000;
            pCpriPara->serdes_data_width    = 32; 
            pCpriPara->cpri_rate_coff       = 48;
            pCpriPara->line_rate            = 24330240000;
            pCpriPara->valid_line_rate      = pCpriPara->cpri_rate_coff * 491500000;
            pCpriPara->encode_flag          = 1; 	
            pCpriPara->rs_fec_flag          = 0; 
            pCpriPara->len_word             = 384;
            pCpriPara->len_ctrl_word        = 128;
            pCpriPara->aux_cnt_pos_max      = 192;
            break;
        }

        default:
        {
            break;
        }
    }

    debug_write((DBG_DDR_IDX_DRV_BASE + 41), pCpriPara->core_clk);      // 0xB7E060A4
}




/**********************************************************************************************************************
*  function: cpri_core_isr
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
extern uint32_t reCfgFlag;
void cpri_core_isr(void)
{
    uint32_t aux_int_flag = AUX_INT_FLAG&0x7;
    uint32_t rfp_int_flag = AUX_RFP_INT_FLAG;
    //uint32_t aux_int_flag = AUX_INT_FLAG&0x3F;    
    uint32_t alm_int_flag = ALARM_FLAG;


    /* clear core&gmac int  */
    AUX_INT_FLAG     = aux_int_flag;
    AUX_RFP_INT_FLAG = rfp_int_flag;
    //ALARM_FLAG       = alm_int_flag;


    if(rfp_int_flag&0x1)    // aux_rfp_rx
    {
        gCpriIntStatus.rxRfpIntCnt++;
        debug_write((DBG_DDR_IDX_DRV_BASE + 120), gCpriIntStatus.rxRfpIntCnt);  // 0xB7E061E0
        //gScrRxRfpValL = CPRI_TMR_RX_SCRL_REG;
		//gScrRxRfpValH = CPRI_TMR_RX_SCRH_REG;
    }

    if(rfp_int_flag&0x2)    // aux_rfp_tx
    {
        gCpriIntStatus.txRfpIntCnt++;
        debug_write((DBG_DDR_IDX_DRV_BASE + 121), gCpriIntStatus.txRfpIntCnt);
    }

    if(alm_int_flag)        // tx_axc_alarm
    {
        gCpriIntStatus.alarmIntCnt++;           // iq buffer full
        debug_write((DBG_DDR_IDX_DRV_BASE + 116), gCpriIntStatus.alarmIntCnt); // 0xB7E061D0 

        if(gCpriIntStatus.alarmIntCnt < 4)
        {
            debug_write((DBG_DDR_IDX_DRV_BASE + 116 + gCpriIntStatus.alarmIntCnt), alm_int_flag);
        }
		//debug_write((DBG_DDR_IDX_CPRI_BASE + 403 + ((2*gCpriAlarmIntCnt)&0xFF)), gCpriTimerPara.tddOffsetIntCnt);
        do_write(&ALARM_FLAG, alm_int_flag);
    }
	
#if 0
    /*	4   buffer1_full    RWC 1’b0    ddr buffer1写满标志
     *	3   buffer0_full    RWC 1’b0    ddr buffer0写满标志
     *	2   hfnsync_flag    RWC 1’b0    aux_hfnsync标志
     *	1   lof_flag        RWC 1’b0    aux_lof标志
     *	0   l1_reset_flag   RWC 1’b0    aux_l1_reset标志
    */
    if(aux_int_flag&0x18)
    {
        gCpriIntStatus.ddrFifoIntCnt++;
    }

    if(aux_int_flag&0x20)
    {
        gCpriIntStatus.rxFifoIntCnt++;
    }
#endif

    if(aux_int_flag&0x1)    // l1_reset_flag
    {
        gCpriIntStatus.l1ResetIntCnt++;
        debug_write((DBG_DDR_IDX_DRV_BASE + 123), gCpriIntStatus.l1ResetIntCnt);
    }

    if(aux_int_flag&0x2)    // lof_flag
    {
        gCpriIntStatus.lofIntCnt++;
        gCpriIntStatus.cpriSyncFlag = 0;
        do_write(CSU_STOP_CMD_ADDR, 1);
        debug_write((DBG_DDR_IDX_DRV_BASE + 124), gCpriIntStatus.lofIntCnt);
    }

    if(aux_int_flag&0x4)    // hfnsync_flag
    {
        gCpriIntStatus.syncIntCnt++;
        if(gCpriIntStatus.syncIntCnt > 1)
        {
            gCpriIntStatus.cpriSyncFlag = 1;
            if(reCfgFlag == 1)
            {
                reCfgFlag = 2;
                debug_write((DBG_DDR_IDX_DRV_BASE + 16), 2);
            }
            debug_write((DBG_DDR_IDX_DRV_BASE + 125), gCpriIntStatus.syncIntCnt);
            debug_write((DBG_DDR_IDX_DRV_BASE + 126), GET_STC_CNT());
        }
    }
}



/**********************************************************************************************************************
*  function: cpri_gmac_isr
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
void cpri_gmac_isr(void)
{
    uint32_t aux_int_flag = AUX_INT_FLAG&0xFF0000;

    if(aux_int_flag)
    {
        gCpriIntStatus.gmacIntCnt++;
    }
}


/**********************************************************************************************************************
*  function: cpri_int_init
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
void cpri_int_init(void)
{
    uint32_t    ape_Id  = get_core_id(); 
    uint32_t    int_num = 0;
    int32_t     ret_val = 0;


    /* core&gmac int enable */
    AUX_INT_EN     = 0x01FF0007;        // enable  core int(aux_l1_reset、aux_lof、aux_hfnsync)，
                                        // disable core int(buffer0_full、buffer1_full、fifo_full)，enable gmac int
    AUX_RFP_INT_EN = 0x3;               // enable  core int(aux_rfp_tx、aux_rfp_rx)
    ALARM_INTEN    = 0x0;               // disable core int(tx_axc_alarm)

    /* core int register */
    int_num = APC_CPRI_SUB_INTR0;
    ret_val = smart_irq_request(int_num, cpri_core_isr);
    if(ret_val != 0)
    {
        UCP_PRINT_EMPTY("attach int num: 0x%x error. errno = 0x%x. \r\n", int_num, ret_val);
        debug_write(DBG_DDR_ERR_IDX(ape_Id, 4), ret_val);
    }

    /* gmac int register */
#if 0
    int_num = APC_CPRI_SUB_INTR1;
    ret_val = smart_irq_request(int_num, cpri_gmac_isr);
    if(ret_val != 0)
    {
        UCP_PRINT_EMPTY("attach int num: 0x%x error. errno = 0x%x. \r\n", int_num, ret_val);
    }
#endif

}


/**********************************************************************************************************************
*  function: cpri_init
*  description: 
*  input : 
*         <name>                     <type>                      <desc>
*          option                    uint32_t                     cpri line speed option
*          mode                      uint32_t                     cpri axc mapping mode
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
void cpri_init(uint32_t option, uint32_t mode)
{
    
#ifdef PALLADIUM_TEST
    int32_t apeId = get_core_id();
    int32_t flag  = 1;
    debug_write((DBG_DDR_IDX_DRV_BASE + 2 + (apeId << 2)), flag++);
#endif

    phy_para_init(PROTOCOL_CPRI, option);

    do_write(&JECS_CRG_PLLSEL, 0xC21);
    ucp_nop(400);

#if 0
    // temp, to be change
    /**********************************************************************/
    //JECS_CTRL_QOS_M3_SRIO_AXIM |= BIT19;

    //                             gnss_sel     stc_sel     cpri_sel
    //JECS_CTRL_CPRI_GMAC_PHY_INT = (0x0 << 4) | (0x5 << 7) | (0x6<<10);
    //do_write(RF_LVDS_PMUX1_REG_ADDR, (do_read_volatile(RF_LVDS_PMUX1_REG_ADDR)|0x000C0000));	// pinmux, GPIO1B25, gpio func, 0x3
    //do_write(GPIO1B_DIR_REG_ADDR, (do_read_volatile(GPIO1B_DIR_REG_ADDR)|BIT25));				// GPIO1B25, output, 10ms trigger for phy
    do_write(PB14_CTRL_REG_ADDR, (do_read_volatile(PB14_CTRL_REG_ADDR)|(1<<7)|(1<<6)));
    do_write(GPIO0_A29_PINMUX_REG_ADDR, (do_read_volatile(GPIO0_A29_PINMUX_REG_ADDR)|0x20000000));	// pinmux, GPIO0A29, gnss pps int func, 0x2
    //do_write(RF_LVDS_PMUX1_REG_ADDR, (do_read_volatile(RF_LVDS_PMUX1_REG_ADDR)|0x30000000));	// pinmux, GPIO1B30, gpio func, 0x3
    do_write(RF_LVDS_PMUX1_REG_ADDR, (do_read_volatile(RF_LVDS_PMUX1_REG_ADDR)|0x30000000));	// pinmux, GPIO1B30, gpio func, 0x3
    //do_write(&JECS_CTRL_QOS_M3_SRIO_AXIM, do_read_volatile(&JECS_CTRL_QOS_M3_SRIO_AXIM) | BIT19); 
    do_write(&JECS_CTRL_CPRI_GMAC_PHY_INT, (0x0 << 4) | (0x5 << 7) | (0x5<<10));

    /**********************************************************************/
#endif

    cpri_para_init(option);
    cpri_delay_init();
    cpri_link_status_init();
    UCP_PRINT_EMPTY("cpri para init finished.\r\n");
#ifdef PALLADIUM_TEST
    debug_write((DBG_DDR_IDX_DRV_BASE + 2 + (apeId << 2)), flag++);
#endif

    cpri_link_init(option);
    UCP_PRINT_EMPTY("cpri link init finished.\r\n");
#ifdef PALLADIUM_TEST
    debug_write((DBG_DDR_IDX_DRV_BASE + 2 + (apeId << 2)), flag++);
#endif

    cpri_int_init();
    UCP_PRINT_EMPTY("cpri int init finished.\r\n");
#ifdef PALLADIUM_TEST
    debug_write((DBG_DDR_IDX_DRV_BASE + 2 + (apeId << 2)), flag++);
#endif

#if 0
    if(NetType == LTE_MODE)
    {
        cpri_mtimer_init(LTE_SCS_ID, 10);   // slot:1000us, one tdd has 10 slots
    }else if(NetType == NR_MODE)
#endif
    {
        cpri_mtimer_init(NR_SCS_30K, 10);   // slot:500us, one tdd has 10 slots
    }
    UCP_PRINT_EMPTY("cpri mtimer init finished.\r\n");
#ifdef PALLADIUM_TEST
    debug_write((DBG_DDR_IDX_DRV_BASE + 2 + (apeId << 2)), flag++);
#endif

#if 0
    while(1 >= gCpriIntStatus.syncIntCnt)
    {
        //debug_write((DBG_DDR_IDX_DRV_BASE+126), gCpriIntStatus.syncIntCnt);
    }
    //smart_irq_disable(APC_CPRI_SUB_INTR0);
    UCP_PRINT_EMPTY("AUX CNT0: 0x%x \r\n", AUX_CNT0);
    UCP_PRINT_EMPTY("AUX CNT1: 0x%x \r\n", AUX_CNT1);
    UCP_PRINT_EMPTY("AUX CNT2: 0x%x \r\n", AUX_CNT2);
    UCP_PRINT_EMPTY("SELF SYNC finish.\r\n");
#endif
#ifdef PALLADIUM_TEST
    debug_write((DBG_DDR_IDX_DRV_BASE + 2 + (apeId << 2)), flag++);
#endif

    cpri_axc_map_cfg(option, mode);
    UCP_PRINT_EMPTY("cpri axc_map config finished.\r\n");
#ifdef PALLADIUM_TEST
    debug_write((DBG_DDR_IDX_DRV_BASE + 2 + (apeId << 2)), flag++);
#endif

    cpri_headerram_tx_init();
    UCP_PRINT_EMPTY("cpri headerram tx init finished.\r\n");
#ifdef PALLADIUM_TEST
    debug_write((DBG_DDR_IDX_DRV_BASE + 2 + (apeId << 2)), flag++); 
#endif

    ecpri_cprimode_init();
    UCP_PRINT_EMPTY("cpri ecpri mode init finished.\r\n");
#ifdef PALLADIUM_TEST
    debug_write((DBG_DDR_IDX_DRV_BASE + 2 + (apeId << 2)), flag++);
#endif

}