/**********************************************************************************************************************
*  Copyright (C), 2022-2026, SMARTLOGIC TECHNOLOGY LTD.
*  File Name: cpri_link.s.c
*  Create Date: 23/12/25
*  Description: CPRI Driver Module Source File
*  Change History: 
*      <author>      <time>      <version>   <desc>
*    1. cheng.wan     23/12/25      1.0       refactor this module
**********************************************************************************************************************/
/* -------------------------------------------------- <INC FILE> --------------------------------------------------- */
#include "ucp_printf.h"
#include "ucp_utility.h"
#include "mem_sections.h"
#include "ucp_sfr_c.h"
#include "ucp_js_ctrl.h"
#include "ucp_js_subcrg.h"
#include "ucp_pet_ctrl.h"
#include "ucp_cpri.h"
#include "ucp_pma.h"
#include "hw_cpri.h"
#include "phy_para.h"
#include "mtimer_drv.h"
#include "cpri_csu.h"


/* --------------------------------------------------- <DEF&DEC> --------------------------------------------------- */




/* -------------------------------------------------- <FUNC DEF> --------------------------------------------------- */
/**********************************************************************************************************************
*  function: cpri_pma_reset
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
void cpri_pma_reset(void)
{
    do_write(&JECS_CRG_PMA16_RST_CTRL,  do_read_volatile(&JECS_CRG_PMA16_RST_CTRL)  | BIT24); 
    do_write(&JECS_CRG_PMA0_RST_CTRL,   do_read_volatile(&JECS_CRG_PMA0_RST_CTRL)   | BIT24); 
    do_write(&JECS_CRG_PMA1_RST_CTRL,   do_read_volatile(&JECS_CRG_PMA1_RST_CTRL)   | BIT24); 
    do_write(&JECS_CRG_PMA2_RST_CTRL,   do_read_volatile(&JECS_CRG_PMA2_RST_CTRL)   | BIT24); 
    do_write(&JECS_CRG_PMA3_RST_CTRL,   do_read_volatile(&JECS_CRG_PMA3_RST_CTRL)   | BIT24); 
    do_write(&JECS_CRG_PMA8_RST_CTRL,   do_read_volatile(&JECS_CRG_PMA8_RST_CTRL)   | BIT24); 
    do_write(&JECS_CRG_PMA9_RST_CTRL,   do_read_volatile(&JECS_CRG_PMA9_RST_CTRL)   | BIT24); 
    do_write(&JECS_CRG_PMA10_RST_CTRL,  do_read_volatile(&JECS_CRG_PMA10_RST_CTRL)  | BIT24); 
    do_write(&JECS_CRG_PMA11_RST_CTRL,  do_read_volatile(&JECS_CRG_PMA11_RST_CTRL)  | BIT24); 
    do_write(&JECS_CRG_PMA12_RST_CTRL,  do_read_volatile(&JECS_CRG_PMA12_RST_CTRL)  | BIT24); 
    do_write(&JECS_CRG_PMA13_RST_CTRL,  do_read_volatile(&JECS_CRG_PMA13_RST_CTRL)  | BIT24); 
    do_write(&JECS_CRG_PMA14_RST_CTRL,  do_read_volatile(&JECS_CRG_PMA14_RST_CTRL)  | BIT24); 
    do_write(&JECS_CRG_PMA15_RST_CTRL,  do_read_volatile(&JECS_CRG_PMA15_RST_CTRL)  | BIT24); 
    do_write(&PET_EQ0_RST_CFG,          do_read_volatile(&PET_EQ0_RST_CFG)          | BIT24); 
    do_write(&PET_EQ1_RST_CFG,          do_read_volatile(&PET_EQ1_RST_CFG)          | BIT24); 
    do_write(&PET_EQ2_RST_CFG,          do_read_volatile(&PET_EQ2_RST_CFG)          | BIT24); 
    do_write(&PET_EQ3_RST_CFG,          do_read_volatile(&PET_EQ3_RST_CFG)          | BIT24); 
    do_write(&PET_PAMRX0_RST_CFG,       do_read_volatile(&PET_PAMRX0_RST_CFG)       | BIT24); 
    do_write(&PET_PAMRX1_RST_CFG,       do_read_volatile(&PET_PAMRX1_RST_CFG)       | BIT24); 
    do_write(&PET_PAMRX2_RST_CFG,       do_read_volatile(&PET_PAMRX2_RST_CFG)       | BIT24); 
    do_write(&PET_PAMRX3_RST_CFG,       do_read_volatile(&PET_PAMRX3_RST_CFG)       | BIT24); 
    do_write(&PET_RX0_RST_CFG,          do_read_volatile(&PET_RX0_RST_CFG)          | BIT24); 
    do_write(&PET_RX1_RST_CFG,          do_read_volatile(&PET_RX1_RST_CFG)          | BIT24); 
    do_write(&PET_RX2_RST_CFG,          do_read_volatile(&PET_RX2_RST_CFG)          | BIT24); 
    do_write(&PET_RX3_RST_CFG,          do_read_volatile(&PET_RX3_RST_CFG)          | BIT24); 
    do_write(&PET_TX0_RST_CFG,          do_read_volatile(&PET_TX0_RST_CFG)          | BIT24); 
    do_write(&PET_TX1_RST_CFG,          do_read_volatile(&PET_TX1_RST_CFG)          | BIT24); 
    do_write(&PET_TX2_RST_CFG,          do_read_volatile(&PET_TX2_RST_CFG)          | BIT24); 
    do_write(&PET_TX3_RST_CFG,          do_read_volatile(&PET_TX3_RST_CFG)          | BIT24); 
    do_write(&PET_PWR_RST_CFG,          do_read_volatile(&PET_PWR_RST_CFG)          | BIT24); 
}


/**********************************************************************************************************************
*  function: cpri_clk_init
*  description: 
*  input : 
*         <name>                     <type>                      <desc>
*         option                     uint32_t                    cpri line speed option
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
void cpri_clk_init(uint32_t option)
{
    if(option <= 7) // 8B/10B
    {
        do_write(&JECS_CRG_PLLSEL, do_read_volatile(&JECS_CRG_PLLSEL)|BIT24);
        JECS_CRG_CPRI_CDR_CLK_CTRL        = 0x523000;               // cdr div clk output divided by 4
        JECS_CRG_CPRI_CORE_CLK_CTRL       = 0x521000;               // u_cpri_core_div div cfg
        JECS_CRG_CPRI_PCS_CORE_CLK_CTRL   = 0x520000;               // u_cpri_pcs_core_div div cfg
    }
    else            // 64B/66B
    {
        do_write(&JECS_CRG_PLLSEL, do_read_volatile(&JECS_CRG_PLLSEL)|(BIT26|BIT24));
        do_write(&JECS_CRG_CPRI_CDR_CLK_CTRL,      0x52a000);       // cdr div clk output divided by 11
        do_write(&JECS_CRG_CPRI_CORE_CLK_CTRL,     0x520000);       // u_cpri_core_div div cfg
        do_write(&JECS_CRG_CPRI_PCS_CORE_CLK_CTRL, 0x521000);       // u_cpri_pcs_core_div div cfg
    }

    ucp_nop(400);

    do_write(&JECS_CRG_PLLSEL, do_read_volatile(&JECS_CRG_PLLSEL)&(~BIT4));
    ucp_nop(400);

    do_write(&JS_CRG_SAM3_CLK_CTRL, 0x520000);
    ucp_nop(400);
}


/**********************************************************************************************************************
*  function: core_clk_normal_to_xtal
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
uint32_t pllsel_temp = 0;
void core_clk_normal_to_xtal(void)
{	
    do_write(&JECS_CRG_CPRI_CORE_CLK_CTRL, 0x121000);                       // 暂停输出core时钟 
    pllsel_temp = do_read_volatile(&JECS_CRG_PLLSEL);
    do_write(&JECS_CRG_PLLSEL, (pllsel_temp&0xF87FFFF0)|(BIT23)|(BIT3));    // ecs pll选择xtal_clk时钟输入并输出给cpri core div
    do_write(&ESP_CLK_CFG_REG, 0x121000);                                   // 暂停apb时钟输出   
    do_write(&CLK_CTL_REG1,  ((do_read_volatile(&CLK_CTL_REG1))&(~BIT2)));  // 切换PLL2时钟源为xtal_clk时钟  
    do_write(&ESP_CLK_CFG_REG, 0x521000);                                   // 使能apb时钟输出  
    do_write(&JECS_CRG_CPRI_CORE_CLK_CTRL, 0x520000);                       // 使能输出core时钟 
}


/**********************************************************************************************************************
*  function: core_clk_xtal_to_normal
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
void core_clk_xtal_to_normal(void)	
{
    do_write(&JECS_CRG_CPRI_CORE_CLK_CTRL, 0x121000);                       // 暂停输出core时钟 
    do_write(&JECS_CRG_PLLSEL, pllsel_temp);                                // 切换CORE时钟源为正常的PMA TX给cpri core div 
    do_write(&ESP_CLK_CFG_REG, 0x121000);                                   // 暂停apb时钟输出
    do_write(&CLK_CTL_REG1, do_read_volatile(&CLK_CTL_REG1) | BIT2);        // 切换PLL2时钟为PLL2输出时钟
    do_write(&ESP_CLK_CFG_REG, 0x524000);                                   // 使能apb时钟输出 
    do_write(&JECS_CRG_CPRI_CORE_CLK_CTRL, 0x520000);                       // 使能输出core时钟 
}


/**********************************************************************************************************************
*  function: cpri_pma_common_cfg
*  description: 
*  input : 
*         <name>                     <type>                      <desc>
*         option                     uint32_t                    cpri line speed option
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
DDR0 uint32_t ate_tx_eq_main[20] = {18, 20, 19, 21, 24, 22, 21, 17, 18, 20, 16, 14, 15, 16, 18, 16, 13, 13, 15, 12};
DDR0 uint32_t ate_tx_eq_post[20] = {24, 16, 20, 12,  0,  0,	 0, 18, 12,	 0, 32, 16, 12,  8,  0,  0, 12, 10,	 0, 24};
DDR0 uint32_t ate_tx_eq_pre[20]  = { 0,  0,  0,  0,  0,  8, 12, 10, 12, 16,  0,  0,  0,  0,  0,  8,  8, 10, 12,  0};
void cpri_pma_common_cfg(uint32_t option)
{
    uint32_t m = 1;
    uint32_t n = 0;
    uint32_t t = 1;
    uint32_t ate_ref_range                = 0;
    uint32_t ate_ref_clk_div2_en          = 0;
    uint32_t ate_ref_raw_clk_div2_en      = 0;
    uint32_t ate_ref_clk_mplla_div        = 0;
    uint32_t ate_lane_ref_sel             = 0;
    uint32_t ate_mplla_fb_clk_div4_en     = 0;
    uint32_t ate_mplla_multiplier         = 0;
    uint32_t ate_mplla_tx_clk_div         = 0;
    uint32_t ate_mplla_word_clk_div       = 0;
    uint32_t ate_mplla_ssc_en             = 0;
    uint32_t ate_mplla_ssc_up_spread      = 0;
    uint32_t ate_mplla_ssc_peak           = 0;
    uint32_t ate_mplla_ssc_step_size      = 0;
    uint32_t ate_mplla_frac_en            = 0;
    uint32_t ate_mplla_frac_quot          = 0;
    uint32_t ate_mplla_frac_den           = 0;
    uint32_t ate_mplla_frac_rem           = 0;
    uint32_t ate_refa_lane_clk_en         = 0;
    uint32_t ate_bs_rx_level              = 0;
    uint32_t ate_bs_tx_lowswing           = 0;
    uint32_t ate_bs_rx_bigswing           = 0;
    uint32_t ate_mplla_init_cal_disable   = 0;
    uint32_t ate_refa_dig_clk_sel         = 0;
    uint32_t ate_ref_dco_bypass           = 0;
    uint32_t ate_refa_dco_ld_val          = 0;
    uint32_t ate_ref_dco_target           = 0;
    uint32_t ate_ref_dco_dig_range        = 0;
    uint32_t ate_mplla_bw_threshold       = 0;
    uint32_t ate_mplla_bw_low             = 0;
    uint32_t ate_mplla_bw_high            = 0;
    uint32_t ate_mplla_ctl_buf_bypass     = 0;
    uint32_t ate_mplla_short_lock_en      = 0;
    uint32_t ate_rx_term_offset           = 0;
    uint32_t ate_txdn_term_offset         = 0;
    uint32_t ate_txup_term_offset         = 0;
    uint32_t ate_sup_misc                 = 0;
    uint32_t ate_rx_vref_ctrl             = 0;
    uint32_t ate_rx_ref_ld_val            = 0;
    uint32_t ate_rx_vco_ld_val            = 0;
    uint32_t ate_rx_cdr_ppm_max           = 0;

    //lane
    uint32_t ate_tx_misc                  = 0;
    uint32_t ate_tx_dcc_ctrl_diff_range   = 0;
    uint32_t ate_tx_dcc_ctrl_cm_range     = 0;
    uint32_t ate_tx_width                 = 0;
    uint32_t ate_tx_ropll_cp_ctl_intg     = 0;
    uint32_t ate_tx_ropll_cp_ctl_prop     = 0;
    uint32_t ate_tx_ropll_rc_filter       = 0;
    uint32_t ate_tx_ropll_v2i_mode        = 0;
    uint32_t ate_tx_ropll_vco_low_freq    = 0;
    uint32_t ate_tx_ropll_postdiv         = 0;
    uint32_t ate_tx_rate                  = 0;
    uint32_t ate_tx_ropll_div16p5_clk_en  = 0;
    uint32_t ate_tx_ropll_125mhz_clk_en   = 0;
    uint32_t ate_tx_term_ctrl             = 0;
    uint32_t ate_tx_dly_cal_en            = 0;
    uint32_t ate_tx_pll_word_clk_freq     = 0;
    uint32_t ate_tx_dig_ropll_div_clk_sel = 0;
    uint32_t ate_tx_dual_cntx_en          = 0;
    uint32_t ate_tx_ropll_bypass          = 0;
    uint32_t ate_tx_ropll_refdiv          = 0;
    uint32_t ate_tx_ropll_refsel          = 0;
    uint32_t ate_tx_ropll_fbdiv           = 0;
    uint32_t ate_tx_ropll_div_clk_en      = 0;
    uint32_t ate_tx_ropll_out_div         = 0;
    uint32_t ate_tx_ropll_word_clk_div_sel= 0;
    uint32_t txX_ropll_word_clk_div_lte   = 0;
    uint32_t ate_tx_fastedge_en           = 0;
  //uint32_t ate_tx_eq_pre                = 0;
  //uint32_t ate_tx_eq_post               = 0;
  //uint32_t ate_tx_eq_main               = 0;
    uint32_t ate_tx_align_wide_xfer_en    = 0;
    uint32_t ate_rx_eq_att_lvl            = 0;
    uint32_t ate_rx_eq_ctle_boost         = 0;
    uint32_t ate_rx_eq_ctle_pole          = 0;
    uint32_t ate_rx_eq_afe_rate           = 0;
    uint32_t ate_rx_eq_vga_gain           = 0;
    uint32_t ate_rx_eq_afe_config         = 0;
    uint32_t ate_rx_eq_dfe_tap1           = 0;
    uint32_t ate_rx_eq_dfe_tap2           = 0;
    uint32_t ate_rx_delta_iq              = 0;
    uint32_t ate_rx_cdr_vco_config        = 0;
    uint32_t ate_rx_dcc_ctrl_diff_range   = 0;
    uint32_t ate_rx_dcc_ctrl_cm_range     = 0;
    uint32_t ate_rx_sigdet_lf_threshold   = 0;
    uint32_t ate_rx_sigdet_hf_threshold   = 0;
    uint32_t ate_rx_misc                  = 0;
    uint32_t ate_rx_term_ctrl             = 0;
    uint32_t ate_rx_width                 = 0;
    uint32_t ate_rx_dig_div_clk_sel       = 0;
    uint32_t ate_rx_div_clk_en            = 0;
    uint32_t ate_rx_div_clk_sel           = 0;
    uint32_t ate_rx_rate                  = 0;
    uint32_t ate_rx_dfe_bypass            = 0;
    uint32_t ate_rx_offcan_cont           = 0;
    uint32_t ate_rx_adapt_cont            = 0;
    uint32_t ate_rx_eq_dfe_float_en       = 0;
    uint32_t ate_rx_div16p5_clk_en        = 0;
    uint32_t ate_rx_125mhz_clk_en         = 0;
    uint32_t ate_rx_cdr_ssc_en            = 0;
    uint32_t ate_rx_sigdet_hf_en          = 0;
    uint32_t ate_rx_sigdet_lfps_filter_en = 0;
    uint32_t ate_rx_term_acdc             = 0;
    uint32_t ate_rx_adapt_sel             = 0;
    uint32_t ate_rx_adapt_mode            = 0;
    uint32_t ate_rx_adapt_en              = 0;

    uint32_t ate_ref_clk_mpllb_div        = 0;
    uint32_t ate_mpllb_fb_clk_div4_en     = 0;
    uint32_t ate_mpllb_multiplier         = 0;
    uint32_t ate_mpllb_tx_clk_div         = 0;
    uint32_t ate_mpllb_word_clk_div       = 0;
    uint32_t ate_mpllb_ssc_en             = 0;
    uint32_t ate_mpllb_ssc_up_spread      = 0;
    uint32_t ate_mpllb_ssc_peak           = 0;
    uint32_t ate_mpllb_ssc_step_size      = 0;
    uint32_t ate_mpllb_frac_en            = 0;
    uint32_t ate_mpllb_frac_quot          = 0;
    uint32_t ate_mpllb_frac_den           = 0;
    uint32_t ate_mpllb_frac_rem           = 0;
    uint32_t ate_refb_lane_clk_en         = 0;
    uint32_t ate_mpllb_init_cal_disable   = 0;
    uint32_t ate_refb_dig_clk_sel         = 0;
    uint32_t ate_refb_dco_ld_val          = 0;
    uint32_t ate_mpllb_bw_high            = 0;
    uint32_t ate_mpllb_bw_low             = 0;
    uint32_t ate_mpllb_bw_threshold       = 0;
    uint32_t ate_mpllb_ctl_buf_bypass     = 0;
    uint32_t ate_mpllb_short_lock_en      = 0;

    //const
    uint32_t ate_mplla_force_en           = 0;
    uint32_t ate_mpllb_force_en           = 0;
    uint32_t ate_pcs_pwr_stable           = 1;
    uint32_t ate_pg_mode_en               = 0;
    uint32_t ate_pg_reset                 = 0;
    uint32_t ate_pma_pwr_stable           = 1;
    uint32_t ate_refa_clk_en              = 1;
    uint32_t ate_refb_clk_en              = 0;
    uint32_t ate_ref_repeat_clk_en        = 1;
    uint32_t ate_rx_pstate                = 2;
    uint32_t ate_rx_invert                = 0;
    uint32_t ate_rx_lpd                   = 0;
    uint32_t ate_rx_term_en               = 1;
    uint32_t ate_tx_pstate                = 2;
    uint32_t ate_tx_clk_rdy               = 1;
    uint32_t ate_tx_invert                = 0;
    uint32_t ate_tx_lpd                   = 0;
    uint32_t ate_tx_mpll_en               = 1;
  
    if(option == 8)
    {
        ate_ref_range                 = 4    ;
        ate_ref_clk_div2_en           = 0    ;
        ate_ref_raw_clk_div2_en       = 0    ;
        ate_ref_clk_mplla_div         = 1    ;
        ate_lane_ref_sel              = 0    ;
        ate_mplla_fb_clk_div4_en      = 0    ;
        ate_mplla_multiplier          = 180  ;
        ate_mplla_tx_clk_div          = 1    ;
        ate_mplla_word_clk_div        = 2    ;
        ate_mplla_ssc_en              = 0    ;
        ate_mplla_ssc_up_spread       = 0    ;
        ate_mplla_ssc_peak            = 0    ;
        ate_mplla_ssc_step_size       = 0    ;
        ate_mplla_frac_en             = 0    ;
        ate_mplla_frac_quot           = 0    ;
        ate_mplla_frac_den            = 0    ;
        ate_mplla_frac_rem            = 0    ;
        ate_refa_lane_clk_en          = 0    ;
        ate_bs_rx_level               = 7    ;
        ate_bs_tx_lowswing            = 0    ;
        ate_bs_rx_bigswing            = 1    ;
        ate_mplla_init_cal_disable    = 0    ;
        ate_refa_dig_clk_sel          = 0    ;
        ate_ref_dco_bypass            = 0    ;
        ate_refa_dco_ld_val           = 123  ;
        ate_ref_dco_target            = 150  ;
        ate_ref_dco_dig_range         = 6    ;
        ate_mplla_bw_threshold        = 75   ;
        ate_mplla_bw_low              = 1599 ;
        ate_mplla_bw_high             = 1599 ;
        ate_mplla_ctl_buf_bypass      = 0    ;
        ate_mplla_short_lock_en       = 0    ;
        ate_rx_term_offset            = 0    ;
        ate_txdn_term_offset          = 0    ;
        ate_txup_term_offset          = 0    ;
        ate_sup_misc                  = 3    ;
        ate_rx_vref_ctrl              = 5    ;
        ate_rx_ref_ld_val             = 17   ;
        ate_rx_vco_ld_val             = 1403 ;
        ate_rx_cdr_ppm_max            = 19   ;
    
        //lan
        ate_tx_misc                   = 0    ;
        ate_tx_dcc_ctrl_diff_range    = 8    ;
        ate_tx_dcc_ctrl_cm_range      = 8    ;
        ate_tx_width                  = 4    ;
        ate_tx_ropll_cp_ctl_intg      = 79   ;
        ate_tx_ropll_cp_ctl_prop      = 79   ;
        ate_tx_ropll_rc_filter        = 4    ;
        ate_tx_ropll_v2i_mode         = 3    ;
        ate_tx_ropll_vco_low_freq     = 2    ;
        ate_tx_ropll_postdiv          = 1    ;
        ate_tx_rate                   = 1    ;
        ate_tx_ropll_div16p5_clk_en   = 1    ;
        ate_tx_ropll_125mhz_clk_en    = 0    ;
        ate_tx_term_ctrl              = 0    ;
        ate_tx_dly_cal_en             = 0    ;
        ate_tx_pll_word_clk_freq      = 6    ;
        ate_tx_dig_ropll_div_clk_sel  = 0    ;
        ate_tx_dual_cntx_en           = 0    ;
        ate_tx_ropll_bypass           = 0    ;
        ate_tx_ropll_refdiv           = 6    ;
        ate_tx_ropll_refsel           = 0    ;
        ate_tx_ropll_fbdiv            = 11   ;
        ate_tx_ropll_div_clk_en       = 0    ;
        ate_tx_ropll_out_div          = 4    ;
        ate_tx_ropll_word_clk_div_sel = 2    ;
        txX_ropll_word_clk_div_lte    = 2    ;
        ate_tx_fastedge_en            = 0    ;
      //ate_tx_eq_pre                 = 0    ;
      //ate_tx_eq_post                = 0    ;
      //ate_tx_eq_main                = 24   ;
        ate_tx_align_wide_xfer_en     = 0    ;
        ate_rx_eq_att_lvl             = 0    ;
        ate_rx_eq_ctle_boost          = 16   ;
        ate_rx_eq_ctle_pole           = 1    ;
        ate_rx_eq_afe_rate            = 5    ;
        ate_rx_eq_vga_gain            = 16   ;
        ate_rx_eq_afe_config          = 332  ;
        ate_rx_eq_dfe_tap1            = 0    ;
        ate_rx_eq_dfe_tap2            = 128  ;
        ate_rx_delta_iq               = 0    ;
        ate_rx_cdr_vco_config         = 1027 ;
        ate_rx_dcc_ctrl_diff_range    = 11   ;
        ate_rx_dcc_ctrl_cm_range      = 11   ;
        ate_rx_sigdet_lf_threshold    = 4    ;
        ate_rx_sigdet_hf_threshold    = 2    ;
        ate_rx_misc                   = 0    ;
        ate_rx_term_ctrl              = 0    ;
        ate_rx_width                  = 4    ;
        ate_rx_dig_div_clk_sel        = 0    ;
        ate_rx_div_clk_en             = 0    ;
        ate_rx_div_clk_sel            = 0    ;
        ate_rx_rate                   = 1    ;
        ate_rx_dfe_bypass             = 0    ;
        ate_rx_offcan_cont            = 1    ;
        ate_rx_adapt_cont             = 1    ;
        ate_rx_eq_dfe_float_en        = 0    ;
        ate_rx_div16p5_clk_en         = 1    ;
        ate_rx_125mhz_clk_en          = 0    ;
        ate_rx_cdr_ssc_en             = 0    ;
        ate_rx_sigdet_hf_en           = 0    ;
        ate_rx_sigdet_lfps_filter_en  = 0    ;
        ate_rx_term_acdc              = 0    ;
        ate_rx_adapt_sel              = 0    ;
        ate_rx_adapt_mode             = 4    ;
        ate_rx_adapt_en               = 1    ;
    }

    #if 0
    if(option == 9)
    {
        ate_ref_range                 = 4    ;
        ate_ref_clk_div2_en           = 0    ;
        ate_ref_raw_clk_div2_en       = 0    ;
        ate_ref_clk_mplla_div         = 1    ;
        ate_lane_ref_sel              = 0    ;
        ate_mplla_fb_clk_div4_en      = 0    ;
        ate_mplla_multiplier          = 180  ;
        ate_mplla_tx_clk_div          = 1    ;
        ate_mplla_word_clk_div        = 2    ;
        ate_mplla_ssc_en              = 0    ;
        ate_mplla_ssc_up_spread       = 0    ;
        ate_mplla_ssc_peak            = 0    ;
        ate_mplla_ssc_step_size       = 0    ;
        ate_mplla_frac_en             = 0    ;
        ate_mplla_frac_quot           = 0    ;
        ate_mplla_frac_den            = 0    ;
        ate_mplla_frac_rem            = 0    ;
        ate_refa_lane_clk_en          = 0    ;
        ate_bs_rx_level               = 7    ;
        ate_bs_tx_lowswing            = 0    ;
        ate_bs_rx_bigswing            = 1    ;
        ate_mplla_init_cal_disable    = 0    ;
        ate_refa_dig_clk_sel          = 0    ;
        ate_ref_dco_bypass            = 0    ;
        ate_refa_dco_ld_val           = 123  ;
        ate_ref_dco_target            = 150  ;
        ate_ref_dco_dig_range         = 6    ;
        ate_mplla_bw_threshold        = 75   ;
        ate_mplla_bw_low              = 1599 ;
        ate_mplla_bw_high             = 1599 ;
        ate_mplla_ctl_buf_bypass      = 0    ;
        ate_mplla_short_lock_en       = 0    ;
        ate_rx_term_offset            = 0    ;
        ate_txdn_term_offset          = 0    ;
        ate_txup_term_offset          = 0    ;
        ate_sup_misc                  = 3    ;
        ate_rx_vref_ctrl              = 5    ;
        ate_rx_ref_ld_val             = 14   ;
        ate_rx_vco_ld_val             = 1386 ;
        ate_rx_cdr_ppm_max            = 18   ;
        
        //lan   
        ate_tx_misc                   = 0    ;
        ate_tx_dcc_ctrl_diff_range    = 8    ;
        ate_tx_dcc_ctrl_cm_range      = 8    ;
        ate_tx_width                  = 4    ;
        ate_tx_ropll_cp_ctl_intg      = 105  ;
        ate_tx_ropll_cp_ctl_prop      = 67   ;
        ate_tx_ropll_rc_filter        = 4    ;
        ate_tx_ropll_v2i_mode         = 3    ;
        ate_tx_ropll_vco_low_freq     = 2    ;
        ate_tx_ropll_postdiv          = 0    ;
        ate_tx_rate                   = 1    ;
        ate_tx_ropll_div16p5_clk_en   = 1    ;
        ate_tx_ropll_125mhz_clk_en    = 0    ;
        ate_tx_term_ctrl              = 0    ;
        ate_tx_dly_cal_en             = 0    ;
        ate_tx_pll_word_clk_freq      = 4    ;
        ate_tx_dig_ropll_div_clk_sel  = 0    ;
        ate_tx_dual_cntx_en           = 0    ;
        ate_tx_ropll_bypass           = 0    ;
        ate_tx_ropll_refdiv           = 5    ;
        ate_tx_ropll_refsel           = 0    ;
        ate_tx_ropll_fbdiv            = 11   ;
        ate_tx_ropll_div_clk_en       = 0    ;
        ate_tx_ropll_out_div          = 4    ;
        ate_tx_ropll_word_clk_div_sel = 2    ;
        txX_ropll_word_clk_div_lte    = 2    ;
        ate_tx_fastedge_en            = 0    ;
        ate_tx_eq_pre                 = 0    ;
        ate_tx_eq_post                = 0    ;
        ate_tx_eq_main                = 24   ;
        ate_tx_align_wide_xfer_en     = 0    ;
        ate_rx_eq_att_lvl             = 0    ;
        ate_rx_eq_ctle_boost          = 16   ;
        ate_rx_eq_ctle_pole           = 1    ;
        ate_rx_eq_afe_rate            = 4    ;
        ate_rx_eq_vga_gain            = 16   ;
        ate_rx_eq_afe_config          = 396  ;
        ate_rx_eq_dfe_tap1            = 10   ;
        ate_rx_eq_dfe_tap2            = 128  ;
        ate_rx_delta_iq               = 6    ;
        ate_rx_cdr_vco_config         = 34   ;
        ate_rx_dcc_ctrl_diff_range    = 11   ;
        ate_rx_dcc_ctrl_cm_range      = 11   ;
        ate_rx_sigdet_lf_threshold    = 4    ;
        ate_rx_sigdet_hf_threshold    = 2    ;
        ate_rx_misc                   = 0    ;
        ate_rx_term_ctrl              = 0    ;
        ate_rx_width                  = 4    ;
        ate_rx_dig_div_clk_sel        = 0    ;
        ate_rx_div_clk_en             = 0    ;
        ate_rx_div_clk_sel            = 0    ;
        ate_rx_rate                   = 1    ;
        ate_rx_dfe_bypass             = 0    ;
        ate_rx_offcan_cont            = 1    ;
        ate_rx_adapt_cont             = 1    ;
        ate_rx_eq_dfe_float_en        = 0    ;
        ate_rx_div16p5_clk_en         = 1    ;
        ate_rx_125mhz_clk_en          = 0    ;
        ate_rx_cdr_ssc_en             = 0    ;
        ate_rx_sigdet_hf_en           = 0    ;
        ate_rx_sigdet_lfps_filter_en  = 0    ;
        ate_rx_term_acdc              = 0    ;
        ate_rx_adapt_sel              = 0    ;
        ate_rx_adapt_mode             = 4    ;
        ate_rx_adapt_en               = 1    ;
    }
    #endif

    if(option == 10)
    {
        ate_ref_range                 = 4   ;
        ate_ref_clk_div2_en           = 0   ;
        ate_ref_raw_clk_div2_en       = 0   ;
        ate_ref_clk_mplla_div         = 1   ;
        ate_lane_ref_sel              = 0   ;
        ate_mplla_fb_clk_div4_en      = 0   ;
        ate_mplla_multiplier          = 180 ;
        ate_mplla_tx_clk_div          = 1   ;
        ate_mplla_word_clk_div        = 2   ;
        ate_mplla_ssc_en              = 0   ;
        ate_mplla_ssc_up_spread       = 0   ;
        ate_mplla_ssc_peak            = 0   ;
        ate_mplla_ssc_step_size       = 0   ;
        ate_mplla_frac_en             = 0   ;
        ate_mplla_frac_quot           = 0   ;
        ate_mplla_frac_den            = 0   ;
        ate_mplla_frac_rem            = 0   ;
        ate_refa_lane_clk_en          = 0   ;
        ate_bs_rx_level               = 7   ;
        ate_bs_tx_lowswing            = 0   ;
        ate_bs_rx_bigswing            = 1   ;
        ate_mplla_init_cal_disable    = 0   ;
        ate_refa_dig_clk_sel          = 0   ;
        ate_ref_dco_bypass            = 0   ;
        ate_refa_dco_ld_val           = 123 ;
        ate_ref_dco_target            = 150 ;
        ate_ref_dco_dig_range         = 6   ;
        ate_mplla_bw_threshold        = 75  ;
        ate_mplla_bw_low              = 1599;
        ate_mplla_bw_high             = 1599;
        ate_mplla_ctl_buf_bypass      = 0   ;
        ate_mplla_short_lock_en       = 0   ;
        ate_rx_term_offset            = 0   ;
        ate_txdn_term_offset          = 0   ;
        ate_txup_term_offset          = 0   ;
        ate_sup_misc                  = 3   ;
        ate_rx_vref_ctrl              = 5   ;
        ate_rx_ref_ld_val             = 14  ;
        ate_rx_vco_ld_val             = 1386;
        ate_rx_cdr_ppm_max            = 18  ;
        //lan 
        ate_tx_misc                   = 0   ;
        ate_tx_dcc_ctrl_diff_range    = 8   ;
        ate_tx_dcc_ctrl_cm_range      = 8   ;
        ate_tx_width                  = 4   ;
        ate_tx_ropll_cp_ctl_intg      = 105 ;
        ate_tx_ropll_cp_ctl_prop      = 67  ;
        ate_tx_ropll_rc_filter        = 4   ;
        ate_tx_ropll_v2i_mode         = 3   ;
        ate_tx_ropll_vco_low_freq     = 2   ;
        ate_tx_ropll_postdiv          = 0   ;
        ate_tx_rate                   = 0   ;
        ate_tx_ropll_div16p5_clk_en   = 1   ;
        ate_tx_ropll_125mhz_clk_en    = 0   ;
        ate_tx_term_ctrl              = 0   ;
        ate_tx_dly_cal_en             = 0   ;
        ate_tx_pll_word_clk_freq      = 6   ;
        ate_tx_dig_ropll_div_clk_sel  = 0   ;
        ate_tx_dual_cntx_en           = 0   ;
        ate_tx_ropll_bypass           = 0   ;
        ate_tx_ropll_refdiv           = 5   ;
        ate_tx_ropll_refsel           = 0   ;
        ate_tx_ropll_fbdiv            = 11  ;
        ate_tx_ropll_div_clk_en       = 0   ;
        ate_tx_ropll_out_div          = 4   ;
        ate_tx_ropll_word_clk_div_sel = 2   ;
        txX_ropll_word_clk_div_lte    = 2   ;
        ate_tx_fastedge_en            = 0   ;
        //ate_tx_eq_pre 			  = 0	;
        //ate_tx_eq_post			  = 0	;
        //ate_tx_eq_main			  = 24	;
        ate_tx_align_wide_xfer_en     = 1   ;
        ate_rx_eq_att_lvl             = 0   ;
        ate_rx_eq_ctle_boost          = 20  ;
        ate_rx_eq_ctle_pole           = 3   ;
        ate_rx_eq_afe_rate			  = 1	;
        ate_rx_eq_vga_gain            = 16  ;
        ate_rx_eq_afe_config		  = (9 << 8) + (m << 6) + (n << 4) + (t << 2) + 0;
        ate_rx_eq_dfe_tap1            = 12  ;
        ate_rx_eq_dfe_tap2            = 128 ;
        ate_rx_delta_iq               = 3   ;
        ate_rx_cdr_vco_config         = 34  ;
        ate_rx_dcc_ctrl_diff_range    = 11  ;
        ate_rx_dcc_ctrl_cm_range      = 11  ;
        ate_rx_sigdet_lf_threshold    = 4   ;
        ate_rx_sigdet_hf_threshold    = 2   ;
        ate_rx_misc                   = 0   ;
        ate_rx_term_ctrl			  = 2	;
        ate_rx_width                  = 4   ;
        ate_rx_dig_div_clk_sel        = 0   ;
        ate_rx_div_clk_en             = 0   ;
        ate_rx_div_clk_sel            = 0   ;
        ate_rx_rate                   = 0   ;
        ate_rx_dfe_bypass             = 0   ;
        ate_rx_offcan_cont            = 1   ;
        ate_rx_adapt_cont             = 1   ;
        ate_rx_eq_dfe_float_en        = 1   ;
        ate_rx_div16p5_clk_en         = 1   ;
        ate_rx_125mhz_clk_en          = 0   ;
        ate_rx_cdr_ssc_en             = 0   ;
        ate_rx_sigdet_hf_en           = 0   ;
        ate_rx_sigdet_lfps_filter_en  = 0   ;
        ate_rx_term_acdc              = 0   ;
        ate_rx_adapt_sel              = 0   ;
        ate_rx_adapt_mode             = 4   ;
        ate_rx_adapt_en				  = 0	;
    }

    ate_tx_term_ctrl               = ate_tx_term_ctrl + 8;
    ate_rx_term_ctrl               = ate_rx_term_ctrl + 8;
    ate_ref_clk_mpllb_div          = ate_ref_clk_mplla_div;
    ate_mpllb_fb_clk_div4_en       = ate_mplla_fb_clk_div4_en;
    ate_mpllb_multiplier           = ate_mplla_multiplier;
    ate_mpllb_tx_clk_div           = ate_mplla_tx_clk_div;
    ate_mpllb_word_clk_div         = ate_mplla_word_clk_div;
    ate_mpllb_ssc_en               = ate_mplla_ssc_en;
    ate_mpllb_ssc_up_spread        = ate_mplla_ssc_up_spread;
    ate_mpllb_ssc_peak             = ate_mplla_ssc_peak;
    ate_mpllb_ssc_step_size        = ate_mplla_ssc_step_size;
    ate_mpllb_frac_en              = ate_mplla_frac_en;
    ate_mpllb_frac_quot            = ate_mplla_frac_quot;
    ate_mpllb_frac_den             = ate_mplla_frac_den;
    ate_mpllb_frac_rem             = ate_mplla_frac_den;
    ate_refb_lane_clk_en           = ate_refa_lane_clk_en;
    ate_mpllb_init_cal_disable     = ate_mplla_init_cal_disable;
    ate_refb_dig_clk_sel           = ate_refa_dig_clk_sel;
    ate_refb_dco_ld_val            = ate_refa_dco_ld_val;
    ate_mpllb_bw_high              = ate_mplla_bw_high;
    ate_mpllb_bw_low               = ate_mplla_bw_low;
    ate_mpllb_bw_threshold         = ate_mplla_bw_threshold;
    ate_mpllb_ctl_buf_bypass       = ate_mplla_ctl_buf_bypass;
    ate_mpllb_short_lock_en        = ate_mplla_short_lock_en;


    do_write(&JECS_PHY_BS_CTRL,                                 BIT4 + (7 << 6));
    do_write(&JECS_PMA1_REF_CLK_CTRL,                           0x10 + ate_ref_repeat_clk_en + (ate_ref_clk_mplla_div << 8) + (ate_ref_clk_mpllb_div << 12));
    do_write(&JECS_PMA1_REFB_CLK_CTRL2,                         (ate_ref_raw_clk_div2_en << 8) + (ate_ref_range << 4) + ate_refb_lane_clk_en);
    do_write(&JECS_PMA1_REFA_CLK_CTRL2,                         (ate_ref_raw_clk_div2_en << 8) + (ate_ref_range << 4) + ate_refa_lane_clk_en);
    do_write(&JECS_PMA1_BROADCAST_LANE_REFCLK_SEL,              ate_lane_ref_sel);
    do_write(&JECS_PMA1_MPLLA_PARAM6,                           (ate_mplla_fb_clk_div4_en << 4));
    do_write(&JECS_PMA1_MPLLB_PARAM6,                           (ate_mpllb_fb_clk_div4_en << 4));
    do_write(&JECS_PMA1_MPLLA_PARAM5,                           (ate_mplla_tx_clk_div << 4) + ate_mplla_short_lock_en + (ate_mplla_word_clk_div << 8));
    do_write(&JECS_PMA1_MPLLB_PARAM5,                           (ate_mpllb_tx_clk_div << 4) + ate_mpllb_short_lock_en + (ate_mpllb_word_clk_div << 8));
    do_write(&JECS_PMA1_REFA_CLK_CTRL1,                         0x100 + (ate_refa_clk_en << 4) + ate_ref_clk_div2_en);
    do_write(&JECS_PMA1_REFB_CLK_CTRL1,                         0x100 + (ate_refb_clk_en << 4) + ate_ref_clk_div2_en);
    do_write(&JECS_PMA1_MPLLA_SSC_CTRL2,                        ate_mplla_ssc_peak);
    do_write(&JECS_PMA1_MPLLB_SSC_CTRL2,                        ate_mpllb_ssc_peak);
    do_write(&JECS_PMA1_MPLLA_SSC_CTRL5,                        ate_mplla_ssc_step_size);
    do_write(&JECS_PMA1_MPLLA_SSC_CTRL5,                        ate_mpllb_ssc_step_size);
    do_write(&JECS_PMA1_MPLLA_FRAC_CTRL1,                       ate_mplla_frac_den);
    do_write(&JECS_PMA1_MPLLB_FRAC_CTRL1,                       ate_mpllb_frac_den);
    do_write(&JECS_PMA1_MPLLA_FRAC_CTRL2,                       ate_mplla_frac_en);
    do_write(&JECS_PMA1_MPLLB_FRAC_CTRL2,                       ate_mpllb_frac_en);
    do_write(&JECS_PMA1_MPLLA_FRAC_CTRL3,                       ate_mplla_frac_quot);
    do_write(&JECS_PMA1_MPLLB_FRAC_CTRL3,                       ate_mpllb_frac_quot);
    do_write(&JECS_PMA1_MPLLA_FRAC_CTRL4,                       ate_mplla_frac_rem);
    do_write(&JECS_PMA1_MPLLB_FRAC_CTRL4,                       ate_mpllb_frac_rem);
    do_write(&JECS_PMA1_MPLLA_PARAM3,                           (ate_mplla_ctl_buf_bypass << 8) + ate_mplla_bw_threshold);
    do_write(&JECS_PMA1_MPLLB_PARAM3,                           (ate_mpllb_ctl_buf_bypass << 8) + ate_mpllb_bw_threshold);
    do_write(&JECS_PMA1_MPLLA_SSC_CTRL1,                        ate_mplla_ssc_en);
    do_write(&JECS_PMA1_MPLLB_SSC_CTRL1,                        ate_mpllb_ssc_en);
    do_write(&JECS_PMA1_MPLLA_SSC_CTRL4,                        ate_mplla_ssc_up_spread);
    do_write(&JECS_PMA1_MPLLB_SSC_CTRL4,                        ate_mpllb_ssc_up_spread);
    do_write(&JECS_PMA1_SUP_MISC,                               ate_sup_misc);
    do_write(&JECS_PMA1_MPLLA_PARAM4,                           ate_mplla_multiplier + (ate_mplla_init_cal_disable << 12));
    do_write(&JECS_PMA1_MPLLB_PARAM4,                           ate_mpllb_multiplier + (ate_mpllb_init_cal_disable << 12));
    do_write(&JECS_PMA1_RTUNE_CTRL1,                            ate_rx_term_offset);
    do_write(&JECS_PMA1_RTUNE_CTRL2,                            ate_txdn_term_offset);
    do_write(&JECS_PMA1_RTUNE_CTRL3,                            ate_txup_term_offset);
    do_write(&JECS_PMA1_MPLLA_PARAM1,                           ate_mplla_bw_high);
    do_write(&JECS_PMA1_MPLLB_PARAM1,                           ate_mpllb_bw_high);
    do_write(&JECS_PMA1_MPLLA_PARAM2,                           ate_mplla_bw_low);
    do_write(&JECS_PMA1_MPLLB_PARAM2,                           ate_mpllb_bw_low);
    do_write(&JECS_PMA1_RX_BIAS_CURRENT_CTRL,                   ate_rx_vref_ctrl+BIT8);
    do_write(&JECS_PMA1_MPLLA_FORCE_EN,                         ate_mplla_force_en);
    do_write(&JECS_PMA1_MPLLB_FORCE_EN,                         ate_mpllb_force_en);
    do_write(&JECS_PMA1_POWER_GATING_SIGNAL1,                   ate_pcs_pwr_stable + (ate_pg_mode_en << 4) + (ate_pg_reset << 8) + (ate_pma_pwr_stable << 12));
    do_write(&JECS_PMA1_BROADCAST_RECEIVER_REQ_PARAM7,          ate_rx_pstate + (ate_rx_rate << 4) + (ate_rx_ref_ld_val << 8));
    do_write(&JECS_PMA1_BROADCAST_RECEIVER_REQ_PARAM8,          ate_rx_vco_ld_val);
    do_write(&JECS_PMA1_BROADCAST_RECEIVER_DATAPATH_SETTING1,   ate_rx_cdr_ppm_max + (ate_rx_cdr_ssc_en << 8) + (ate_rx_invert << 12));
    do_write(&JECS_PMA1_BROADCAST_TRANSMITTER_CONTROL2,         ate_tx_misc + (ate_tx_term_ctrl << 8));
    do_write(&JECS_PMA1_BROADCAST_TRANSMITTER_REQ_PARAM2,       (ate_tx_width << 4) + ate_tx_rate);
    do_write(&JECS_PMA1_BROADCAST_ETH_CLK_CTRL,                 (ate_tx_pll_word_clk_freq << 12) + (ate_tx_ropll_div16p5_clk_en << 8) + (ate_tx_ropll_125mhz_clk_en << 4) + ate_rx_125mhz_clk_en);
    do_write(&JECS_PMA1_BROADCAST_TX_DIV_CLK_CTRL,              ate_tx_dig_ropll_div_clk_sel);
    do_write(&JECS_PMA1_BROADCAST_RECEIVER_CONTROL,             (ate_rx_term_ctrl << 4) + ate_rx_term_acdc + (ate_rx_term_en << 8));
    do_write(&JECS_PMA1_BROADCAST_RECEIVER_REQ_PARAM4,          (ate_rx_eq_ctle_pole << 8) + ate_rx_eq_ctle_boost);
    do_write(&JECS_PMA1_BROADCAST_RECEIVER_REQ_PARAM3,          (ate_rx_eq_afe_rate << 4) + ate_rx_dfe_bypass +  (ate_rx_eq_att_lvl << 8));
    do_write(&JECS_PMA1_BROADCAST_RECEIVER_REQ_PARAM9,          ate_rx_width);
    do_write(&JECS_PMA1_BROADCAST_RX_EQ_CTRL2,                  (ate_rx_eq_vga_gain << 8) + ate_rx_eq_dfe_tap2);
    do_write(&JECS_PMA1_BROADCAST_RX_EQ_CTRL1,                  (ate_rx_eq_dfe_float_en << 12) + ate_rx_eq_afe_config);
    do_write(&JECS_PMA1_BROADCAST_RECEIVER_REQ_PARAM2,          ate_rx_cdr_vco_config + (ate_rx_delta_iq << 12));
    do_write(&JECS_PMA1_BROADCAST_RX_ADAPT_CTRL,                ate_rx_adapt_sel );
    do_write(&JECS_PMA1_BROADCAST_RECEIVER_ADAPT_SETTING,       (ate_rx_adapt_mode << 4) + ate_rx_adapt_cont + (ate_rx_offcan_cont << 8));
    
    if(option == 8)
    {
        do_write(&JECS_PMA1_BROADCAST_TRANSMITTER_EQ1,          ate_tx_eq_main[4]);
        do_write(&JECS_PMA1_BROADCAST_TRANSMITTER_EQ2,          ate_tx_eq_post[4]);
        do_write(&JECS_PMA1_BROADCAST_TRANSMITTER_EQ3,          ate_tx_eq_pre[4]);
    }else if(option == 10)
    {
        do_write(&JECS_PMA1_BROADCAST_TRANSMITTER_EQ1,          ate_tx_eq_main[4]);
        do_write(&JECS_PMA1_BROADCAST_TRANSMITTER_EQ2,          ate_tx_eq_post[4]);
        do_write(&JECS_PMA1_BROADCAST_TRANSMITTER_EQ3,          ate_tx_eq_pre[4]);
    }

    do_write(&JECS_PMA1_BROADCAST_RECEIVER_REQ_PARAM6,          (ate_rx_misc << 8));
    do_write(&JECS_PMA1_BROADCAST_RX_DCC_CTRL,                  ate_rx_dcc_ctrl_cm_range + (ate_rx_dcc_ctrl_diff_range << 4));
    do_write(&JECS_PMA1_BROADCAST_RECEIVER_REQ_PARAM5,          (ate_rx_lpd << 12) + ate_rx_eq_dfe_tap1);
    do_write(&JECS_PMA1_BROADCAST_RECEIVER_DATAPATH_SETTING2,   (ate_rx_sigdet_hf_en << 4) + (ate_rx_sigdet_lf_threshold << 12) + (ate_rx_sigdet_hf_threshold << 8) + ate_rx_div16p5_clk_en);
    do_write(&JECS_PMA1_BROADCAST_RECEIVER_DATAPATH_SETTING3,   ate_rx_sigdet_lfps_filter_en);
    do_write(&JECS_PMA1_BROADCAST_TRANS_REQ_CTRL2,              ate_tx_ropll_cp_ctl_intg + (ate_tx_ropll_cp_ctl_prop << 8) + (ate_tx_ropll_bypass << 7));
    do_write(&JECS_PMA1_BROADCAST_TRANS_REQ_CTRL4,              ate_tx_ropll_postdiv + (ate_tx_ropll_rc_filter << 4) + (ate_tx_ropll_refdiv << 8) + (ate_tx_ropll_refsel << 12) + (ate_tx_ropll_v2i_mode << 14));
    do_write(&JECS_PMA1_BROADCAST_TRANS_REQ_CTRL3,              ate_tx_ropll_fbdiv + (ate_tx_ropll_out_div << 8) + (ate_tx_ropll_div_clk_en << 7));
    do_write(&JECS_PMA1_BROADCAST_TRANS_REQ_CTRL5,              ate_tx_ropll_vco_low_freq + (ate_tx_ropll_word_clk_div_sel << 4) + 0x100);
    do_write(&JECS_PMA1_BROADCAST_TRANSMITTER_REQ_PARAM1,       (ate_tx_pstate << 8) + ate_tx_lpd+(ate_tx_mpll_en << 4));
    do_write(&JECS_PMA1_BROADCAST_TRANSMITTER_DATAPATH_CLKRDY,  ate_tx_clk_rdy);
    do_write(&JECS_PMA1_BROADCAST_TRANSMITTER_DATAPATH_SETTING, ate_tx_align_wide_xfer_en + (ate_tx_invert << 4));
    do_write(&JECS_PMA1_BROADCAST_TRANS_REQ_CTRL1,              ate_tx_dcc_ctrl_cm_range + (ate_tx_dcc_ctrl_diff_range << 4) + (ate_tx_fastedge_en << 8));
    do_write(&JECS_PMA1_BROADCAST_CONTEXT_RESTORE_CTRL3,        ate_tx_dual_cntx_en);
    do_write(&JECS_PMA1_BROADCAST_TRANS_INTERFACE_CTRL,         ate_tx_dly_cal_en);
    do_write(&JECS_PMA1_BROADCAST_RX_DIV_CLK_CTRL,              ate_rx_dig_div_clk_sel + (ate_rx_div_clk_en << 4) + (ate_rx_div_clk_sel << 8));
}


/**********************************************************************************************************************
*  function: cpri_pma_cfg
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
extern DDR0 uint32_t pma_fw[16384*2];
void cpri_pma_cfg(uint32_t option)
{
    uint32_t i =0;
    uint32_t temp =0;   

    /* pcs protocol select */
    temp = do_read_volatile(&JECS_CTRL_PROTOCOL_SEL);
    temp |= BIT2;
    __ucps2_synch(0);
    do_write(&JECS_CTRL_PROTOCOL_SEL,  temp); 
    do_write(&JECS_PMA1_CPRI_PCS_STATUS_CTRL, 0); 

    /* pcs bit rev */
    if(option <= 7)
    {
        do_write(&JECS_PMA1_PCS_BIT_REV_CTRL, 0x211); 
    }
    else
    {
        do_write(&JECS_PMA1_PCS_BIT_REV_CTRL, 0x111); 
    }

    do_write(&JECS_PMA1_CPRI_SIGDET, (BIT4|BIT0)); 

    ucp_nop(400);   

    if(option == 7)
    {   
        do_write(&JECS_PMA1_TX_CLK_SEL, 0); 
    }
    else if((option == 3)|(option == 9))
    {
        do_write(&JECS_PMA1_TX_CLK_SEL, 0x2); 
    }
    else if(option == 2)
    {
        do_write(&JECS_PMA1_TX_CLK_SEL, 0x3); 
    }
    else
    {
        do_write(&JECS_PMA1_TX_CLK_SEL, 0x1); 
    }

    ucp_nop(400);   

    cpri_pma_common_cfg(option);
    ucp_nop(400);   

    do_write(&JECS_PMA1_PHY_RESET, 0x1); 
    ucp_nop(400);   
    do_write(&JECS_PMA1_SRAM_CTRL, 0x101); 
    ucp_nop(400);
    do_write(&JECS_PMA1_PHY_RESET, 0x0);   
    __ucps2_synch(0);

    while((do_read_volatile(&JECS_PMA1_SRAM_STATUS) & BIT0)!= 1){}  

    for(i = 0; i < 16384; i++)
    {
        temp = do_read_volatile(&pma_fw[i]);
        __ucps2_synch(0);
        do_write((volatile uint32_t *)(JECS_PMA1_CFG+0xc000*4+i*4),temp);
        __ucps2_synch(0);
    }   
    do_write((volatile uint32_t *)(JECS_PMA1_CFG + 0x101*4), do_read_volatile((volatile uint32_t *)(JECS_PMA1_CFG + 0x101*4))|BIT1); 
    __ucps2_synch(0);

    for(i = 0; i < 16384; i++)
    {
        temp = do_read_volatile(&pma_fw[i + 16384]);
        __ucps2_synch(0);
        do_write((volatile uint32_t *)(JECS_PMA1_CFG + 0xc000*4 + i*4), temp);
        __ucps2_synch(0);
    }

    do_write(&JECS_PMA1_SRAM_CTRL, 0x1101); 
    __ucps2_synch(0);

}





/**********************************************************************************************************************
*  function: cpri_pma_tx
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
void cpri_pma_tx(void)
{
    /* disable tx and rx rst */
    do_write(&JECS_PMA1_LANE0_TRANSMITTER_CONTROL1,  0x0);
    do_write(&JECS_PMA1_LANE0_RECEIVER_RESET,        0x0);
    do_write(&JECS_PMA1_LANE1_TRANSMITTER_CONTROL1,  0x0);
    do_write(&JECS_PMA1_LANE1_RECEIVER_RESET,        0x0);
    do_write(&JECS_PMA1_LANE2_TRANSMITTER_CONTROL1,  0x0);
    do_write(&JECS_PMA1_LANE2_RECEIVER_RESET,        0x0);
    do_write(&JECS_PMA1_LANE3_TRANSMITTER_CONTROL1,  0x0);
    do_write(&JECS_PMA1_LANE3_RECEIVER_RESET,        0x0);
    ucp_nop(400);

    /* wait ack done */
    __ucps2_synch(0);
    while(do_read_volatile(&JECS_PMA1_LANE0_RECEIVER_REQ_ACK)&BIT0);

    __ucps2_synch(0);
    while(do_read_volatile(&JECS_PMA1_LANE0_TRANSMITTER_REQ_ACK)&BIT0);

    /* enable tx and rx req */
    do_write(&JECS_PMA1_LANE0_TRANSMITTER_REQ,  0x1);

    /* wait ack and disable req */
    __ucps2_synch(0);
    while(!(do_read_volatile(&JECS_PMA1_LANE0_TRANSMITTER_REQ_ACK)&BIT0));
    do_write(&JECS_PMA1_LANE0_TRANSMITTER_REQ,  0x0);
    do_write(&JECS_PMA1_LANE0_RECEIVER_REQ,     0x0);

    /* wait ack done */
    __ucps2_synch(0);
    while(do_read_volatile(&JECS_PMA1_LANE0_TRANSMITTER_REQ_ACK)&BIT0);

    __ucps2_synch(0);
    do_write(&JECS_PMA1_LANE0_TRANSMITTER_REQ_PARAM1, do_read_volatile(&JECS_PMA1_LANE0_TRANSMITTER_REQ_PARAM1)&0xFF);

    do_write(&JECS_PMA1_LANE0_TRANSMITTER_REQ,  0x1);

    __ucps2_synch(f_SM);
    while(!(do_read_volatile(&JECS_PMA1_LANE0_TRANSMITTER_REQ_ACK)&BIT0));

    do_write(&JECS_PMA1_LANE0_TRANSMITTER_REQ,  0x00);
    do_write(&JECS_PMA1_LANE0_RECEIVER_REQ,     0x0);

    /* wait ack done */
    __ucps2_synch(f_SM);
    while(do_read_volatile(&JECS_PMA1_LANE0_TRANSMITTER_REQ_ACK)&BIT0);

    /* enable data */
    do_write(&JECS_PMA1_LANE0_TRANSMITTER_DATAPATH_EN,  0x1);

    /* p0 pstate */
    __ucps2_synch(f_SM);
    while(do_read_volatile(&JECS_PMA1_LANE0_TRANS_PLL_STATE) == 0){}
}





/**********************************************************************************************************************
*  function: cpri_pma_rx
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
void cpri_pma_rx(void)
{
    /* enable tx and rx req */
    do_write(&JECS_PMA1_LANE0_RECEIVER_REQ, 0x1); 

    /* wait ack and disable req */
    while(!(do_read_volatile(&JECS_PMA1_LANE0_RECEIVER_REQ_ACK)&BIT0));   
    do_write(&JECS_PMA1_LANE0_TRANSMITTER_REQ,  0x0); 
    do_write(&JECS_PMA1_LANE0_RECEIVER_REQ,     0x0); 

    /* wait ack done */
    while(do_read_volatile(&JECS_PMA1_LANE0_RECEIVER_REQ_ACK)&BIT0); 

    /* p0 pstate */
    do_write(&JECS_PMA1_LANE0_RECEIVER_REQ_PARAM7, do_read_volatile(&JECS_PMA1_LANE0_RECEIVER_REQ_PARAM7)&0xFFF0); 
    do_write(&JECS_PMA1_LANE0_RECEIVER_REQ, 0x1); 

    /* wait ack and disable req */
    while(!(do_read_volatile(&JECS_PMA1_LANE0_RECEIVER_REQ_ACK)&BIT0)); 
    do_write(&JECS_PMA1_LANE0_TRANSMITTER_REQ,  0x0); 
    do_write(&JECS_PMA1_LANE0_RECEIVER_REQ,     0x0); 

    /* wait ack done */
    while(do_read_volatile(&JECS_PMA1_LANE0_RECEIVER_REQ_ACK)&BIT0);   

    /* enable data */
    do_write(&JECS_PMA1_LANE0_RECEIVER_DATAPATH_EN, 0x1);

    /* wait rx valid */
    while(do_read_volatile(&JECS_PMA1_LANE0_RX_VALID_PHY) == 0);


}




/**********************************************************************************************************************
*  function: cpri_pma_eq
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
void cpri_pma_eq(void)
{
    uint32_t link_fom  = 0;
    uint32_t adapt_cnt = 0;
    do
    {
        do_write(&JECS_PMA1_LANE0_RECEIVER_DATAPATH_EN, BIT4); 

        /* assert rx_adapt_in_prog to the phy */
        do_write(&JECS_PMA1_LANE0_RECEIVER_ADAPT_REQ, BIT4); 

        /* toggle rx_reset to the phy */
        do_write(&JECS_PMA1_LANE0_RECEIVER_RESET, do_read_volatile(&JECS_PMA1_LANE0_RECEIVER_RESET) | BIT4); 
        do_write(&JECS_PMA1_LANE0_RECEIVER_RESET, do_read_volatile(&JECS_PMA1_LANE0_RECEIVER_RESET) & (~BIT4)); 
        __ucps2_synch(f_SM);

        /* wait for de-assertion of rx_ack from the phy */
        while((do_read_volatile(&JECS_PMA1_LANE0_RECEIVER_REQ_ACK) & 0x1 )!= 0); 

        /* assert rx_data_en to the phy */
        do_write(&JECS_PMA1_LANE0_RECEIVER_DATAPATH_EN, BIT4|BIT0); 
        __ucps2_synch(f_SM);

        /* Wait for assertion of rx_valid from the phy */
        while((do_read_volatile(&JECS_PMA1_LANE0_RX_VALID_PHY) & BIT0 )!= 0x1); 

        /* perform an rx adaptation request and assert rx_adapt_req */
        do_write(&JECS_PMA1_LANE0_RECEIVER_ADAPT_REQ, BIT4|BIT0);		
        __ucps2_synch(f_SM);

        /* wait for assertion of rx_adapt_ack from the phy */
        while((do_read_volatile(&JECS_PMA1_LANE0_RECEIVER_ADAPT_REQ_ACK)&BIT0) != BIT0);

        /* de-assert rx_adapt_req to the phy */
        do_write(&JECS_PMA1_LANE0_RECEIVER_ADAPT_REQ, BIT4);

        /* de-assert rx_adapt_in_prog to the phy */
        do_write(&JECS_PMA1_LANE0_RECEIVER_ADAPT_REQ, 0);

        link_fom = (do_read_volatile(&JECS_PMA1_LANE0_RECEIVER_ADAPT_REQ_ACK) >> 8)&0xFF;
        adapt_cnt++;
    }while(link_fom < 0x80);
    
    debug_write((DBG_DDR_IDX_CPRI_BASE + 0), (adapt_cnt<<16) | link_fom);    // 0xB7E28000
}





/**********************************************************************************************************************
*  function: cpri_link_init
*  description: 
*  input : 
*         <name>                     <type>                      <desc>
*         option                     uint32_t                    cpri line speed option
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
void cpri_link_init(uint32_t option)
{

    do_write(&JECS_CTRL_PROTOCOL_SEL, do_read_volatile(&JECS_CTRL_PROTOCOL_SEL)|BIT4);

    #if 0
    /************************************
    配置光模块使能：PD16/AP_GPIO0B29
    低：光模块发射打开
    高：光模块发射关闭
    *****************************************/
    //do_write((GPIO0_A29_PINMUX_REG_ADDR+0xc), (do_read_volatile((GPIO0_A29_PINMUX_REG_ADDR+0xc))|0x3));
    do_write((0x0445818c), (do_read_volatile(0x0445818c)|0x3));//pinmux
    __ucps2_synch(f_SM);
    do_write((0x04450010), (do_read_volatile(0x04450010)|(BIT29))); //dir:1:out;0:in
    __ucps2_synch(f_SM);
    do_write((0x0445000c), (do_read_volatile(0x0445000c)&(~(BIT29)))); // data:0
    __ucps2_synch(f_SM);
    delay_us(1000);
    #endif
	
    cpri_pma_reset();
    cpri_clk_init(option);
    cpri_pma_cfg(option);
    cpri_pma_tx();

    do_write(SERDES_INIT_FLAG_ADDR, 1); // cpri serdes clk init finished

    do_write(&JECS_CRG_CPRI0_RST_CTRL, do_read_volatile(&JECS_CRG_CPRI0_RST_CTRL) | BIT24);
    do_write(&JECS_CRG_CPRI1_RST_CTRL, do_read_volatile(&JECS_CRG_CPRI1_RST_CTRL) | BIT24);
    do_write(&JECS_CRG_CPRI2_RST_CTRL, do_read_volatile(&JECS_CRG_CPRI2_RST_CTRL) | BIT24);
    do_write(&JECS_CRG_CPRI3_RST_CTRL, do_read_volatile(&JECS_CRG_CPRI3_RST_CTRL) | BIT24);
    do_write(&JECS_CRG_CPRI4_RST_CTRL, do_read_volatile(&JECS_CRG_CPRI4_RST_CTRL) | BIT24);
    do_write(&JECS_CRG_CPRI5_RST_CTRL, do_read_volatile(&JECS_CRG_CPRI5_RST_CTRL) | BIT24);
    do_write(&JECS_CRG_CPRI6_RST_CTRL, do_read_volatile(&JECS_CRG_CPRI6_RST_CTRL) | BIT24);
    do_write(&ECPRI_RST_CFG_REG,       do_read_volatile(&ECPRI_RST_CFG_REG)       | BIT24);
    do_write(&AUX_CTRL_REG,            do_read_volatile(&AUX_CTRL_REG) & (~BIT2));

    ucp_nop(400);

    if(option == 8)          // option8
    {
        do_write(&CPRI_PCS_64B66B_CFG,  0);
        do_write(&CPRI_FRAME_RX_CFG,    0x14|BIT8);

        core_clk_normal_to_xtal();

        do_write(&CPRI_PCS_ADDR_CFG,    0);
        do_write(&CPRI_PCS_DATA_TX_CFG, BIT13|BIT6);
        do_write(&CPRI_PCS_CTRL_CFG,    0x2);

        __ucps2_synch(f_SM);
        do_write(&CPRI_PCS_ADDR_CFG,    0x1C);
        do_write(&CPRI_PCS_DATA_TX_CFG, 0);
        do_write(&CPRI_PCS_CTRL_CFG,    0x2);

        __ucps2_synch(f_SM);
        do_write(&CPRI_PCS_ADDR_CFG,    0xB8);
        do_write(&CPRI_PCS_DATA_TX_CFG, 0x20<<7|BIT14);
        do_write(&CPRI_PCS_CTRL_CFG,    0x2);

        __ucps2_synch(f_SM);
        do_write(&CPRI_PCS_ADDR_CFG,    0xBC);
        do_write(&CPRI_PCS_DATA_TX_CFG, 0x20<<7|BIT14);
        do_write(&CPRI_PCS_CTRL_CFG,    0x2);
        __ucps2_synch(f_SM);

        core_clk_xtal_to_normal();

        do_write(&CPRI_FRAME_TX_CM_CFG,    0x114); 
        do_write(&CPRI_FRAME_TX_PROT_VER,  0x1); 
        do_write(&CPRI_FRAME_TX_BFN_INIT,  0x10000); 
        do_write(&CPRI_FRAME_TX_SCRAMBLER, 0x0); 
        do_write(&CPRI_FRAME_TX_CFG,       BIT2|BIT1); 
    }
    else if(option == 10)       // option10
    {
        do_write(&CPRI_PCS_64B66B_CFG,  0);
        do_write(&CPRI_FRAME_RX_CFG,    0x30|BIT8);

        core_clk_normal_to_xtal();

        do_write(&CPRI_PCS_ADDR_CFG,    0);
        do_write(&CPRI_PCS_DATA_TX_CFG, BIT13|BIT6);
        do_write(&CPRI_PCS_CTRL_CFG,    0x2);

        __ucps2_synch(f_SM);
        do_write(&CPRI_PCS_ADDR_CFG,    0x1C);
        do_write(&CPRI_PCS_DATA_TX_CFG, 0);
        do_write(&CPRI_PCS_CTRL_CFG,    0x2);

        __ucps2_synch(f_SM);
        do_write(&CPRI_PCS_ADDR_CFG,    0xB8);
        do_write(&CPRI_PCS_DATA_TX_CFG, 0x20<<7|BIT14);
        do_write(&CPRI_PCS_CTRL_CFG,    0x2);

        __ucps2_synch(f_SM);
        do_write(&CPRI_PCS_ADDR_CFG,    0xBC);
        do_write(&CPRI_PCS_DATA_TX_CFG, 0x20<<7|BIT14);
        do_write(&CPRI_PCS_CTRL_CFG,    0x2);
        __ucps2_synch(f_SM);

        #if 1
        do_write(&CPRI_PCS_ADDR_CFG,    0x320);
        do_write(&CPRI_PCS_DATA_TX_CFG, BIT2);      // enable rs-fec
        do_write(&CPRI_PCS_CTRL_CFG,    0x2);
        __ucps2_synch(f_SM);
        #endif

        core_clk_xtal_to_normal();

        do_write(&CPRI_FRAME_TX_CM_CFG,    0x114); 
        do_write(&CPRI_FRAME_TX_PROT_VER,  0x1); 
        do_write(&CPRI_FRAME_TX_BFN_INIT,  0x10000); 
        do_write(&CPRI_FRAME_TX_SCRAMBLER, 0x0); 
        do_write(&CPRI_FRAME_TX_CFG,       BIT2|BIT1); 
    }

    cpri_pma_rx();
    cpri_pma_eq();

    do_write(&ALARM_FLAG, 0x0);
}


/**********************************************************************************************************************
*  function: cpri_pcs_reset
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
void cpri_pcs_reset(void)
{
    core_clk_normal_to_xtal();

    do_write(&CPRI_PCS_ADDR_CFG,    0); 
    do_write(&CPRI_PCS_DATA_TX_CFG, BIT15|BIT13|BIT6); 
    do_write(&CPRI_PCS_CTRL_CFG,    0x2);

    __ucps2_synch(f_SM);
    do_write(&CPRI_PCS_ADDR_CFG,    0); 
    do_write(&CPRI_PCS_DATA_TX_CFG, BIT13|BIT6); 
    do_write(&CPRI_PCS_CTRL_CFG,    0x2); 

    __ucps2_synch(f_SM);
    do_write(&CPRI_PCS_ADDR_CFG,    0x1C); 
    do_write(&CPRI_PCS_DATA_TX_CFG, 0); 
    do_write(&CPRI_PCS_CTRL_CFG,    0x2); 

    __ucps2_synch(f_SM);
    do_write(&CPRI_PCS_ADDR_CFG,    0xB8); 
    do_write(&CPRI_PCS_DATA_TX_CFG, 0x20<<7|BIT14); 
    do_write(&CPRI_PCS_CTRL_CFG,    0x2); 

    __ucps2_synch(f_SM);
    do_write(&CPRI_PCS_ADDR_CFG,    0xBC); 
    do_write(&CPRI_PCS_DATA_TX_CFG, 0x20<<7|BIT14); 
    do_write(&CPRI_PCS_CTRL_CFG,    0x2); 

    __ucps2_synch(f_SM);

    core_clk_xtal_to_normal();
}


/**********************************************************************************************************************
*  function: cpri_link_monitor
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
extern  stCpriIntStat   gCpriIntStatus;
        uint32_t        cpri_async_flag  = 0;
        uint32_t        cpri_monitor_cnt = 0;
        uint32_t        cpri_resync_cnt  = 0;
        uint32_t        cpri_pcs_thr_cnt = 0;
void cpri_link_monitor(void)
{
    uint32_t link_fom  = 0;
    uint32_t link_stat = 0;


    cpri_monitor_cnt++;

    link_stat = do_read_volatile(&CPRI_FRAME_RX_STAT);
    if(link_stat != 0x1E)
    {
        if(cpri_async_flag == 0)
        {
            cpri_async_flag = 1;          
            gCpriIntStatus.cpriSyncFlag = 0;
            UCP_API_CPRI_CSU_STOP();
            cpri_pma_rx();
        }
    }

    if(cpri_async_flag == 1)
    {
        cpri_resync_cnt++;
        cpri_pma_eq();

        link_stat = do_read_volatile(&CPRI_FRAME_RX_STAT);
        link_fom  = (do_read_volatile(&JECS_PMA1_LANE0_RECEIVER_ADAPT_REQ_ACK) >> 8)&0xFF;
        if(link_fom > 0x80)
        {
            if(link_stat == 0x1E)
            {
                cpri_async_flag  = 0;
                cpri_pcs_thr_cnt = 0;
                set_cpri_tmr_ctrl();
            }else
            {
                cpri_pcs_thr_cnt++;
                if(cpri_pcs_thr_cnt%100 == 0)
                {
                    cpri_pcs_reset();
                }
            }
        }
        
        debug_write((DBG_DDR_IDX_CPRI_BASE + 4), link_fom);
        debug_write((DBG_DDR_IDX_CPRI_BASE + 5), cpri_resync_cnt);
        debug_write((DBG_DDR_IDX_CPRI_BASE + 6), cpri_pcs_thr_cnt);
    }

    debug_write((DBG_DDR_IDX_CPRI_BASE + 7), cpri_monitor_cnt);
}