/**********************************************************************************************************************
*  Copyright (C), 2022-2026, SMARTLOGIC TECHNOLOGY LTD.
*  File Name: cpri_mode.s.c
*  Create Date: 23/12/25
*  Description: CPRI Driver Module Source File
*  Change History: 
*      <author>      <time>      <version>   <desc>
*    1. cheng.wan     23/12/25      1.0       refactor this module
**********************************************************************************************************************/

/* -------------------------------------------------- <INC FILE> --------------------------------------------------- */
#include "ucp_utility.h"
#include "ucp_printf.h"
#include "spu_shell.h"
#include "mtimer_com.h"
#include "mtimer_drv.h"
#include "cpri_csu.h"
#include "cpri_driver.h"
#include "cpri_mode.h"


/* --------------------------------------------------- <DEF&DEC> --------------------------------------------------- */
extern  void  cpri_rt_test_enter();
extern  void  cpri_rt_test_exit();

/* -------------------------------------------------- <FUNC DEF> --------------------------------------------------- */
/**********************************************************************************************************************
*  function: cpri_mode_proc
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
uint32_t  cpri_mode_proc_cnt = 0;
uint32_t  cpri_test_mode_cnt = 0;
void cpri_mode_proc(void) 
{
    cpri_mode_proc_cnt++;

    if(cpri_mode_proc_cnt == 1)
    {
        do_write(CPRI_WORK_MODE_ADDR, 0);
    }

#ifndef TEST_ENABLE
    /* enter test mode */
    if(do_read_volatile(CPRI_WORK_MODE_ADDR) == 1)
    {
        cpri_test_mode_cnt++;
        debug_write((DBG_DDR_IDX_CPRI_BASE + 1026), cpri_test_mode_cnt);

        if(cpri_test_mode_cnt == 1)
        {
            cpri_rt_test_enter();
        }
    }

    /* exit test mode */
    if(do_read_volatile(CPRI_WORK_MODE_ADDR) == 2)
    {
        if(cpri_test_mode_cnt > 10000)
        {
            cpri_test_mode_cnt = 0;
            cpri_rt_test_exit();
        }
    }
#endif

    debug_write((DBG_DDR_IDX_CPRI_BASE + 1025), cpri_mode_proc_cnt); 
}



/**********************************************************************************************************************
*  function: cpri_cmd_func
*  description: 
*  input : 
*         <name>                     <type>                      <desc>
*          mode                      uint32_t                     real time test mode enter/exit
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
void cpri_cmd_func(uint32_t mode)
{
    if(mode == 1)
    {
        do_write(CPRI_WORK_MODE_ADDR, 1);
    }
    
    if(mode == 2)
    {
        do_write(CPRI_WORK_MODE_ADDR, 2);
    }
}



/**********************************************************************************************************************
*  function: spu_shell_cpri
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
void spu_shell_cpri(void)
{
	spu_insert_cmd_ext("cpri_ape", (OSP_FUNCPTR)cpri_cmd_func, "set cpri mode", 1);
}