/**********************************************************************************************************************
*  Copyright (C), 2022-2026, SMARTLOGIC TECHNOLOGY LTD.
*  File Name: cpri_test_case.c
*  Create Date: 23/12/25
*  Description: 
*  Change History: 
*      <author>      <time>      <version>   <desc>
*    1. xinxin.li    23/01/11      1.0       build this module
*    2. cheng.wan    24/02/01      2.0       refactor
**********************************************************************************************************************/
/* -------------------------------------------------- <INC FILE> --------------------------------------------------- */
#include "typedef.h"
#include "ucp_utility.h"
#include "ucp_printf.h"
#include "cpri_timer.h"
#include "cpri_driver.h"
#include "rfm1_drv.h"
#include "cpri_test.h"
#include "cpri_csu_nr_7ds2u.h"
#include "cpri_test_case.h"


/* --------------------------------------------------- <DEF&DEC> --------------------------------------------------- */




/* --------------------------------------------------- <FUNC DEF> --------------------------------------------------- */
/**********************************************************************************************************************
*  function: ctrl_word_init
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
extern uint32_t ctrlword_tx_test_data[48*2];

void ctrl_word_init()
{
    for(uint32_t i = 0; i < 48*2; i++)
    {
        do_write(((uint32_t *)&ctrlword_tx_test_data[0] + i), 0x12345678 + i);
    }    
}


/**********************************************************************************************************************
*  function: ctrl_word_check
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
extern uint32_t ctrlword_insert_cnt;
       uint32_t ctrlword_error_cnt = 0;

void ctrl_word_check(void)
{
    uint32_t i;
    uint32_t xs_idx, ns_idx;
    uint32_t ctrlword_rx_data[48];

    while(1)
    {
        if((cpri_aux_get_rx_hfn() == (SL_TEST_CTRL_WORD_TX_HFN + 2)))
        {
            debug_write((DBG_DDR_IDX_CPRI_BASE + 142), do_read_volatile(&AUX_CNT0));
            debug_write((DBG_DDR_IDX_CPRI_BASE + 143), do_read_volatile(&AUX_CNT2));
            break;
        }
    }

    for(xs_idx = 0; xs_idx < 4; xs_idx++)
    {
        for(ns_idx = 8; ns_idx < 20; ns_idx++)
        {
            ctrlword_rx_data[xs_idx*12 + ns_idx - 8] = cpri_headerram_rx(64*xs_idx + ns_idx, 0);
        }
    }

    for(i = 0; i < 48; i++)
    {
        if(ctrlword_rx_data[i] != ctrlword_tx_test_data[(ctrlword_insert_cnt%2)*48 + i])
        {
            ctrlword_error_cnt++;
        }
    }

    debug_write((DBG_DDR_IDX_CPRI_BASE + 140), ctrlword_error_cnt);
}



/**********************************************************************************************************************
*  function: axc_data_init
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
void axc_data_init()
{
    uint16_t    data_id;
    uint16_t    bf_idx;
    uint16_t    slot_type;      //0:even slot, 1:odd slot
    uint16_t    symb_type;      //0:symbol0~6, 1:symbol7~13
    uint16_t    symb_idx;
    uint16_t    word_idx;

    uint32_t    *p_addr  = NULL;
    uint32_t    src_addr = 0;
    uint32_t    dst_addr = 0;
    uint32_t    data_len = 0;
    uint16_t    byte_per_bf = 0;
    uint32_t    bf_per_slot = LONGCP_BF_CNT + SHORTCP_BF_CNT*13;
    uint32_t    bf_per_f7   = LONGCP_BF_CNT + SHORTCP_BF_CNT*6;
    uint32_t    bf_per_b7   = SHORTCP_BF_CNT*7;
    uint32_t    bf_per_symb = 0;
    uint32_t    word_in_slot= 0;
    uint32_t    value = 0;
    uint16_t    bit_shift; 



    /* generate compress factor test data for dl part(7DS2U) */
    for(slot_type = 0; slot_type <= 1; slot_type++)
    {
        byte_per_bf = 2;

        if(slot_type == 0)	    //even slot
        {
            dst_addr = CPRI_NR7DS2U_TX_SLOT_EVEN_COMPRESS_ADDR;
        }
        else				    //odd slot
        {
            dst_addr = CPRI_NR7DS2U_TX_SLOT_ODD_COMPRESS_ADDR;
        }

        p_addr = (uint32_t *)dst_addr;
        for(bf_idx = 0; bf_idx < (bf_per_slot >> 1); bf_idx++)
        {
            value = (slot_type << 28) | ((bf_idx << 1) << 8) | (0);
            do_write((uint32_t)(p_addr), value);
            p_addr++;
        }
    }

    /* generate iq test data for dl part(7DS2U) */
    for(data_id = 1; data_id < 5; data_id++)
    {
        byte_per_bf = 64;

        for(slot_type = 0; slot_type <= 1; slot_type++)
        {
            word_in_slot = 0;
            for(symb_type = 0; symb_type <= 1; symb_type++)
            {
                if((slot_type == 0) && (symb_type == 0))	    //even slot, symbol0~6
                {
                    dst_addr = CPRI_NR7DS2U_TX_SLOT_EVEN_F7SYMBOL_ADDR + (data_id - 1)*(bf_per_f7 << 6);
                }
                else if((slot_type == 0) && (symb_type == 1))	//even slot, symbol7~13
                {
                    dst_addr = CPRI_NR7DS2U_TX_SLOT_EVEN_B7SYMBOL_ADDR + (data_id - 1)*(bf_per_b7 << 6);
                }
                else if((slot_type == 1) && (symb_type == 0))	//odd slot, symbol0~6
                {
                    dst_addr = CPRI_NR7DS2U_TX_SLOT_ODD_F7SYMBOL_ADDR  + (data_id - 1)*(bf_per_f7 << 6);
                }
                else if((slot_type == 1) && (symb_type == 1))	//odd slot, symbol7~13
                {
                    dst_addr = CPRI_NR7DS2U_TX_SLOT_ODD_B7SYMBOL_ADDR  + (data_id - 1)*(bf_per_b7 << 6);
                }

                p_addr = (uint32_t *)dst_addr;
                for(symb_idx = 0; symb_idx < 7; symb_idx++)
                {
                    if((symb_idx == 0) && (symb_type == 0))
                    {
                        bf_per_symb = LONGCP_BF_CNT;
                    }
                    else
                    {
                        bf_per_symb = SHORTCP_BF_CNT;
                    }

                    for(bf_idx = 0; bf_idx < bf_per_symb; bf_idx++)
                    {
                        for(word_idx = 0; word_idx < (byte_per_bf >> 2); word_idx++)
                        {
                            value = (slot_type << 28) | (data_id << 24) | ((word_in_slot++) << 8) | (word_idx);
                            do_write((uint32_t)(p_addr), value);
                            p_addr++;
                        }
                    }
                }
            }
        }
    }

    /* generate agc factor test data for dl part(7DS2U) */
    for(slot_type = 0; slot_type <= 1; slot_type++)
    {
        byte_per_bf = 2;

        if(0 == slot_type)
        {
            dst_addr = CPRI_NR7DS2U_TX_SLOT_EVEN_AGC_ADDR;
        }
        else
        {
            dst_addr = CPRI_NR7DS2U_TX_SLOT_ODD_AGC_ADDR;
        }

        p_addr = (uint32_t *)dst_addr;
        for(bf_idx = 0; bf_idx < (bf_per_slot >> 1); bf_idx++)
        {
            value = (slot_type << 28) | (5 << 24) | ((bf_idx << 1) << 8) | (0);
            do_write((uint32_t)(p_addr), value);
            p_addr++;
        }
    }



    /* generate compress factor test data for ul part(7DS2U) */ 
     bit_shift = 1; //2byte per bf

    /* symbol 6~13 in slot S */
    src_addr = CPRI_NR7DS2U_TX_SLOT_ODD_COMPRESS_ADDR + ((LONGCP_BF_CNT + SHORTCP_BF_CNT*5) << bit_shift);
    dst_addr = CPRI_NR7DS2U_TX_DUMMY_COMPRESS_ADDR;
    data_len = (SHORTCP_BF_CNT*8) << bit_shift;
    spu_csu_dma_1D_transfer(src_addr, dst_addr, data_len);

    /* slot 8 */
    src_addr  = CPRI_NR7DS2U_TX_SLOT_EVEN_COMPRESS_ADDR;
    dst_addr += data_len;
    data_len  = bf_per_slot << bit_shift;
    spu_csu_dma_1D_transfer(src_addr, dst_addr, data_len);

    /* slot 9 */
    src_addr  = CPRI_NR7DS2U_TX_SLOT_ODD_COMPRESS_ADDR;
    dst_addr += data_len;
    data_len  = bf_per_slot << bit_shift;
    spu_csu_dma_1D_transfer(src_addr, dst_addr, data_len);


    /* generate iq test data for ul part(7DS2U) */ 
    bit_shift = 6;  //64byte per bf

    for(uint8_t antId = 0; antId < 4; antId++)
    {
        /* symbol 6 in slot S */
        src_addr = CPRI_NR7DS2U_TX_SLOT_ODD_F7SYMBOL_ADDR + (antId*(bf_per_f7 << bit_shift)) + ((LONGCP_BF_CNT + SHORTCP_BF_CNT*5) << bit_shift);
        dst_addr = CPRI_NR7DS2U_TX_DUMMY_AXC1DATA_ADDR + antId*CPRI_NR7DS2U_TX_DUMMY_AXCDATA_LEN;
        data_len = (SHORTCP_BF_CNT) << bit_shift;
        spu_csu_dma_1D_transfer(src_addr, dst_addr, data_len);

        /* symbol 7~13 in slot S */
        src_addr  = CPRI_NR7DS2U_TX_SLOT_ODD_B7SYMBOL_ADDR + (antId*(bf_per_b7 << bit_shift));
        dst_addr += data_len;
        data_len  = bf_per_b7 << bit_shift;
        spu_csu_dma_1D_transfer(src_addr, dst_addr, data_len);

        /* symbol 0~6 in slot 8 */
        src_addr  = CPRI_NR7DS2U_TX_SLOT_EVEN_F7SYMBOL_ADDR + (antId*(bf_per_f7 << bit_shift));
        dst_addr += data_len;
        data_len  = bf_per_f7 << bit_shift;
        spu_csu_dma_1D_transfer(src_addr, dst_addr, data_len);

        /* symbol 7~13 in slot 8 */
        src_addr  = CPRI_NR7DS2U_TX_SLOT_EVEN_B7SYMBOL_ADDR + (antId*(bf_per_b7 << bit_shift));
        dst_addr += data_len;
        data_len  = bf_per_b7 << bit_shift;
        spu_csu_dma_1D_transfer(src_addr, dst_addr, data_len);

        /* symbol 0~6 in slot 9 */
        src_addr  = CPRI_NR7DS2U_TX_SLOT_ODD_F7SYMBOL_ADDR + (antId*(bf_per_f7 << bit_shift));
        dst_addr += data_len;
        data_len  = bf_per_f7 << bit_shift;
        spu_csu_dma_1D_transfer(src_addr, dst_addr, data_len);

        /* symbol 7~13 in slot 9 */
        src_addr  = CPRI_NR7DS2U_TX_SLOT_ODD_B7SYMBOL_ADDR + (antId*(bf_per_b7 << bit_shift));
        dst_addr += data_len;
        data_len  = bf_per_b7 << bit_shift;
        spu_csu_dma_1D_transfer(src_addr, dst_addr, data_len);
    }

    /* generate agc factor test data for ul part(7DS2U) */ 
    bit_shift = 1;  //2byte per bf

    /* symbol 6~13 in slot S */
    src_addr = CPRI_NR7DS2U_TX_SLOT_ODD_AGC_ADDR + ((LONGCP_BF_CNT + SHORTCP_BF_CNT*5) << bit_shift);
    dst_addr = CPRI_NR7DS2U_TX_DUMMY_AGC_ADDR;
    data_len = (SHORTCP_BF_CNT*8) << bit_shift;
    spu_csu_dma_1D_transfer(src_addr, dst_addr, data_len);

    /* slot 8 */
    src_addr  = CPRI_NR7DS2U_TX_SLOT_EVEN_AGC_ADDR;
    dst_addr += data_len;
    data_len  = bf_per_slot << bit_shift;
    spu_csu_dma_1D_transfer(src_addr, dst_addr, data_len);

    /* slot 9 */
    src_addr  = CPRI_NR7DS2U_TX_SLOT_ODD_AGC_ADDR;
    dst_addr += data_len;
    data_len  = bf_per_slot << bit_shift;
    spu_csu_dma_1D_transfer(src_addr, dst_addr, data_len);
}




/**********************************************************************************************************************
*  function: axc_data_check
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
uint32_t    comp_word_sum_cnt = 0;
uint32_t    comp_word_err_cnt = 0;
uint32_t    comp_data_id_cnt  = 0;
uint32_t    bf_start_err_cnt  = 0;
void axc_data_check(uint32_t slot_idx)
{
    uint32_t    data_id   = 0;
    uint32_t    src_addr1 = 0;
    uint32_t    src_addr2 = 0;
    uint32_t    real_addr = 0;
    uint16_t    bit_shift = 0;
    uint32_t    bf_per_slot = (LONGCP_BF_CNT + SHORTCP_BF_CNT*13);
    uint16_t	word_per_bf = 0;

    uint16_t    slot_type = 0;
    int32_t     bf_start  = 0;
    uint32_t    comp_val  = 0;
    uint32_t    recv_val  = 0;
    uint32_t    recv_addr = 0;


    for(data_id = 0; data_id < 5; data_id++)
    {
        comp_data_id_cnt++;
        bf_start = 0;

        if((slot_idx >= 0) && (slot_idx <= 6))
        {
            slot_type = slot_idx & 0x1;

            if(data_id == 0)
            {
                bit_shift   = 1;      //compress factor, 2byte per bf
                word_per_bf = 1;
                src_addr1   = CPRI_NR7DS2U_RX_DUMMY_COMPRESS_ADDR + slot_idx*(bf_per_slot << bit_shift); 
            }
            else
            {
                bit_shift   = 6;      //iq data, 64byte per bf
                word_per_bf = 64 >> 2;
                src_addr1   = CPRI_NR7DS2U_RX_DUMMY_AXC1DATA_ADDR + ((data_id - 1)*CPRI_NR7DS2U_RX_DUMMY_AXCDATA_LEN) + slot_idx*(bf_per_slot << bit_shift); 
            }
        }
        else if(slot_idx == 7)        //compare S slot, odd slot
        {
            bf_start  = (LONGCP_BF_CNT + SHORTCP_BF_CNT*9);
            slot_type = 1;
            if(data_id == 0)
            {
                bit_shift   = 1; 
                word_per_bf = 1;
                src_addr1   = CPRI_NR7DS2U_RX_DUMMY_COMPRESS_ADDR + slot_idx*(bf_per_slot << bit_shift);
                src_addr2   = CPRI_NR7DS2U_RX_SLOTS_COMPRESS_ADDR;
            }
            else
            {
                bit_shift   = 6;
                word_per_bf = (64 >> 2);
                src_addr1   = CPRI_NR7DS2U_RX_DUMMY_AXC1DATA_ADDR + ((data_id - 1)*CPRI_NR7DS2U_RX_DUMMY_AXCDATA_LEN) + slot_idx*(bf_per_slot << bit_shift);
                src_addr2   = CPRI_NR7DS2U_RX_SLOTS_AXCDATA_ADDR  + (data_id - 1)*((SHORTCP_BF_CNT*4) << bit_shift);
            }
        }
        else if(slot_idx == 8)
        {
            slot_type = 0;
            if(data_id == 0)
            {
                bit_shift   = 1;
                word_per_bf = 1;
                src_addr1   = CPRI_NR7DS2U_RX_SLOT_EVEN_COMPRESS_ADDR;
            }
            else
            {
                bit_shift   = 6;
                word_per_bf = (64 >> 2);
                src_addr1   = CPRI_NR7DS2U_RX_SLOT_EVEN_AXCDATA_ADDR + ((data_id - 1)*(bf_per_slot << bit_shift));
            }
        }
        else if(slot_idx == 9)
        {
            slot_type = 1;
            if(data_id == 0)
            {
                bit_shift   = 1;
                word_per_bf = 1;
                src_addr1   = CPRI_NR7DS2U_RX_SLOT_ODD_COMPRESS_ADDR;
            }
            else
            {
                bit_shift   = 6;
                word_per_bf = (64 >> 2);
                src_addr1   = CPRI_NR7DS2U_RX_SLOT_ODD_AXCDATA_ADDR + ((data_id - 1)*(bf_per_slot << bit_shift));
            }
        }


        if(data_id == 0)   //check compress factor
        {
            for(int32_t bf_idx = 0; bf_idx < (bf_per_slot >> 1); bf_idx++)
            {
                for(uint32_t word_idx = 0; word_idx < word_per_bf; word_idx++)
                {
                    comp_val = (slot_type << 28) | (data_id << 24) | ((bf_idx << 1) << 8) | (word_idx);

                    debug_write((DBG_DDR_IDX_DRV_BASE + 1026), comp_word_sum_cnt); 
                    comp_word_sum_cnt++;
                    __ucps2_synch(0);

                    if((slot_idx == 7) && (bf_idx >= 686))
                    {
                        recv_addr = (uint32_t)((uint32_t*)src_addr2 + (bf_idx - (bf_start >> 1))*word_per_bf + word_idx);
                        if((bf_idx - (bf_start >> 1)) < 0)
                        {
                            bf_start_err_cnt++;
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1027), bf_start_err_cnt); 
                        }
                        real_addr = src_addr2;
                    }
                    else
                    {
                        recv_addr = (uint32_t)((uint32_t*)src_addr1 + bf_idx*word_per_bf + word_idx);
                        real_addr = src_addr1;
                    }

                    recv_val = do_read_volatile(recv_addr);
                    __ucps2_synch(0);
                    if(recv_val != comp_val)
                    {
                        if(comp_word_err_cnt < 0x100)
                        {
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1028 + ((comp_word_err_cnt << 3)&0x7FF)), comp_val);
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1029 + ((comp_word_err_cnt << 3)&0x7FF)), recv_val);
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1030 + ((comp_word_err_cnt << 3)&0x7FF)), recv_addr);
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1031 + ((comp_word_err_cnt << 3)&0x7FF)), real_addr);
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1032 + ((comp_word_err_cnt << 3)&0x7FF)), (slot_idx + (data_id << 4) + (bf_idx << 8)));
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1033 + ((comp_word_err_cnt << 3)&0x7FF)), bf_start);
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1034 + ((comp_word_err_cnt << 3)&0x7FF)), bf_per_slot);	
                        } 
                        comp_word_err_cnt++;
                    }
                }
            }
        }
        else            //check axc data
        {
            for(int32_t bf_idx = 0; bf_idx < bf_per_slot; bf_idx++)
            {
                for(uint32_t word_idx = 0; word_idx < word_per_bf; word_idx++)
                {
                    comp_val = (slot_type << 28) | (data_id << 24) | (((bf_idx << 4) + word_idx) << 8) | (word_idx);

                    debug_write((DBG_DDR_IDX_DRV_BASE + 1026), comp_word_sum_cnt); 
                    comp_word_sum_cnt++;
                    __ucps2_synch(0);

                    if((slot_idx == 7) && (bf_idx >= 1372))
                    {
                        recv_addr = (uint32_t)((uint32_t*)src_addr2 + (bf_idx - bf_start)*word_per_bf + word_idx);
                        real_addr = src_addr2;
                        if((bf_idx - bf_start) < 0)
                        {
                            bf_start_err_cnt++;
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1027), bf_start_err_cnt); 
                        }
                    }
                    else
                    {
                        recv_addr = (uint32_t)((uint32_t*)src_addr1 + bf_idx*word_per_bf + word_idx);
                        real_addr = src_addr1;
                    }

                    recv_val = do_read_volatile(recv_addr); 
                    __ucps2_synch(0);
                    if(recv_val != comp_val)
                    {
                        if(comp_word_err_cnt < 0x100)
                        {
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1028 + ((comp_word_err_cnt << 3)&0x7FF)), comp_val);
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1029 + ((comp_word_err_cnt << 3)&0x7FF)), recv_val);
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1030 + ((comp_word_err_cnt << 3)&0x7FF)), recv_addr);
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1031 + ((comp_word_err_cnt << 3)&0x7FF)), real_addr);
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1032 + ((comp_word_err_cnt << 3)&0x7FF)), (slot_idx + (data_id << 4) + (bf_idx << 8)));	
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1033 + ((comp_word_err_cnt << 3)&0x7FF)), bf_start);
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1034 + ((comp_word_err_cnt << 3)&0x7FF)), bf_per_slot); 
                        }
                        comp_word_err_cnt++;
                    }
                }
            }
        }

        debug_write((DBG_DDR_IDX_DRV_BASE + 1024), comp_data_id_cnt);
        debug_write((DBG_DDR_IDX_DRV_BASE + 1025), comp_word_err_cnt);
    }
}



/**********************************************************************************************************************
*  function: fh_data_init
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
extern  uint32_t  gCpriTestMode;

int32_t fh_data_init(void)
{
    gCpriTestMode = CPRI_TEST_MODE;
    debug_write((DBG_DDR_IDX_DRV_BASE + 192), gCpriTestMode);

    axc_data_init();
    ctrl_word_init();
    return 0;
}




/**********************************************************************************************************************
*  function: fh_data_check
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
extern  stMtimerIntStat  gMtimerIntCnt[SCS_MAX_NUM];
void fh_data_check(uint32_t times)
{
    uint32_t         slot_idx;
    stMtimerIntStat  *pMtimerInt;
    
     
    pMtimerInt = &gMtimerIntCnt[MTIMER_CPRI_ID];
    if(pMtimerInt->csuEnCnt >= 4)
    {
        comp_word_sum_cnt = 0;

        for(slot_idx = 0; slot_idx < (CPRI_CASE34_SLOT_NUM >> 1); slot_idx++)
        {
            axc_data_check(slot_idx);
        }

        if(pMtimerInt->csuEnCnt >= 24000)
        { 
            if(comp_word_err_cnt == 0)
            { 
                UCP_PRINT_WARN("cpri test pass!\r\n");
            }
            else
            {
                UCP_PRINT_WARN("cpri test fail!\r\n");
            }
        }

        ctrl_word_check();
    }
}



/**********************************************************************************************************************
*  function: fh_test_case
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
void fh_test_case()
{
	UCP_API_CPRI_CSU_START();
}


/**********************************************************************************************************************
*  function: fh_csu_test_init
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
extern  void  cpri_csu_axcctrl_init_timing();

int32_t fh_csu_test_init(void)
{
    stCpriCsuCmdFifoInfo    txTestCmdFifo;
    stCpriCsuCmdFifoInfo    rxTestCmdFifo;

    cpri_csu_axc_init_nr_7ds2u(CPRI_DUMMY_USE_DDR_ADDR, &txTestCmdFifo, &rxTestCmdFifo);
    cpri_csu_axcctrl_init_timing();
    UCP_API_CPRI_CSU_Get_CmdFIFO(&txTestCmdFifo, &rxTestCmdFifo);

    return 0;
}


/**********************************************************************************************************************
*  function: fh_drv_init
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
int32_t fh_drv_init(void)
{
    stFrontHaulDrvPara fhDrvPara;


    memset_ucp(&fhDrvPara, 0, sizeof(stFrontHaulDrvPara));

    fhDrvPara.protocolSel = PROTOCOL_CPRI;
    fhDrvPara.rateOption  = CPRI_OPTION_8;
    fhDrvPara.mapOption   = OTIC_MAP_FIGURE12;
    fronthaul_drv_cfg(&fhDrvPara);

    return 0;
}


/**********************************************************************************************************************
*  function: cpri_rt_test_enter
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
void cpri_rt_test_enter(void)
{
    axc_data_init();
    cpri_csu_axc_link_init_nr_7ds2u(CPRI_DUMMY_USE_DDR_ADDR);       //修改CSU链表为测试模式
}



/**********************************************************************************************************************
*  function: cpri_rt_test_exit
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
#define CPRI_TEST_SLOT_NUM   (20)
void cpri_rt_test_exit(void)
{
    stMtimerIntStat* pMtimerInt;
    
    pMtimerInt = &gMtimerIntCnt[MTIMER_CPRI_ID];
    if(pMtimerInt->csuEnCnt >= 4)
    {
        for(int32_t i = 0; i < (CPRI_TEST_SLOT_NUM >> 1); i++)
        {
            axc_data_check(i);
        }
    }

    cpri_csu_axc_link_init_nr_7ds2u(CPRI_DUMMY_USE_DUMMY_ADDR);     //修改CSU链表为正式模式
}

