/**********************************************************************************************************************
*  Copyright (C), 2022-2026, SMARTLOGIC TECHNOLOGY LTD.
*  File Name: cpri_test_case.c
*  Create Date: 23/12/25
*  Description: 
*  Change History: 
*      <author>      <time>      <version>   <desc>
*    1. xinxin.li    23/01/11      1.0       build this module
*    2. cheng.wan    24/02/01      2.0       refactor
**********************************************************************************************************************/
/* -------------------------------------------------- <INC FILE> --------------------------------------------------- */
#include "typedef.h"
#include "ucp_utility.h"
#include "ucp_printf.h"
#include "cpri_timer.h"
#include "cpri_driver.h"
#include "rfm1_drv.h"
#include "cpri_test.h"
#include "cpri_csu_nr_7ds2u.h"
#include "cpri_test_case.h"



/* --------------------------------------------------- <DEF&DEC> --------------------------------------------------- */
extern uint32_t nr_comp[1920];
extern uint32_t nr_ant0[61440];
extern uint32_t nr_ant1[61440];
extern uint32_t nr_ant2[61440];
extern uint32_t nr_ant3[61440];


/* --------------------------------------------------- <FUNC DEF> --------------------------------------------------- */
/**********************************************************************************************************************
*  function: axc_data_init
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
void axc_data_init()
{
    uint32_t    data_id   = 0;
    uint32_t    slot_type = 0;      //0:even slot, 1:odd slot
    uint32_t    symb_type = 0;      //0:symbol0~6, 1:symbol7~13
    uint32_t    src_addr  = 0;
    uint32_t    dst_addr  = 0;
    uint32_t    data_len  = 0;
    uint32_t    copy_cnt  = 0;
    uint16_t    byte_per_bf = 0;
    uint32_t    bf_per_slot = LONGCP_BF_CNT+SHORTCP_BF_CNT*13;
    uint32_t    bf_per_f7   = LONGCP_BF_CNT+SHORTCP_BF_CNT*6;
    uint32_t    bf_per_b7   = SHORTCP_BF_CNT*7;



    /* compress factor */
    for(slot_type = 0; slot_type <= 1; slot_type++)
    {
        byte_per_bf = 2;

        if(slot_type == 0)
        {
            src_addr = (uint32_t)(&nr_comp[0]);
            dst_addr = CPRI_NR7DS2U_TX_SLOT_EVEN_COMPRESS_ADDR;
        }
        else
        {
            src_addr = (uint32_t)(&nr_comp[1920 >> 1]);
            dst_addr = CPRI_NR7DS2U_TX_SLOT_ODD_COMPRESS_ADDR;
        }

        data_len = (byte_per_bf*bf_per_slot); 

        debug_write((DBG_DDR_IDX_DRV_BASE + 196 + (copy_cnt << 2) + 0), (uint32_t)src_addr);
        debug_write((DBG_DDR_IDX_DRV_BASE + 196 + (copy_cnt << 2) + 1), (uint32_t)dst_addr);
        debug_write((DBG_DDR_IDX_DRV_BASE + 196 + (copy_cnt << 2) + 2), (uint32_t)data_len);

        copy_cnt++;
        memcpy_ucp((void*)dst_addr,(void*)src_addr, data_len);
    }

    /* iq data */ 	
    for(data_id = 1; data_id < 5; data_id++)
    {
        byte_per_bf = 64;
        for(slot_type = 0; slot_type <= 1; slot_type++)
        {
            for(symb_type = 0; symb_type <= 1; symb_type++)
            {
                if((slot_type == 0) && (symb_type == 0))        //even slot, symbol0~6
                {
                    data_len = byte_per_bf * bf_per_f7;

                    switch(data_id)
                    {
                        case 1:
                            src_addr = (uint32_t)(&nr_ant0[0]);
                            break;
                        case 2:
                            src_addr = (uint32_t)(&nr_ant1[0]);
                            break;	
                        case 3:
                            src_addr = (uint32_t)(&nr_ant2[0]);
                            break;
                        case 4:
                            src_addr = (uint32_t)(&nr_ant3[0]);
                            break;
                    }

                    dst_addr = CPRI_NR7DS2U_TX_SLOT_EVEN_F7SYMBOL_ADDR + (data_id - 1)*data_len;
                }
                else if((slot_type == 0) && (symb_type == 1))   // even slot, symbol7~13
                {
                    data_len = byte_per_bf * bf_per_b7;

                    switch(data_id)
                    {
                        case 1:
                            src_addr = (uint32_t)(&nr_ant0[15376]);
                            break;
                        case 2:
                            src_addr = (uint32_t)(&nr_ant1[15376]);
                            break;	
                        case 3:
                            src_addr = (uint32_t)(&nr_ant2[15376]);
                            break;
                        case 4:
                            src_addr = (uint32_t)(&nr_ant3[15376]);
                            break;
                    }

                    dst_addr = CPRI_NR7DS2U_TX_SLOT_EVEN_B7SYMBOL_ADDR + (data_id - 1)*data_len;
                }
                else if((slot_type == 1) && (symb_type == 0))   //odd slot, symbol0~6
                {
                    data_len = byte_per_bf * bf_per_f7;

                    switch(data_id)
                    {
                        case 1:
                            src_addr = (uint32_t)(&nr_ant0[30720]);
                            break;
                        case 2:
                            src_addr = (uint32_t)(&nr_ant1[30720]);
                            break;	
                        case 3:
                            src_addr = (uint32_t)(&nr_ant2[30720]);
                            break;
                        case 4:
                            src_addr = (uint32_t)(&nr_ant3[30720]);
                            break;
                    }

                    dst_addr = CPRI_NR7DS2U_TX_SLOT_ODD_F7SYMBOL_ADDR + (data_id - 1)*data_len;
                }
                else if((slot_type == 1) && (symb_type == 1))   //odd slot, symbol7~13
                {
                    data_len = byte_per_bf * bf_per_b7;
                    switch(data_id)
                    {
                        case 1:
                            src_addr = (uint32_t)(&nr_ant0[30720 + 15376]);
                            break;  
                        case 2:  
                            src_addr = (uint32_t)(&nr_ant1[30720 + 15376]);
                            break;	  
                        case 3:  
                            src_addr = (uint32_t)(&nr_ant2[30720 + 15376]);
                            break;  
                        case 4:  
                            src_addr = (uint32_t)(&nr_ant3[30720 + 15376]);
                            break;
                    }

                    dst_addr = CPRI_NR7DS2U_TX_SLOT_ODD_B7SYMBOL_ADDR + (data_id - 1)*data_len;
                }

                debug_write((DBG_DDR_IDX_DRV_BASE + 196 + (copy_cnt << 2) + 0), (uint32_t)src_addr);
                debug_write((DBG_DDR_IDX_DRV_BASE + 196 + (copy_cnt << 2) + 1), (uint32_t)dst_addr);
                debug_write((DBG_DDR_IDX_DRV_BASE + 196 + (copy_cnt << 2) + 2), (uint32_t)data_len);

                copy_cnt++;
                memcpy_ucp((void*)dst_addr,(void*)src_addr, data_len);
            }
        }
    }
}


/**********************************************************************************************************************
*  function: axc_data_check
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
void axc_data_check(uint32_t slot_idx)
{
    return;
}





/**********************************************************************************************************************
*  function: fh_data_init
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
extern  uint32_t  gCpriTestMode;

int32_t fh_data_init(void)
{
    gCpriTestMode = CPRI_TEST_MODE;
    debug_write((DBG_DDR_IDX_DRV_BASE + 192), gCpriTestMode);

    axc_data_init();

    return 0;
}


/**********************************************************************************************************************
*  function: fh_data_check
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
void fh_data_check(uint32_t times)
{
    return;
}


/**********************************************************************************************************************
*  function: fh_test_case
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
void fh_test_case()
{
	UCP_API_CPRI_CSU_START();
}


/**********************************************************************************************************************
*  function: fh_csu_test_init
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
extern  void  cpri_csu_axcctrl_init_timing();

int32_t fh_csu_test_init(void)
{
    stCpriCsuCmdFifoInfo    txTestCmdFifo;
    stCpriCsuCmdFifoInfo    rxTestCmdFifo;

    cpri_csu_axc_init_nr_7ds2u(CPRI_DUMMY_USE_DDR_ADDR, &txTestCmdFifo, &rxTestCmdFifo);
    cpri_csu_axcctrl_init_timing();
    UCP_API_CPRI_CSU_Get_CmdFIFO(&txTestCmdFifo, &rxTestCmdFifo);

    return 0;
}


/**********************************************************************************************************************
*  function: fh_drv_init
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
int32_t fh_drv_init(void)
{
    stFrontHaulDrvPara fhDrvPara;


    memset_ucp(&fhDrvPara, 0, sizeof(stFrontHaulDrvPara));

    fhDrvPara.protocolSel = PROTOCOL_CPRI;
    fhDrvPara.rateOption  = CPRI_OPTION_8;
    fhDrvPara.mapOption   = OTIC_MAP_FIGURE12;
    fronthaul_drv_cfg(&fhDrvPara);

    return 0;
}



