/**********************************************************************************************************************
*  Copyright (C), 2022-2026, SMARTLOGIC TECHNOLOGY LTD.
*  File Name: cpri_test_case70.c
*  Create Date: 24/2/20
*  Description: 
*  Change History: 
*      <author>      <time>      <version>   <desc>
*    1. cheng.wan    24/02/20      2.0       refactor
**********************************************************************************************************************/
/* -------------------------------------------------- <INC FILE> --------------------------------------------------- */
#include "typedef.h"
#include "ucp_utility.h"
#include "ucp_printf.h"
#include "cpri_timer.h"
#include "cpri_driver.h"
#include "rfm1_drv.h"
#include "cpri_test.h"
#include "nr_mem_def.h"
#include "cpri_csu.h"
#include "phy_timer_csu_config.h"
#include "cpri_test_case.h"

/* --------------------------------------------------- <DEF&DEC> --------------------------------------------------- */




/* -------------------------------------------------- <FUNC DEF> --------------------------------------------------- */
/**********************************************************************************************************************
*  function: axc_data_init
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
void axc_data_init()
{
    uint16_t  data_id   = 0;
    uint16_t  slot_type = 0;      //0:even slot, 1:odd slot
    uint16_t  symb_type = 0;      //0:symbol0~6, 1:symbol7~13
    uint16_t  symb_idx  = 0;
    uint16_t  bf_idx    = 0;
    uint16_t  word_idx  = 0;

    uint32_t  *p_addr   = NULL;
    uint32_t  src_addr  = 0;
    uint32_t  dst_addr  = 0;
    uint32_t  data_len  = 0;
    uint16_t  byte_per_bf = 0;
    uint32_t  bf_per_slot = LONGCP_BF_CNT + SHORTCP_BF_CNT*13;
    uint32_t  bf_per_f7   = LONGCP_BF_CNT + SHORTCP_BF_CNT*6;
    uint32_t  bf_per_b7   = SHORTCP_BF_CNT*7;
    uint32_t  bf_per_symb = 0;
    uint32_t  word_in_slot= 0;
    uint32_t  value = 0;
    uint16_t  bit_shift; 


   /* generate nr cell0&cell1's compress factor test data for dl part(7DS2U) */
    for(data_id = 0; data_id < 2; data_id++)
    {
        for(slot_type = 0; slot_type <= 1; slot_type++)
        {
            byte_per_bf = 2;

            if(slot_type == 0)
            {
                if(data_id == 0)
                {
                    dst_addr = SM1_NR_CELL0_EVEN_COMP_FACTOR_ADDR;
                }
                else
                {
                    dst_addr = SM3_NR_CELL1_EVEN_COMP_FACTOR_ADDR;
                }
            }
            else
            {
                if(data_id == 0)
                {
                    dst_addr = SM1_NR_CELL0_ODD_TX_COMP_FACTOR_ADDR;
                }
                else
                {
                    dst_addr = SM3_NR_CELL1_ODD_TX_COMP_FACTOR_ADDR;
                }
            }

            p_addr = ((uint32_t*)dst_addr);
            for(bf_idx = 0; bf_idx < (bf_per_slot >> 1); bf_idx++)
            {
                value = (slot_type << 28) | (data_id << 24)| ((bf_idx << 1) << 8) | (data_id);
                do_write((uint32_t)p_addr, value);	
                p_addr++;
            }
        }
    }

    /* generate nr cell0&cell1's iq test data for dl part(7DS2U) */
    for(data_id = 2; data_id < 6; data_id++)
    {
        byte_per_bf = 64*2;     //total 2 ant data

        for(slot_type = 0; slot_type <= 1; slot_type++)
        {
            word_in_slot = 0;

            for(symb_type = 0; symb_type <= 1; symb_type++)
            {
                if((slot_type == 0) && (symb_type == 0))
                {
                    if((data_id == 2) || (data_id == 3))
                    {
                        dst_addr = SM0_NR_CELL0_EVEN_F7_TX_DATA_ADDR + (data_id - 2)*(bf_per_f7 << 7);
                    }
                    else
                    {
                        dst_addr = SM2_NR_CELL1_EVEN_F7_TX_DATA_ADDR + (data_id - 4)*(bf_per_f7 << 7);
                    }
                }
                else if((slot_type == 0) && (symb_type == 1))
                    {
                    if((data_id == 2) || (data_id == 3))
                    {
                        dst_addr = SM4_NR_CELL0_EVEN_B7_TX_DATA_ADDR + (data_id - 2)*(bf_per_b7 << 7);
                    }
                    else
                    {
                        dst_addr = SM5_NR_CELL1_EVEN_B7_TX_DATA_ADDR + (data_id - 4)*(bf_per_b7 << 7);
                    }
                }
                else if((slot_type == 1) && (symb_type == 0))
                {
                    if((data_id == 2) || (data_id == 3))
                    {
                        dst_addr = SM1_NR_CELL0_ODD_F7_TX_DATA_ADDR + (data_id - 2)*(bf_per_f7 << 7);
                    }
                    else
                    {
                        dst_addr = SM3_NR_CELL1_ODD_F7_TX_DATA_ADDR + (data_id - 4)*(bf_per_f7 << 7);
                    }
                }
                else if((slot_type == 1) && (symb_type == 1))
                {
                    if((data_id == 2) || (data_id == 3))
                    {
                        dst_addr = SM4_NR_CELL0_ODD_B7_TX_DATA_ADDR + (data_id - 2)*(bf_per_b7 << 7);
                    }
                    else
                    {
                        dst_addr = SM5_NR_CELL1_ODD_B7_TX_DATA_ADDR + (data_id - 4)*(bf_per_b7 << 7);
                    }
                }

                for(symb_idx = 0; symb_idx < 7; symb_idx++)
                {
                    p_addr = ((uint32_t*)dst_addr);

                    if((symb_idx == 0) && (symb_type == 0))
                    {
                        bf_per_symb = LONGCP_BF_CNT;
                    }
                    else
                    {
                        bf_per_symb = SHORTCP_BF_CNT;
                    }

                    for(bf_idx = 0; bf_idx < bf_per_symb; bf_idx++)
                    {
                        for(word_idx = 0; word_idx < (byte_per_bf >> 2); word_idx++)
                        {
                            value = (slot_type << 28) | (data_id << 24) | ((word_in_slot++) << 8) | (word_idx);
                            do_write((uint32_t)p_addr, value);	
                            p_addr++;
                        }
                    }

                    data_len  = bf_per_symb*byte_per_bf;
                    dst_addr += data_len;
                }
            }
        }
    }


    /* generate nr cell0&cell1's compress factor test data for ul part(7DS2U) */ 
    bit_shift = 1;  //2byte per bf

    /* symbol 6~13 in slot S */
    src_addr = SM1_NR_CELL0_ODD_TX_COMP_FACTOR_ADDR + ((LONGCP_BF_CNT + SHORTCP_BF_CNT*5) << bit_shift);
    dst_addr = CPRI_NRCELL_TXDUMMY_SLOTS_COMPRESS_ADDR;
    data_len = (SHORTCP_BF_CNT*8) << bit_shift;
    spu_csu_dma_1D_transfer(src_addr, dst_addr, data_len);

    /* slot 8 */
    src_addr  = SM1_NR_CELL0_EVEN_COMP_FACTOR_ADDR;
    dst_addr += data_len;
    data_len  = bf_per_slot << bit_shift;
    spu_csu_dma_1D_transfer(src_addr, dst_addr, data_len);

    /* slot 9 */
    src_addr  = SM1_NR_CELL0_ODD_TX_COMP_FACTOR_ADDR;
    dst_addr += data_len;
    data_len  = bf_per_slot << bit_shift;
    spu_csu_dma_1D_transfer(src_addr, dst_addr, data_len);


    /* symbol 6~13 in slot S */
    src_addr = SM3_NR_CELL1_ODD_TX_COMP_FACTOR_ADDR + ((LONGCP_BF_CNT + SHORTCP_BF_CNT*5) << bit_shift);
    dst_addr = CPRI_NRCELL_TXDUMMY_SLOTS_COMPRESS_ADDR + (4936 << bit_shift);//CPRI_NR7DS2U_TX_DUMMY_COMP_FACTOR_LEN;
    data_len = (SHORTCP_BF_CNT*8) << bit_shift;
    spu_csu_dma_1D_transfer(src_addr, dst_addr, data_len);

    /* slot 8 */
    src_addr  = SM3_NR_CELL1_EVEN_COMP_FACTOR_ADDR;
    dst_addr += data_len;
    data_len  = bf_per_slot << bit_shift;
    spu_csu_dma_1D_transfer(src_addr, dst_addr, data_len);

    /* slot 9 */
    src_addr  = SM3_NR_CELL1_ODD_TX_COMP_FACTOR_ADDR;
    dst_addr += data_len;
    data_len  = bf_per_slot << bit_shift;
    spu_csu_dma_1D_transfer(src_addr, dst_addr, data_len);


    /* generate nr cell0&cell1's iq test data for ul part(7DS2U) */ 
    bit_shift = 7;  //128byte per bf

    for(uint8_t antId = 0; antId < 4; antId++)
    {
        /* symbol 6 in slot S */
        if(antId < 2)   //cell0
        {
            src_addr = SM1_NR_CELL0_ODD_F7_TX_DATA_ADDR + (antId*(bf_per_f7 << bit_shift)) + ((LONGCP_BF_CNT + SHORTCP_BF_CNT*5) << bit_shift);
        }
        else            //cell1
        {
            src_addr = SM3_NR_CELL1_ODD_F7_TX_DATA_ADDR + ((antId - 2)*(bf_per_f7 << bit_shift)) + ((LONGCP_BF_CNT + SHORTCP_BF_CNT*5) << bit_shift);
        }

        dst_addr = CPRI_NRCELL_TXDUMMY_SLOTS_AXCDATA_ADDR + antId*(4936 << bit_shift);
        data_len = (SHORTCP_BF_CNT) << bit_shift;	
        spu_csu_dma_1D_transfer(src_addr, dst_addr, data_len);

        /* symbol 7~13 in slot S */
        if(antId < 2)
        {
            src_addr = SM4_NR_CELL0_ODD_B7_TX_DATA_ADDR + (antId*(bf_per_b7 << bit_shift));
        }
        else
        {
            src_addr = SM5_NR_CELL1_ODD_B7_TX_DATA_ADDR + ((antId - 2)*(bf_per_b7 << bit_shift));
        }		
        dst_addr += data_len;
        data_len  = bf_per_b7 << bit_shift;	
        spu_csu_dma_1D_transfer(src_addr, dst_addr, data_len);

        /* symbol 0~6 in slot 8 */
        if(antId < 2)
        {
            src_addr = SM0_NR_CELL0_EVEN_F7_TX_DATA_ADDR + (antId*(bf_per_f7 << bit_shift));
        }
        else
        {
            src_addr = SM2_NR_CELL1_EVEN_F7_TX_DATA_ADDR + ((antId - 2)*(bf_per_f7 << bit_shift));
        }	
        dst_addr += data_len;
        data_len  = bf_per_f7 << bit_shift;		
        spu_csu_dma_1D_transfer(src_addr, dst_addr, data_len);

        /* symbol 7~13 in slot 8 */
        if(antId < 2)
        {
            src_addr = SM4_NR_CELL0_EVEN_B7_TX_DATA_ADDR + (antId*(bf_per_b7 << bit_shift));
        }
        else
        {
            src_addr = SM5_NR_CELL1_EVEN_B7_TX_DATA_ADDR + ((antId - 2)*(bf_per_b7 << bit_shift));
        }	
        dst_addr += data_len;
        data_len  = bf_per_b7 << bit_shift;	
        spu_csu_dma_1D_transfer(src_addr, dst_addr, data_len);

        /* symbol 0~6 in slot 9 */
        if(antId < 2)
        {
            src_addr = SM1_NR_CELL0_ODD_F7_TX_DATA_ADDR + (antId*(bf_per_f7 << bit_shift));
        }
        else
        {
            src_addr = SM3_NR_CELL1_ODD_F7_TX_DATA_ADDR + ((antId - 2)*(bf_per_f7 << bit_shift));
        }	
        dst_addr += data_len;
        data_len  = bf_per_f7 << bit_shift;		
        spu_csu_dma_1D_transfer(src_addr, dst_addr, data_len);

        /* symbol 7~13 in slot 9 */	
        if(antId < 2)
        {
            src_addr = SM4_NR_CELL0_ODD_B7_TX_DATA_ADDR + (antId*(bf_per_b7 << bit_shift));
        }
        else
        {
            src_addr = SM5_NR_CELL1_ODD_B7_TX_DATA_ADDR + ((antId - 2)*(bf_per_b7 << bit_shift));
        }

        dst_addr += data_len;
        data_len  = bf_per_b7 << bit_shift;		
        spu_csu_dma_1D_transfer(src_addr, dst_addr, data_len);
    }
}





/**********************************************************************************************************************
*  function: axc_data_check
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
uint32_t comp_word_sum_cnt = 0;
uint32_t comp_word_err_cnt = 0;
uint32_t comp_data_id_cnt  = 0;
uint32_t bf_start_err_cnt  = 0;
void axc_data_check(uint32_t slot_idx)
{
    uint32_t    data_id   = 0;
    uint32_t    src_addr1 = 0;
    uint32_t    src_addr2 = 0;
    uint32_t    real_addr = 0;
    uint32_t    data_len  = 0;
    uint16_t    bit_shift = 0;
    uint32_t    bf_per_slot = (LONGCP_BF_CNT + SHORTCP_BF_CNT*13);
    uint16_t    word_per_bf = 0;
    uint16_t    slot_type = 0;
    int32_t     bf_start  = 0;
    uint32_t    comp_val  = 0;
    uint32_t    recv_val  = 0;
    uint32_t    recv_addr = 0;


    for(data_id = 0; data_id < 6; data_id++)
    {
        comp_data_id_cnt++;
        bf_start = 0;

        if((slot_idx >= 0) && (slot_idx <= 6))
        {
            slot_type = slot_idx & 0x1;

            if(data_id < 2)
            {
                bit_shift   = 1;	// one BF, 2B
                word_per_bf = 1;
                src_addr1   = CPRI_NRCELL_RXDUMMY_SLOTS_COMPRESS_ADDR + slot_idx*(bf_per_slot << bit_shift) + data_id*CPRI_NR7DS2U_RX_DUMMY_COM_LEN; 
            }
            else
            {
                bit_shift   = 7;	// one BF, 64*2B
                word_per_bf = (128>>2);
                src_addr1   = CPRI_NRCELL_RXDUMMY_SLOTS_AXCDATA_ADDR + ((data_id - 2)*CPRI_NR7DS2U_RX_DUMMY_AXC_LEN) + slot_idx*(bf_per_slot << bit_shift); 
            }
            data_len = bf_per_slot << bit_shift;

        }
        else if(slot_idx == 7)		// compare S slot, odd slot
        {
            bf_start = (LONGCP_BF_CNT + SHORTCP_BF_CNT*9);
            slot_type = 1;
            if (data_id == 0)
            {
                bit_shift   = 1;
                word_per_bf = 1;
                src_addr1   = CPRI_NRCELL_RXDUMMY_SLOTS_COMPRESS_ADDR + slot_idx*(bf_per_slot << bit_shift);//Rx symbol 0~5地址
                src_addr2   = CPRI_NRCELL0_RX_SLOTS_COMPRESS_ADDR;//Rx symbol 6~13地址
            }
            else if(data_id == 1)
            {
            bit_shift   = 1;
            word_per_bf = 1;
            src_addr1   = CPRI_NRCELL_RXDUMMY_SLOTS_COMPRESS_ADDR + slot_idx*(bf_per_slot << bit_shift) + data_id*CPRI_NR7DS2U_RX_DUMMY_COM_LEN;
            src_addr2   = CPRI_NRCELL1_RX_SLOTS_COMPRESS_ADDR;
            }
            else if((data_id == 2)||(data_id == 3))
            {
                bit_shift   = 7;
                word_per_bf = (128 >> 2);
                src_addr1   = CPRI_NRCELL_RXDUMMY_SLOTS_AXCDATA_ADDR + ((data_id - 2)*CPRI_NR7DS2U_RX_DUMMY_AXC_LEN) + slot_idx*(bf_per_slot << bit_shift);
                src_addr2   = CPRI_NRCELL0_RX_SLOTS_COMPRESS_ADDR + 0x448 + (data_id - 2)*((SHORTCP_BF_CNT*4) << bit_shift);//0x448是cell0的压缩因子的长度
            }
            else
            {
                bit_shift   = 7;
                word_per_bf = (128 >> 2);
                src_addr1   = CPRI_NRCELL_RXDUMMY_SLOTS_AXCDATA_ADDR + ((data_id - 2)*CPRI_NR7DS2U_RX_DUMMY_AXC_LEN) + slot_idx*(bf_per_slot << bit_shift);
                src_addr2   = CPRI_NRCELL1_RX_SLOTS_COMPRESS_ADDR +  0x448 + (data_id - 4)*((SHORTCP_BF_CNT*4) << bit_shift);
            }

            data_len = bf_per_slot << bit_shift;
        }
        else if(slot_idx == 8)		// current slot is even slot, compare even slot, slot8
        {
            slot_type = 0;
            if (data_id == 0)
            {
                bit_shift   = 1;
                word_per_bf = 1;
                src_addr1   = SM1_NR_CELL0_EVEN_RX_DATA_ADDR;
            }
            else if(data_id == 1)
            {
                bit_shift   = 1;
                word_per_bf = 1;
                src_addr1   = SM4_NR_CELL1_EVEN_RX_DATA_ADDR;
            }
            else if((data_id == 2)||(data_id == 3))
            {
                bit_shift   = 7;
                word_per_bf = (128 >> 2);
                src_addr1   = SM1_NR_CELL0_EVEN_RX_DATA_ADDR + 0xF00 + ((data_id - 2)*(bf_per_slot << bit_shift));
            }
            else
            {
                bit_shift   = 7;
                word_per_bf = (128 >> 2);
                src_addr1   = SM4_NR_CELL1_EVEN_RX_DATA_ADDR + 0xF00 + ((data_id - 4)*(bf_per_slot << bit_shift));
            }

            data_len = bf_per_slot << bit_shift;
        }
        else if(slot_idx == 9)	// compare odd slot, slot9
        {
            slot_type = 1;
            if (data_id == 0)
            {
                bit_shift   = 1;
                word_per_bf = 1;
                src_addr1   = SM3_NR_CELL0_ODD_RX_DATA_ADDR;
            }
            else if(data_id == 1)
            {
                bit_shift   = 1;
                word_per_bf = 1;
                src_addr1   = SM5_NR_CELL1_ODD_RX_DATA_ADDR;
            }
            else if((data_id == 2)||(data_id == 3))
            {
                bit_shift   = 7;
                word_per_bf = (128 >> 2);
                src_addr1   = SM3_NR_CELL0_ODD_RX_DATA_ADDR +  0xF00 +((data_id - 2)*(bf_per_slot << bit_shift));
            }
            else 
            {
                bit_shift   = 7;
                word_per_bf = (128 >> 2);
                src_addr1   = SM5_NR_CELL1_ODD_RX_DATA_ADDR +  0xF00+ ((data_id - 4)*(bf_per_slot << bit_shift));
            }

            data_len = bf_per_slot << bit_shift;
        }

        if(data_id == 0)
        {
            for(int32_t bf_idx = 0; bf_idx <  (bf_per_slot >> 1); bf_idx++)
            {
                for(uint32_t word_idx = 0; word_idx < word_per_bf; word_idx++)
                {
                    comp_val = (slot_type << 28) | (data_id << 24) | ((bf_idx << 1) << 8) | (data_id);

                    debug_write((DBG_DDR_IDX_DRV_BASE + 1026), comp_word_sum_cnt); 
                    comp_word_sum_cnt++;
                    __ucps2_synch(0);

                    if((slot_idx == 7) && (bf_idx >= 686))
                    {
                        recv_addr = (uint32_t)((uint32_t*)src_addr2 + (bf_idx -(bf_start >> 1))*word_per_bf + word_idx);
                        if((bf_idx - (bf_start >> 1)) < 0)
                        {
                            bf_start_err_cnt++;
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1027), bf_start_err_cnt); 
                        }
                        real_addr = src_addr2;
                    }
                    else 
                    {
                        recv_addr = (uint32_t)((uint32_t*)src_addr1 + bf_idx*word_per_bf + word_idx);
                        real_addr = src_addr1;
                    }

                    recv_val = do_read_volatile(recv_addr); // *((uint32_t*)recv_addr);
                    __ucps2_synch(0);
                    if(recv_val != comp_val)
                    {
                        if(comp_word_err_cnt < 0x100)
                        {
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1028 + ((comp_word_err_cnt << 3)&0x7FF)), comp_val);
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1029 + ((comp_word_err_cnt << 3)&0x7FF)), recv_val);
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1030 + ((comp_word_err_cnt << 3)&0x7FF)), recv_addr);
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1031 + ((comp_word_err_cnt << 3)&0x7FF)), real_addr);
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1032 + ((comp_word_err_cnt << 3)&0x7FF)), (slot_idx + (data_id << 4) + (bf_idx << 8)));
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1033 + ((comp_word_err_cnt << 3)&0x7FF)), bf_start);
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1034 + ((comp_word_err_cnt << 3)&0x7FF)), bf_per_slot);
                        }
                        comp_word_err_cnt++;
                    }				
                }
            }
        }
        else if((data_id == 2) || (data_id == 3))
        {
            for(int32_t bf_idx = 0; bf_idx < bf_per_slot; bf_idx++)
            {
                for(uint32_t word_idx = 0; word_idx < word_per_bf; word_idx++)
                {
                    comp_val = (slot_type << 28) | (data_id << 24) | (((bf_idx << 5)+word_idx) << 8) | (word_idx);

                    debug_write((DBG_DDR_IDX_DRV_BASE + 1026), comp_word_sum_cnt); 
                    comp_word_sum_cnt++;
                    __ucps2_synch(0);

                    if((slot_idx == 7) && (bf_idx >= 1372))
                    {
                        recv_addr = (uint32_t)((uint32_t*)src_addr2 + (bf_idx - bf_start)*word_per_bf + word_idx);
                        real_addr = src_addr2;
                        if((bf_idx - bf_start) < 0)
                        {
                            bf_start_err_cnt++;
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1027), bf_start_err_cnt); 
                        }
                    }
                    else 
                    {
                        recv_addr = (uint32_t)((uint32_t*)src_addr1 + bf_idx*word_per_bf + word_idx);
                        real_addr = src_addr1;
                    }

                    recv_val = do_read_volatile(recv_addr);
                    __ucps2_synch(0);
                    if(recv_val != comp_val)
                    {
                        if(comp_word_err_cnt < 0x100)
                        {
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1028 + ((comp_word_err_cnt << 3)&0x7FF)), comp_val);
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1029 + ((comp_word_err_cnt << 3)&0x7FF)), recv_val);
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1030 + ((comp_word_err_cnt << 3)&0x7FF)), recv_addr);	
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1031 + ((comp_word_err_cnt << 3)&0x7FF)), real_addr);
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1032 + ((comp_word_err_cnt << 3)&0x7FF)), (slot_idx + (data_id << 4) + (bf_idx << 8)));	
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1033 + ((comp_word_err_cnt << 3)&0x7FF)), bf_start);
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1034 + ((comp_word_err_cnt << 3)&0x7FF)), bf_per_slot);
                        }
                        comp_word_err_cnt++;
                    }
                }
            }
        }
 
        debug_write((DBG_DDR_IDX_DRV_BASE + 1024), comp_data_id_cnt);
        debug_write((DBG_DDR_IDX_DRV_BASE + 1025), comp_word_err_cnt);
    }
}


/**********************************************************************************************************************
*  function: fh_data_init
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
extern  uint32_t  gCpriTestMode;

int32_t fh_data_init(void)
{
    gCpriTestMode = CPRI_TEST_MODE;
    debug_write((DBG_DDR_IDX_DRV_BASE + 192), gCpriTestMode);

    axc_data_init();
    return 0;
}


/**********************************************************************************************************************
*  function: fh_data_check
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
extern stMtimerIntStat  gMtimerIntCnt[SCS_MAX_NUM];
void fh_data_check(uint32_t times)
{
    uint32_t         slot_idx;
    stMtimerIntStat  *pMtimerInt;
    
     
    pMtimerInt = &gMtimerIntCnt[MTIMER_CPRI_ID];
    if(pMtimerInt->csuEnCnt >= 4)
    {
        comp_word_sum_cnt = 0;

        for(slot_idx = 0; slot_idx < (CPRI_CASE70_SLOT_NUM >> 1); slot_idx++)
        {
            axc_data_check(slot_idx);
        }
    }
}


/**********************************************************************************************************************
*  function: fh_test_case
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
void fh_test_case()
{
	UCP_API_CPRI_CSU_START();
}


/**********************************************************************************************************************
*  function: fh_csu_test_init
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
int32_t fh_csu_test_init(void)
{
    Phy_Timer_Csu_Config_Nr();
    return 0;
}

 
/**********************************************************************************************************************
*  function: fh_drv_init
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
int32_t fh_drv_init(void)
{
    stFrontHaulDrvPara fhDrvPara;

    memset_ucp(&fhDrvPara, 0, sizeof(stFrontHaulDrvPara));

    fhDrvPara.protocolSel = PROTOCOL_CPRI;
    fhDrvPara.rateOption  = CPRI_OPTION_10;
    fhDrvPara.mapOption   = OTIC_MAP_FIGURE16;
    fronthaul_drv_cfg(&fhDrvPara);

    return 0;
}


