/**********************************************************************************************************************
*  Copyright (C), 2022-2026, SMARTLOGIC TECHNOLOGY LTD.
*  File Name: cpri_test_case80.c
*  Create Date: 24/2/20
*  Description: 
*  Change History: 
*      <author>      <time>      <version>   <desc>
*    1. cheng.wan    24/02/20      2.0       refactor
**********************************************************************************************************************/
/* -------------------------------------------------- <INC FILE> --------------------------------------------------- */
#include "typedef.h"
#include "ucp_utility.h"
#include "ucp_printf.h"
#include "cpri_timer.h"
#include "cpri_driver.h"
#include "rfm1_drv.h"
#include "cpri_test.h"
#include "nr_mem_def.h"
#include "cpri_csu.h"
#include "cpri_test_case.h"


//* --------------------------------------------------- <DEF&DEC> --------------------------------------------------- */




/* -------------------------------------------------- <FUNC DEF> --------------------------------------------------- */
/**********************************************************************************************************************
*  function: axc_data_init
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
void axc_data_init()
{
    uint32_t    data_id   = 0;
    uint32_t    slot_type = 0;
    uint32_t    symb_type = 0;
    uint32_t    symb_idx  = 0;
    uint32_t    bf_idx    = 0;
    uint32_t    word_idx  = 0;

    uint32_t    *p_addr  = 0;
    uint32_t    dst_addr = 0;
    uint32_t    data_len = 0;
    uint16_t    byte_per_bf = 0;
    uint32_t    bf_per_slot = (LONGCP_BF_CNT + SHORTCP_BF_CNT*13)*2;
    uint32_t    bf_per_f7   = (LONGCP_BF_CNT + SHORTCP_BF_CNT*6)*2;
    uint32_t    bf_per_b7   = (SHORTCP_BF_CNT*7)*2;
    uint32_t    bf_per_symb = 0;
    uint32_t    word_in_slot= 0;
    uint32_t    value = 0;

    // valid data
    // compress factor
    for(slot_type = 0; slot_type <= 1; slot_type++)
    {
        byte_per_bf = 2;

        if(slot_type == 0)
        {		
            dst_addr = SM1_NR_CELL0_EVEN_COMP_FACTOR_ADDR;
        }
        else
        {			
            dst_addr = SM1_NR_CELL0_ODD_TX_COMP_FACTOR_ADDR;
        }

        p_addr = ((uint32_t*)dst_addr);
        for(bf_idx = 0; bf_idx < (bf_per_slot >> 1); bf_idx++)
        {
            value = (slot_type << 28) | (0 << 24) | ((bf_idx << 1) << 8) | (0);
            do_write(((uint32_t)p_addr), value);			
            p_addr++;
        }
    }

    // IQ data	
    for(data_id = 1; data_id < 5; data_id++)
    {
        byte_per_bf = 64;
        for(slot_type = 0; slot_type <= 1; slot_type++)
        {
            word_in_slot = 0;
            for(symb_type = 0; symb_type <= 1; symb_type++)
            {
                if((slot_type == 0) && (symb_type == 0))	    //even slot, symbol0~6
                {
                    data_len = byte_per_bf * bf_per_f7;
                    if(data_id < 3)
                    {
                        dst_addr = SM0_NR_CELL0_EVEN_F7_TX_DATA_ADDR + (data_id - 1)*data_len;
                    }
                    else
                    {
                        dst_addr = CSU_TX_DUMMYBUFFER_ADDR;
                    }
                }
                else if((slot_type == 0) && (symb_type == 1))	//even slot, symbol7~13
                {
                    data_len = byte_per_bf * bf_per_b7;
                    if(data_id < 3)
                    {
                        dst_addr = SM4_NR_CELL0_EVEN_B7_TX_DATA_ADDR + (data_id - 1)*data_len;
                    }
                    else
                    {
                        dst_addr = CSU_TX_DUMMYBUFFER_ADDR;
                    }
                }
                else if((slot_type == 1) && (symb_type == 0))	//odd slot, symbol0~6
                {
                    data_len = byte_per_bf * bf_per_f7;
                    if(data_id < 3)
                    {
                        dst_addr = SM1_NR_CELL0_ODD_F7_TX_DATA_ADDR + (data_id - 1)*data_len;
                    }
                    else
                    {
                        dst_addr = CSU_TX_DUMMYBUFFER_ADDR;
                    }
                }
                else if((slot_type == 1) && (symb_type == 1))	//odd slot, symbol7~13
                {
                    data_len = byte_per_bf * bf_per_b7;
                    if(data_id < 3)
                    {
                        dst_addr = SM4_NR_CELL0_ODD_B7_TX_DATA_ADDR + (data_id - 1)*data_len;
                    }
                    else
                    {
                        dst_addr = CSU_TX_DUMMYBUFFER_ADDR;
                    }
                }

                p_addr = ((uint32_t*)dst_addr);
                for(symb_idx = 0; symb_idx < 7; symb_idx++)
                {
                    if((symb_idx == 0) && (symb_type == 0))
                    {
                        bf_per_symb = (LONGCP_BF_CNT*2);
                    }
                    else
                    {
                        bf_per_symb = (SHORTCP_BF_CNT*2);
                    }

                    for(bf_idx = 0; bf_idx < bf_per_symb; bf_idx++)
                    {
                        for(word_idx = 0; word_idx < (byte_per_bf >> 2); word_idx++)
                        {
                            value = (slot_type << 28) | (data_id << 24) | ((word_in_slot++) << 8) | (word_idx);
                            do_write(((uint32_t)p_addr), value);
                            p_addr++;
                        }
                    }
                    data_len = bf_per_symb*byte_per_bf;
                }
            }
        }
    }
}


/**********************************************************************************************************************
*  function: axc_data_check
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
uint32_t comp_word_sum_cnt = 0;
uint32_t comp_word_err_cnt = 0;
uint32_t comp_data_id_cnt  = 0;
uint32_t bf_start_err_cnt  = 0;
void axc_data_check(uint32_t slot_idx)
{
    uint32_t    src_addr  = 0;
    uint32_t    real_addr = 0;
    uint32_t    bf_per_slot = (LONGCP_BF_CNT+SHORTCP_BF_CNT*13)*2;
    uint32_t	word_per_bf = 0;
    uint32_t    slot_type = 0;
    uint32_t    data_id   = 0;
    int32_t     bf_start  = 0;
    uint32_t    comp_val  = 0;
    uint32_t    recv_val  = 0;
    uint32_t    recv_addr = 0;


 
    for (data_id = 0; data_id < 6; data_id++)
    {
        comp_data_id_cnt++;
        bf_start = 0;


        if((slot_idx & 0x1) == 1) //奇时隙
        {
            slot_type = 1;
            if(data_id == 0)                            //NR:压缩因子和AGC:2B
            {
                word_per_bf = 1;
                src_addr    = SM3_NR_CELL0_ODD_RX_DATA_ADDR;
            }		
            else if((1 <= data_id) && (data_id <= 2))   //NR:2条天线数据交织:64B
            {
                word_per_bf = (64 >> 2);
                src_addr    = (SM3_NR_CELL0_ODD_RX_DATA_ADDR + (2*bf_per_slot) + ((data_id - 1)*64*bf_per_slot));
            }
            else if((3 <= data_id) && (data_id <= 4))   //NR:2条天线数据交织:64B
            {
                word_per_bf = (64 >> 2);
                src_addr    = CSU_RX_DUMMYBUFFER_ADDR;
            }
            else                                        //NR:AGC:2B
            {
                word_per_bf = 1;
                src_addr    = SM1_NR_CELL0_ODD_RX_AGC_ADDR;
            }
        }
        else//偶时隙
        {
            slot_type = 0;
            if(data_id == 0)
            {
                word_per_bf = 1;
                src_addr    = SM1_NR_CELL0_EVEN_RX_DATA_ADDR;
            }		
            else if((data_id >= 1) && (data_id <= 2))
            {
                word_per_bf = (64 >> 2);
                src_addr    = (SM1_NR_CELL0_EVEN_RX_DATA_ADDR + (2*bf_per_slot)+ ((data_id - 1)*64*bf_per_slot));
            }
            else if((data_id >= 3) && (data_id <= 4))
            {
                word_per_bf = (64 >> 2);
                src_addr    = CSU_RX_DUMMYBUFFER_ADDR;
            }
            else
            {
                word_per_bf = 1;
                src_addr    = SM1_NR_CELL0_EVNE_RX_AGC_ADDR;
            }
        }

        if (data_id == 0)	// compress factor:NR
        {
            for (int32_t bf_idx = 0; bf_idx < (bf_per_slot >> 1); bf_idx++)
            {
                for (uint32_t word_idx = 0; word_idx < word_per_bf; word_idx++)
                {
                    comp_val = (slot_type << 28) | (data_id << 24) | ((bf_idx << 1) << 8) | (word_idx);
                    debug_write((DBG_DDR_IDX_DRV_BASE + 1026), comp_word_sum_cnt); 
                    comp_word_sum_cnt++;

                    __ucps2_synch(0);

                    recv_addr = (uint32_t)((uint32_t*)src_addr + bf_idx*word_per_bf + word_idx);
                    real_addr = src_addr;
                    recv_val = do_read_volatile(recv_addr); 
                    __ucps2_synch(0);
                    if (recv_val != comp_val)
                    {
                        if (comp_word_err_cnt < 0x100)
                        {
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1028 + ((comp_word_err_cnt << 3)&0x7FF)), comp_val);	
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1029 + ((comp_word_err_cnt << 3)&0x7FF)), recv_val);
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1030 + ((comp_word_err_cnt << 3)&0x7FF)), recv_addr);
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1031 + ((comp_word_err_cnt << 3)&0x7FF)), real_addr);	
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1032 + ((comp_word_err_cnt << 3)&0x7FF)), (slot_idx + (data_id << 4) + (bf_idx << 8)));
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1033 + ((comp_word_err_cnt << 3)&0x7FF)), bf_start);
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1034 + ((comp_word_err_cnt << 3)&0x7FF)), bf_per_slot);
                        }
                        comp_word_err_cnt++;
                    }
                }
            }
        }
        else if((data_id >= 1) && (data_id <= 2))		// 天线0,1
        {
            for(int32_t bf_idx = 0; bf_idx < bf_per_slot; bf_idx++)
            {
                for(uint32_t word_idx = 0; word_idx < word_per_bf; word_idx++)
                {
                    comp_val = (slot_type << 28) | (data_id << 24) | (((bf_idx << 4)+word_idx) << 8) | (word_idx);

                    debug_write((DBG_DDR_IDX_DRV_BASE + 1026), comp_word_sum_cnt); 
                    comp_word_sum_cnt++;					
                    __ucps2_synch(0);

                    recv_addr = (uint32_t)((uint32_t*)src_addr + bf_idx*word_per_bf + word_idx);
                    real_addr = src_addr;
                    recv_val  = do_read_volatile(recv_addr); 
                    __ucps2_synch(0);

                    if(recv_val != comp_val)
                    {
                        if(comp_word_err_cnt < 0x100)
                        {
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1028 + ((comp_word_err_cnt << 3) & 0x7FF)), comp_val);
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1029 + ((comp_word_err_cnt << 3) & 0x7FF)), recv_val);
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1030 + ((comp_word_err_cnt << 3) & 0x7FF)), recv_addr);
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1031 + ((comp_word_err_cnt << 3) & 0x7FF)), real_addr);
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1032 + ((comp_word_err_cnt << 3) & 0x7FF)), (slot_idx + (data_id << 4) + (bf_idx << 8)));	
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1033 + ((comp_word_err_cnt << 3) & 0x7FF)), bf_start);
                            debug_write((DBG_DDR_IDX_DRV_BASE + 1034 + ((comp_word_err_cnt << 3) & 0x7FF)), bf_per_slot);
                        }
                        comp_word_err_cnt++;
                    }
                }
            }
        }

        debug_write((DBG_DDR_IDX_DRV_BASE + 1024), comp_data_id_cnt);
        debug_write((DBG_DDR_IDX_DRV_BASE + 1025), comp_word_err_cnt);
    }
}


/**********************************************************************************************************************
*  function: fh_data_init
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
extern  uint32_t  gCpriTestMode;

int32_t fh_data_init(void)
{
    gCpriTestMode = CPRI_TEST_MODE;
    debug_write((DBG_DDR_IDX_DRV_BASE + 192), gCpriTestMode);

    axc_data_init();
    return 0;
}


/**********************************************************************************************************************
*  function: fh_data_check
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
extern stMtimerIntStat  gMtimerIntCnt[SCS_MAX_NUM];
void fh_data_check(uint32_t times)
{
    uint32_t         slot_idx;
    stMtimerIntStat  *pMtimerInt;
    
     
    pMtimerInt = &gMtimerIntCnt[MTIMER_CPRI_ID];
    if(pMtimerInt->csuEnCnt >= 4)
    {
        comp_word_sum_cnt = 0;

        for(slot_idx = 0; slot_idx < (CPRI_CASE80_SLOT_NUM >> 1); slot_idx++)
        {
            axc_data_check(slot_idx);
        }
    }
}


/**********************************************************************************************************************
*  function: fh_test_case
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
void fh_test_case()
{
	UCP_API_CPRI_CSU_START();
}


/**********************************************************************************************************************
*  function: fh_csu_test_init
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
int32_t fh_csu_test_init(void)
{
    Config_Cpri_Csu_Nr(0);
    return 0;
}


/**********************************************************************************************************************
*  function: fh_drv_init
*  description: 
*  input : none
*  output: none
*  return: none
*  others: none
**********************************************************************************************************************/
int32_t fh_drv_init(void)
{
    stFrontHaulDrvPara fhDrvPara;

    memset_ucp(&fhDrvPara, 0, sizeof(stFrontHaulDrvPara));

    fhDrvPara.protocolSel = PROTOCOL_CPRI;
    fhDrvPara.rateOption  = CPRI_OPTION_8;
    fhDrvPara.mapOption   = OTIC_MAP_FIGURE12;
    fronthaul_drv_cfg(&fhDrvPara);

    return 0;
}


